@@ -1,10 +1,10 @@
-/* $Id: board.cc,v 1.82 2002/06/28 01:33:05 bergo Exp $ */
+/* $Id: board.cc,v 1.92 2003/02/05 19:57:01 bergo Exp $ */
 
 /*
 
     eboard - chess client
     http://eboard.sourceforge.net
-    Copyright (C) 2000-2002 Felipe Paulo Guazzi Bergo
+    Copyright (C) 2000-2003 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -23,10 +23,11 @@
 
 */
 
-#include <iostream.h>
+#include <iostream>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include "eboard.h"
 #include "board.h"
 #include "chess.h"
 #include "global.h"
@@ -41,10 +42,8 @@ Board * Board::PopupOwner = 0;
 
 Board::Board() : WidgetProxy() {
   int i;
-  GtkWidget *tl2;
 
   global.debug("Board","Board");
-  global.create("Board");
 
   global.addPieceClient(this);
 
@@ -80,7 +79,7 @@ Board::Board() : WidgetProxy() {
   morey=0;
 
   if ((!f1)||(!f2)||(!f3)) {
-    cerr << "<Board::Board> ** failed to load one or more board fonts - install X11 75 dpi and 100 dpi fonts, restart X, and try again.\n";
+    cerr << _("<Board::Board> ** failed to load one or more board fonts - install X11 75 dpi and 100 dpi fonts, restart X, and try again.\n");
     exit(2);
   }
 
@@ -95,12 +94,19 @@ Board::Board() : WidgetProxy() {
   gtk_widget_set_events(yidget,GDK_EXPOSURE_MASK|GDK_BUTTON_PRESS_MASK|
 			GDK_BUTTON_RELEASE_MASK|GDK_BUTTON1_MOTION_MASK|
 			GDK_POINTER_MOTION_HINT_MASK);
-  buffer=gdk_pixmap_new(MainWindow::RefWindow,SCRAP_WIDTH,SCRAP_WIDTH,-1);
-  clkbar=gdk_pixmap_new(MainWindow::RefWindow,250,SCRAP_WIDTH,-1);
+
+  sqside=59;
+  sqw = 2*borx + 8*sqside + 1;
+  sqh = 2*bory + morey + 8*sqside + 1;
+
+  buffer=gdk_pixmap_new(MainWindow::RefWindow,sqw,sqh,-1);
+  clkbar=gdk_pixmap_new(MainWindow::RefWindow,250,sqh,-1);
+
   clkgc=gdk_gc_new(clkbar);
   wgc=0;
-  scrap=(rgbptr)g_malloc(SCRAP_WIDTH*SCRAP_WIDTH*3);
-  memset(scrap,0,SCRAP_WIDTH*SCRAP_WIDTH*3);
+  
+  scrap=(rgbptr)g_malloc(sqw*sqh*3);
+  memset(scrap,0,sqw*sqh*3);
 
   gtk_signal_connect (GTK_OBJECT (yidget), "expose_event",
                       (GtkSignalFunc) board_expose_event, (gpointer) this);
@@ -116,7 +122,7 @@ Board::Board() : WidgetProxy() {
                       (GtkSignalFunc) board_motion_event,
 		      (gpointer) this);
 
-  gtk_widget_show(widget);
+  gshow(widget);
 }
 
 bool Board::hasGame() {
@@ -124,7 +130,7 @@ bool Board::hasGame() {
 }
 
 Board::~Board() {
-  int i;
+  unsigned int i;
   global.debug("Board","~Board");
   global.removeBoard(this);
   global.removePieceClient(this);
@@ -144,7 +150,6 @@ Board::~Board() {
   gdk_gc_destroy(clkgc);
   gdk_pixmap_unref(clkbar);
   gdk_pixmap_unref(buffer);
-  global.destroy("Board");
 }
 
 void Board::reloadFonts() {
@@ -157,7 +162,7 @@ void Board::reloadFonts() {
   f3=global.loadFont(EF_InfoFont);
 
   if ((!f1)||(!f2)||(!f3)) {
-    cerr << "<Board::Board> ** failed to load one or more board fonts\n";
+    cerr << _("<Board::Board> ** failed to load one or more board fonts\n");
     exit(2);
   }
   clock_ch=1;
@@ -174,7 +179,7 @@ void Board::reloadFonts() {
 */
 
 void Board::reset() {
-  int i;
+  unsigned int i;
   global.debug("Board","reset");
   while(gotAnimationLoop)
     global.WrappedMainIteration();
@@ -319,14 +324,14 @@ void RootBoard::clockString(int val,char *z) {
 
   if (H) {
     if (neg)
-      sprintf(z,"-%d:%0.2d:%0.2d",H,M,S);
+      sprintf(z,"-%d:%.2d:%.2d",H,M,S);
     else
-      sprintf(z,"%d:%0.2d:%0.2d",H,M,S);
+      sprintf(z,"%d:%.2d:%.2d",H,M,S);
   } else {
     if (neg)
-      sprintf(z,"-%0.2d:%0.2d",M,S);
+      sprintf(z,"-%.2d:%.2d",M,S);
     else
-      sprintf(z,"%0.2d:%0.2d",M,S);
+      sprintf(z,"%.2d:%.2d",M,S);
   }
 }
 
@@ -349,7 +354,7 @@ void Board::renderClock() {
 
   sq=sqside;
 
-  C.prepare(clkbar,gc,0,0,250,SCRAP_WIDTH);
+  C.prepare(clkbar,gc,0,0,250,sqh);
   C.setColor(0);
   C.drawRect(0,0,C.W,C.H,true);
   C.H = Height();
@@ -379,7 +384,7 @@ void Board::renderClock() {
     C.setColor(0);
   }
 
-  C.drawString(j,i,0,"Black");
+  C.drawString(j,i,0,_("Black"));
   clockString(clock.getValue(1),z);
   C.drawString(j,i+th,1,z);
 
@@ -391,7 +396,7 @@ void Board::renderClock() {
     C.drawRect(15, i - (ch+2), 200-30, bh+4, true);
     C.setColor(0);
   }
-  C.drawString(j,i,0,"White");
+  C.drawString(j,i,0,_("White"));
   clockString(clock.getValue(0),z);
   C.drawString(j,i+th,1,z);
 
@@ -565,6 +570,14 @@ void Board::renderClock() {
   clock_ch=0;
 }
 
+ChessClock * Board::getClock() {
+  return(&clock);
+}
+
+int Board::getClockValue(bool white) {
+  return(clock.getValue(white?0:1));
+}
+
 void Board::setClock(int wsecs,int bsecs,int actv,int cdown) {
   global.debug("Board","setClock");
   clock_ch=1;
@@ -626,8 +639,8 @@ void Board::setInfo(int idx,string &s) {
   queueRepaint();
 }
 
-void Board::update(bool sndflag=false) {
-  int i,j,k,rate;
+void Board::update(bool sndflag) {
+  unsigned int i,rate;
   AnimatedPiece *ap;  
   Position rpv;
   bool ef;
@@ -711,9 +724,9 @@ void Board::sendMove() {
       if (!position.isMoveLegalCartesian(ax[0],ay[0],ax[1],ay[1],
 					 mygame->MyColor,mygame->Variant)) {
 	char z[128];
-	sprintf(z,"Illegal Move %c%d%c%d (Legality Checking On)",
+	sprintf(z,_("Illegal Move %c%d%c%d (Legality Checking On)"),
 		'a'+ax[0],ay[0]+1,'a'+ax[1],ay[1]+1);
-	global.status->setText(z);
+	global.status->setText(z,15);
 	goto sendmove_cleanup;	
       }      
     position.moveCartesian(ax[0],ay[0],ax[1],ay[1],REGULAR,true);
@@ -880,9 +893,9 @@ void Board::drop(piece p) {
 				mygame->MyColor,mygame->Variant))
 	{
 	  char z[128];
-	  sprintf(z,"Illegal Drop on %c%d (Legality Checking On)",
+	  sprintf(z,_("Illegal Drop on %c%d (Legality Checking On)"),
 		  'a'+dropsq[0],dropsq[1]+1);
-	  global.status->setText(z);
+	  global.status->setText(z,15);
 	  goto drop_cleanup;
 	}
     position.moveDrop(p, dropsq[0], dropsq[1]);
@@ -900,7 +913,7 @@ void Board::popupDropMenu(int col,int row,int sx,int sy,guint32 etime) {
   GtkWidget *popmenu;
   GtkWidget *mi[7];
   Position pos;
-  int i,j;
+  int i;
   char z[64];
 
   // sanity checks
@@ -917,32 +930,32 @@ void Board::popupDropMenu(int col,int row,int sx,int sy,guint32 etime) {
 
   pos=mygame->getLastPosition();
   
-  mi[0]=gtk_menu_item_new_with_label("Drop Piece:");
+  mi[0]=gtk_menu_item_new_with_label(_("Drop Piece:"));
   gtk_widget_set_sensitive(mi[0],FALSE);
   mi[1]=gtk_menu_item_new();
 
-  sprintf(z,"Pawn    %d",i=pos.getStockCount(PAWN|mygame->MyColor));
+  sprintf(z,_("Pawn    %d"),i=pos.getStockCount(PAWN|mygame->MyColor));
   mi[2]=gtk_menu_item_new_with_label(z);
   if (!i) gtk_widget_set_sensitive(mi[2],FALSE);
 
-  sprintf(z,"Rook    %d",i=pos.getStockCount(ROOK|mygame->MyColor));
+  sprintf(z,_("Rook    %d"),i=pos.getStockCount(ROOK|mygame->MyColor));
   mi[3]=gtk_menu_item_new_with_label(z);
   if (!i) gtk_widget_set_sensitive(mi[3],FALSE);
 
-  sprintf(z,"Knight  %d",i=pos.getStockCount(KNIGHT|mygame->MyColor));
+  sprintf(z,_("Knight  %d"),i=pos.getStockCount(KNIGHT|mygame->MyColor));
   mi[4]=gtk_menu_item_new_with_label(z);
   if (!i) gtk_widget_set_sensitive(mi[4],FALSE);
 
-  sprintf(z,"Bishop  %d",i=pos.getStockCount(BISHOP|mygame->MyColor));
+  sprintf(z,_("Bishop  %d"),i=pos.getStockCount(BISHOP|mygame->MyColor));
   mi[5]=gtk_menu_item_new_with_label(z);
   if (!i) gtk_widget_set_sensitive(mi[5],FALSE);
 
-  sprintf(z,"Queen   %d",i=pos.getStockCount(QUEEN|mygame->MyColor));
+  sprintf(z,_("Queen   %d"),i=pos.getStockCount(QUEEN|mygame->MyColor));
   mi[6]=gtk_menu_item_new_with_label(z);
   if (!i) gtk_widget_set_sensitive(mi[6],FALSE);
 
   for(i=0;i<7;i++) {
-    gtk_widget_show(mi[i]);
+    gshow(mi[i]);
     gtk_menu_append(GTK_MENU(popmenu),mi[i]);
   }
 
@@ -969,7 +982,7 @@ void Board::popupProtocolMenu(int x,int y, guint32 etime) {
   GtkWidget *mi, *hs, *popmenu;
   vector<string *> *v;
   char z[64];
-  int i;
+  unsigned int i;
   
   if (!global.protocol) return;
   if (!mygame) return;
@@ -984,7 +997,7 @@ void Board::popupProtocolMenu(int x,int y, guint32 etime) {
   for(i=0;i<v->size();i++) {
     sprintf(z,"%s: %s",mygame->PlayerName[0], ((*v)[i])->c_str() );
     mi=gtk_menu_item_new_with_label(z);
-    gtk_widget_show(mi);
+    gshow(mi);
     gtk_menu_append(GTK_MENU(popmenu),mi);
 
     gtk_signal_connect(GTK_OBJECT(mi),"activate",
@@ -995,14 +1008,13 @@ void Board::popupProtocolMenu(int x,int y, guint32 etime) {
   mi=gtk_menu_item_new();
   hs=gtk_hseparator_new();
   gtk_container_add(GTK_CONTAINER(mi),hs);
-  gtk_widget_show(hs);
-  gtk_widget_show(mi);
+  Gtk::show(hs,mi,0);
   gtk_menu_append(GTK_MENU(popmenu),mi);
 
   for(i=0;i<v->size();i++) {
     sprintf(z,"%s: %s",mygame->PlayerName[1], ((*v)[i])->c_str() );
     mi=gtk_menu_item_new_with_label(z);
-    gtk_widget_show(mi);
+    gshow(mi);
     gtk_menu_append(GTK_MENU(popmenu),mi);
 
     gtk_signal_connect(GTK_OBJECT(mi),"activate",
@@ -1014,16 +1026,15 @@ void Board::popupProtocolMenu(int x,int y, guint32 etime) {
     mi=gtk_menu_item_new();
     hs=gtk_hseparator_new();
     gtk_container_add(GTK_CONTAINER(mi),hs);
-    gtk_widget_show(hs);
-    gtk_widget_show(mi);
+    Gtk::show(hs,mi,0);
     gtk_menu_append(GTK_MENU(popmenu),mi);
 
     v=global.protocol->getGameActions();
 
     for(i=0;i<v->size();i++) {
-      sprintf(z,"Game #%d: %s",mygame->GameNumber, ((*v)[i])->c_str() );
+      sprintf(z,_("Game #%d: %s"),mygame->GameNumber, ((*v)[i])->c_str() );
       mi=gtk_menu_item_new_with_label(z);
-      gtk_widget_show(mi);
+      gshow(mi);
       gtk_menu_append(GTK_MENU(popmenu),mi);
       
       gtk_signal_connect(GTK_OBJECT(mi),"activate",
@@ -1155,7 +1166,7 @@ void Board::composeVecBoard(GdkGC *gc) {
 
   i=Width();
   gdk_rgb_gc_set_foreground(gc,0);
-  gdk_draw_rectangle(buffer,gc,TRUE,0,i,i,SCRAP_WIDTH-Height());
+  gdk_draw_rectangle(buffer,gc,TRUE,0,i,i,sqh-Height());
 
   currently_rendered=*(joker);
 }
@@ -1202,7 +1213,8 @@ gboolean board_expose_event(GtkWidget *widget,GdkEventExpose *ee,
                             gpointer data) {
   Board *me;
   GdkGC *gc;
-  int i,j,k,w,h,ww,wh,sq,pw,fw,fh;
+  unsigned int i;
+  int w,h,sq,pw,fw,fh;
 
   me=(Board *)data;
   w=ee->area.width;
@@ -1220,14 +1232,14 @@ gboolean board_expose_event(GtkWidget *widget,GdkEventExpose *ee,
     gdk_draw_rectangle(widget->window,widget->style->black_gc,TRUE,
 		       me->Width()+250,0,fw-me->Width()-250,fh);
 
-  if (fh>SCRAP_WIDTH) {
+  if (fh > me->sqh) {
     gdk_draw_rectangle(widget->window,widget->style->black_gc,TRUE,
-		       0,SCRAP_WIDTH,fw,fh-SCRAP_WIDTH);
-    fh=SCRAP_WIDTH;
+		       0,me->sqh,fw,fh-me->sqh);
+    fh=me->sqh;
   }
 
-  if (fw>SCRAP_WIDTH)
-    fw=SCRAP_WIDTH;
+  if (fw > me->sqw)
+    fw=me->sqw;
   
   pw=fw;
   if (fw> me->Width() ) pw=me->Width();
@@ -1268,7 +1280,7 @@ gboolean board_expose_event(GtkWidget *widget,GdkEventExpose *ee,
 	}
       
       if (me->sp)
-	for(i=0;i<me->sp;i++)
+	for(i=0;i < (unsigned int) (me->sp);i++)
 	  me->outlineRectangle(gc,me->ax[i],me->ay[i],me->sel_color,4);
       
       if (me->premoveq) {
@@ -1323,6 +1335,7 @@ gboolean board_configure_event(GtkWidget *widget,GdkEventConfigure *ce,
   Board *me;
   int optimal_square,sq;
   int ww,wh,i,j;
+  int oldsqw, oldsqh;
   Position *joker;
 
   me=(Board *)data;
@@ -1363,7 +1376,12 @@ gboolean board_configure_event(GtkWidget *widget,GdkEventConfigure *ce,
     me->currently_rendered.invalidate();
     me->clock_ch=1;
   }
+
+  oldsqw=me->sqw;
+  oldsqh=me->sqh;
   me->sqside=sq;
+  me->sqw = 2*me->borx + 8 * me->sqside + 1;
+  me->sqh = 2*me->bory + me->morey + 8 * me->sqside + 1;
 
   if (global.UseVectorPieces)
     return 1;
@@ -1371,7 +1389,21 @@ gboolean board_configure_event(GtkWidget *widget,GdkEventConfigure *ce,
   if ((me->currently_rendered == me->position)&&(!me->update_queue))
     return 1;
 
-  memset(me->scrap,0,SCRAP_WIDTH*SCRAP_WIDTH*3);
+  if (oldsqw!= me->sqw || oldsqh != me->sqh) {
+    gdk_pixmap_unref(me->clkbar);
+    gdk_pixmap_unref(me->buffer);
+    if (me->clkgc) { gdk_gc_destroy(me->clkgc); me->clkgc=0; }
+    if (me->scrap) { g_free(me->scrap); me->scrap=0; }
+
+    me->buffer=gdk_pixmap_new(MainWindow::RefWindow,
+			      me->sqw,me->sqh,-1);
+    me->clkbar=gdk_pixmap_new(MainWindow::RefWindow,
+			      250,me->sqh,-1);
+    me->clkgc=gdk_gc_new(me->clkbar);
+    me->scrap=(rgbptr)g_malloc(me->sqw * me->sqh * 3);
+  }
+
+  memset(me->scrap,0,me->sqw * me->sqh * 3);
 
   if (me->gotAnimationLoop)
     joker=&(me->fake);
@@ -1387,14 +1419,14 @@ gboolean board_configure_event(GtkWidget *widget,GdkEventConfigure *ce,
       for(j=0;j<8;j++)
 	me->cur->drawPieceAndSquare(joker->getPiece(7-i,j),
 				    me->scrap,me->borx+i*sq,
-				    me->morey+me->bory+j*sq,SCRAP_WIDTH,
+				    me->morey+me->bory+j*sq,me->sqw,
 				    (i+j)%2);
   else
     for(i=0;i<8;i++)
       for(j=0;j<8;j++)
 	me->cur->drawPieceAndSquare(joker->getPiece(i,7-j),
 				    me->scrap,me->borx+i*sq,
-				    me->morey+me->bory+j*sq,SCRAP_WIDTH,
+				    me->morey+me->bory+j*sq,me->sqw,
 				    (i+j)%2);
 
   me->cur->endQueueing();
@@ -1402,8 +1434,8 @@ gboolean board_configure_event(GtkWidget *widget,GdkEventConfigure *ce,
 
   // copy scrap to pixmap buffer
   gdk_draw_rgb_image(me->buffer,widget->style->black_gc,0,0,
-		     SCRAP_WIDTH,SCRAP_WIDTH,
-		     GDK_RGB_DITHER_NORMAL,me->scrap,SCRAP_WIDTH*3);
+		     me->sqw, me->sqh,
+		     GDK_RGB_DITHER_NORMAL,me->scrap,me->sqw*3);
 
   if (global.ShowCoordinates) {
     if (!me->wgc)
@@ -1415,6 +1447,7 @@ gboolean board_configure_event(GtkWidget *widget,GdkEventConfigure *ce,
   return 1;
 }
 
+/*
 static bool not_disjoint_intervals(int a, int b, int c, int d) {
   return( (c<=b) && (d>=a) );
 }
@@ -1424,6 +1457,7 @@ static bool rects_overlap(int x1,int y1,int w1,int h1,
   return( not_disjoint_intervals(x1,x1+w1, x2, x2+w2) &&
 	  not_disjoint_intervals(y1,y1+h1, y2, y2+h2) );
 }
+*/
 
 static GdkPixmap *GCPbuffer=0;
 static int gcpw=0,gcph=0;
@@ -1511,8 +1545,8 @@ gboolean board_motion_event(GtkWidget *widget,
   gdk_rgb_gc_set_foreground(gcpgc,0);
 
   sw=ww; sh=wh;
-  if (sw>SCRAP_WIDTH) sw=SCRAP_WIDTH;
-  if (sh>SCRAP_WIDTH) sh=SCRAP_WIDTH;
+  if (sw> me->sqw) sw=me->sqw;
+  if (sh> me->sqh) sh=me->sqh;
 
   if ( (gcpw < ww)|| (gcph < wh) ) {
     gcpw=ww; gcph=wh;
@@ -1627,8 +1661,8 @@ gboolean board_motion_event(GtkWidget *widget,
   // prevent segfaults (bug sf#459164)
   if (x < 0) x=0;
   if (y < 0) y=0;
-  if (x > SCRAP_WIDTH) x=SCRAP_WIDTH;
-  if (y > SCRAP_WIDTH) y=SCRAP_WIDTH;
+  if (x > ww) x=ww;
+  if (y > wh) y=wh;
 
   dx = x - me->dr_ox;
   dy = y - me->dr_oy;
@@ -2005,7 +2039,7 @@ gboolean board_animate(gpointer data) {
 
 // ------------- targets
 
-DropSource::DropSource(piece p, int x1,int y1,int w,int h,bool d=true) {
+DropSource::DropSource(piece p, int x1,int y1,int w,int h,bool d) {
   X=x1; Y=y1;
   X2=X+w; Y2=Y+h;
   P=p;
@@ -2104,7 +2138,7 @@ void EditBoard::popRunEngine(int x,int y) {
   GtkWidget *popmenu, *note=0, *e;
   vector<GtkWidget *> mi;  
   list<EngineBookmark *>::iterator ei;
-  int i;
+  unsigned int i;
   static int sindex[4]={0,1,2,3};
 
   eb_last=this;
@@ -2114,7 +2148,7 @@ void EditBoard::popRunEngine(int x,int y) {
   mi.push_back(gtk_menu_item_new_with_label("GNU Chess 4..."));
   mi.push_back(gtk_menu_item_new_with_label("Crafty..."));
   mi.push_back(gtk_menu_item_new_with_label("Sjeng..."));
-  mi.push_back(gtk_menu_item_new_with_label("Generic XBoard Engine..."));
+  mi.push_back(gtk_menu_item_new_with_label(_("Generic XBoard Engine...")));
   mi.push_back(gtk_menu_item_new());
 
   for(i=0;i<4;i++)
@@ -2123,9 +2157,9 @@ void EditBoard::popRunEngine(int x,int y) {
 		       (gpointer) (&sindex[i]) );
   
   if (!global.EnginePresets.empty()) {
-    note=gtk_menu_item_new_with_label("If you pick a bookmark, the engine\n"\
+    note=gtk_menu_item_new_with_label(_("If you pick a bookmark, the engine\n"\
 				      "will play the next move, ignoring\n"\
-				      "the side setting in the bookmark.");
+				      "the side setting in the bookmark."));
     gtk_widget_set_sensitive(note,FALSE);
   }
 
@@ -2142,12 +2176,12 @@ void EditBoard::popRunEngine(int x,int y) {
     mi.push_back(gtk_menu_item_new());
 
   for(i=0;i<mi.size();i++) {
-    gtk_widget_show(mi[i]);
+    gshow(mi[i]);
     gtk_menu_append(GTK_MENU(popmenu),mi[i]);
   }
 
   if (note) {
-    gtk_widget_show(note);
+    gshow(note);
     gtk_menu_append(GTK_MENU(popmenu), note);
   }
 
@@ -2186,32 +2220,32 @@ void EditBoard::renderClock() {
   
   C.consumeTop(i*2+16);
   
-  j=C.drawButton(10,C.Y,-1,20,"Empty",2,0xffffff, 0x505070);
+  j=C.drawButton(10,C.Y,-1,20,_("Empty"),2,0xffffff, 0x505070);
   addTarget(new DropSource(EMPTY,Width()+10,C.Y,j,20,false));
   
   k=10+j+10;
-  j=C.drawButton(k,C.Y,-1,20,"Initial Position",2,0xffffff,0x505070);
+  j=C.drawButton(k,C.Y,-1,20,_("Initial Position"),2,0xffffff,0x505070);
   
   addTarget(new DropSource(WHITE|BLACK,Width()+k,C.Y,j,20,false));
 
   k+=10+j;
-  j=C.drawButton(k,C.Y,-1,20,"From FEN",2,0xffffff,0x505070);
+  j=C.drawButton(k,C.Y,-1,20,_("From FEN"),2,0xffffff,0x505070);
 
   addTarget(new DropSource(WASPAWN,Width()+k,C.Y,j,20,false));
   
   C.consumeTop(28);
   
-  j=C.drawButton(10,C.Y,-1,20,"Run Engine...",
+  j=C.drawButton(10,C.Y,-1,20,_("Run Engine..."),
 		 2,0xffffff,0xa07050);
   addTarget(new DropSource(WHITE,Width()+10,C.Y,j,20,false));
 
   k=j+20;
-  j+=20+C.stringWidth(2,"Side to move: ");
+  j+=20+C.stringWidth(2,_("Side to move: "));
 
   C.drawButton( k,C.Y,j-k+10+24,20,NULL,2,0xffffff,0x505070);
   addTarget(new DropSource(BLACK,Width()+k,C.Y,j-k+10+24,20,false));
 
-  C.drawString( k+10, C.Y+14, 2, "Side to move: ");
+  C.drawString( k+10, C.Y+14, 2, _("Side to move: "));
 
   C.setColor( mygame->getSideHint() ? 0xffffff : 0);
   C.drawEllipse( 10+j+3, C.Y+3, 14, 14, true);
@@ -2244,7 +2278,7 @@ void EditBoard::applyFEN(string &s) {
 }
 
 GetFENDialog::GetFENDialog(EditBoard *_owner) 
-  : ModalDialog("Enter FEN Position")
+  : ModalDialog(N_("Enter FEN Position"))
 {
   GtkWidget *v,*hs,*hb,*ok,*cancel;
 
@@ -2267,21 +2301,15 @@ GetFENDialog::GetFENDialog(EditBoard *_owner)
   gtk_button_box_set_spacing(GTK_BUTTON_BOX(hb), 5);
   gtk_box_pack_start(GTK_BOX(v), hb, FALSE, TRUE, 2); 
 
-  ok=gtk_button_new_with_label("Ok");
+  ok=gtk_button_new_with_label(_("Ok"));
   GTK_WIDGET_SET_FLAGS(ok,GTK_CAN_DEFAULT);
-  cancel=gtk_button_new_with_label("Cancel");
+  cancel=gtk_button_new_with_label(_("Cancel"));
   GTK_WIDGET_SET_FLAGS(cancel,GTK_CAN_DEFAULT);
   gtk_box_pack_start(GTK_BOX(hb),ok,TRUE,TRUE,0);
   gtk_box_pack_start(GTK_BOX(hb),cancel,TRUE,TRUE,0);
   gtk_widget_grab_default(ok);
 
-  gtk_widget_show(ok);
-  gtk_widget_show(cancel);
-  gtk_widget_show(hb);
-  gtk_widget_show(hs);
-  gtk_widget_show(e);
-  gtk_widget_show(v);
-
+  Gtk::show(ok,cancel,hb,hs,e,v,0);
   setDismiss(GTK_OBJECT(cancel),"clicked");
 
   gtk_signal_connect(GTK_OBJECT(ok),"clicked",
@@ -2298,3 +2326,53 @@ void getfen_ok(GtkWidget *w, gpointer data) {
   me->owner->applyFEN(s);
   me->release();
 }
+
+// former animate.cc
+
+int AnimatedPiece::over() { return(ower<=0); }
+
+int AnimatedPiece::getX() { return X; }
+
+int AnimatedPiece::getY() { return Y; }
+
+piece AnimatedPiece::getPiece() { return Piece; }
+
+bool AnimatedPiece::getSoundHold() { return((SoundHold)&&(ower>0)); }
+
+// ================================================
+// flat implementation
+// ================================================
+
+FlatAnimatedPiece::FlatAnimatedPiece(piece p,int x0,int y0,
+				     int xf,int yf,int s,bool sndhold) :
+  AnimatedPiece()
+{
+  Piece=p;
+  X0=x0; Y0=y0; XF=xf; YF=yf;
+  steps=s; X=X0; Y=Y0;
+  DX=(XF-X0)/steps;
+  DY=(YF-Y0)/steps;
+  ower=steps;
+  SoundHold=sndhold;
+}
+
+void FlatAnimatedPiece::create(GdkWindow *parent,PieceSet *set,int sqside) {
+  square=sqside;
+}
+
+void FlatAnimatedPiece::lemming() {
+  delete this;
+}
+
+void FlatAnimatedPiece::destroy() {
+
+}
+
+void FlatAnimatedPiece::step() {
+  if (ower > 1) {
+    X+=DX;
+    Y+=DY;
+  }
+  if (ower==1) { X=XF; Y=YF; }
+  --ower;  
+}
