@@ -1,10 +1,10 @@
-/* $Id: chess.cc,v 1.62 2002/06/22 18:42:42 bergo Exp $ */
+/* $Id: chess.cc,v 1.79 2003/02/18 02:49:48 bergo Exp $ */
 
 /*
 
     eboard - chess client
     http://eboard.sourceforge.net
-    Copyright (C) 2000-2002 Felipe Paulo Guazzi Bergo
+    Copyright (C) 2000-2003 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -23,8 +23,7 @@
 
 */
 
-
-#include <iostream.h>
+#include <iostream>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -32,6 +31,7 @@
 #include <ctype.h>
 #include <time.h>
 #include <errno.h>
+#include "eboard.h"
 #include "chess.h"
 #include "movelist.h"
 #include "tstring.h"
@@ -41,7 +41,7 @@
 // --- stream operators
 
 ostream &operator<<(ostream &s, PGNheader h) {
-  int i;
+  unsigned int i;
   for(i=0;i<h.header.size();i++)
     s << '[' << h.header[i]->name << " \"" << h.header[i]->value << "\"]\n";
   return(s);
@@ -49,6 +49,9 @@ ostream &operator<<(ostream &s, PGNheader h) {
 
 // --- classes
 
+bool             ChessGame::GlyphsInited = false;
+vector<string *> ChessGame::Glyphs;
+
 ChessGame::ChessGame() {
   int i;
 
@@ -80,13 +83,13 @@ ChessGame::ChessGame() {
   source=GS_Other;
   source_data="n/a";
   AmPlaying=false;
-
-  global.create("ChessGame");
 }
 
 ChessGame::ChessGame(int _number,int _tyme,int _inc, int _rated,
 		     variant _variant,
 		     char *p1,char *p2) {
+  int i;
+
   GameNumber=_number;
   STime=_tyme;
   Inc=_inc;
@@ -112,11 +115,13 @@ ChessGame::ChessGame(int _number,int _tyme,int _inc, int _rated,
   source_data="n/a";
   AmPlaying=false;
 
-  global.create("ChessGame");
+  for(i=0;i<8;i++)
+    protodata[i]=0;
 }
 
 ChessGame::ChessGame(ChessGame *src) {
   list<Position>::iterator i;
+  int j;
   bool cset=false;
 
   GameNumber = src->GameNumber;
@@ -139,6 +144,9 @@ ChessGame::ChessGame(ChessGame *src) {
   PGNSource[0]=0;
   SourceOffset=0;
 
+  for(j=0;j<8;j++)
+    protodata[j] = src->protodata[j];
+
   // clone move list
   for(i=src->moves.begin();i!=src->moves.end();i++) {
     moves.push_back(Position(*i));
@@ -152,13 +160,10 @@ ChessGame::ChessGame(ChessGame *src) {
   source=src->source;
   source_data=src->source_data;
   AmPlaying=src->AmPlaying;
-
-  global.create("ChessGame");
 }
 
 ChessGame::~ChessGame() {
   moves.clear();
-  global.destroy("ChessGame");
 }
 
 int ChessGame::operator==(int gnum) {
@@ -171,7 +176,7 @@ void ChessGame::setFree() {
 }
 
 void ChessGame::acknowledgeInfo() {
-  char s[64],t[64];
+  char s[64];
 
   if (!myboard)
     return;
@@ -181,14 +186,14 @@ void ChessGame::acknowledgeInfo() {
     strcpy(s,info0);
   } else
     if ((STime)||(Inc)) {
-      sprintf(s,"Game #%d - %d %d %s",
+      sprintf(s,_("Game #%d - %d %d %s"),
 	      GameNumber,
 	      STime/60,Inc,
-	      Rated?"rated":"unrated");
+	      Rated?_("rated"):_("unrated"));
     } else {
-      sprintf(s,"Game #%d - untimed %s",
+      sprintf(s,_("Game #%d - untimed %s"),
 	      GameNumber,
-	      Rated?"rated":"unrated");
+	      Rated?_("rated"):_("unrated"));
     }
 
   
@@ -255,11 +260,35 @@ void ChessGame::fixExamineZigZag(Position &suspect) {
       }
 }
 
+// updates clocks and just-below-clocks strings
+void ChessGame::updateClockAndInfo(int wclock, int bclock,
+				   int blacktomove, char *infoline,
+				   bool sndflag)
+{
+  if (myboard) {
+    myboard->freeze();
+
+    if (StopClock)
+      myboard->setClock(wclock,bclock,blacktomove?2:-2,clock_regressive);
+    else
+      myboard->setClock(wclock,bclock,blacktomove?1:-1,clock_regressive);
+    
+    if (infoline)
+      myboard->setInfo(2,infoline);
+
+    if (!moves.empty()) {
+      myboard->setInfo(4,moves.back().getMaterialString(Variant));
+      myboard->setInfo(5,moves.back().getHouseString());
+    }
+    myboard->update(sndflag);
+    myboard->thaw();
+    myboard->contentUpdated();
+  }
+}
+
 void ChessGame::updatePosition(Position &p,int movenum,int blacktomove,
 			       int wclock,int bclock,char *infoline, 
-			       bool sndflag=false) {
-  char z[32];
-
+			       bool sndflag) {
   global.debug("ChessGame","updatePosition");
 
   if (over)
@@ -271,7 +300,7 @@ void ChessGame::updatePosition(Position &p,int movenum,int blacktomove,
     global.bugpane->setPosition(p);
 
     if (StopClock)
-      global.bugpane->setClock(wclock,bclock,0,clock_regressive);
+      global.bugpane->setClock(wclock,bclock,blacktomove?2:-2,clock_regressive);
     else
       global.bugpane->setClock(wclock,bclock,blacktomove?1:-1,clock_regressive);
 
@@ -281,7 +310,8 @@ void ChessGame::updatePosition(Position &p,int movenum,int blacktomove,
   if (last_half_move>=0)
     if (! moves.empty() )
       if (p == moves.back() ) {
-	myboard->update(sndflag);
+	updateClockAndInfo(wclock,bclock,blacktomove,infoline,
+			   sndflag);
 	return;
       }
 
@@ -297,27 +327,21 @@ void ChessGame::updatePosition(Position &p,int movenum,int blacktomove,
   cursor--;
   last_half_move=(movenum*2)+(blacktomove?0:1);
 
-  if (myboard) {
-    myboard->freeze();
-
-    if (StopClock)
-      myboard->setClock(wclock,bclock,0,clock_regressive);
-    else
-      myboard->setClock(wclock,bclock,blacktomove?1:-1,clock_regressive);
-    
-    if (infoline)
-      myboard->setInfo(2,infoline);
-    myboard->setInfo(4,np.getMaterialString(Variant));
-    myboard->setInfo(5,np.getHouseString());
-    myboard->update(sndflag);
-    myboard->thaw();
-    myboard->contentUpdated();
-  }
+  updateClockAndInfo(wclock,bclock,blacktomove,infoline,
+		     sndflag);
 
   if (mymovelist)
     mymovelist->updateList(moves);
 }
 
+void ChessGame::incrementActiveClock(int secs) {
+  ChessClock *c;
+  if (myboard) {
+    c = myboard->getClock();
+    c->incrementClock(0,secs);
+  }
+}
+
 void ChessGame::fireWhiteClock(int wval,int bval) {
   myboard->setClock(wval,bval,-1,clock_regressive);
 }
@@ -648,10 +672,10 @@ void ChessGame::guessInfoFromPGN() {
   static char *tcsep = "+ -/\t";
   const char *cp;
 
-  if (cp=pgn.get("White"))
+  if ((cp=pgn.get("White")))
     strcpy(PlayerName[0],cp);
 
-  if (cp=pgn.get("Black"))
+  if ((cp=pgn.get("Black")))
     strcpy(PlayerName[1],cp);
 
   cp=pgn.get("Result");
@@ -662,14 +686,14 @@ void ChessGame::guessInfoFromPGN() {
     result=UNDEF;
   }
 
-  if (cp=pgn.get("TimeControl")) {
+  if ((cp=pgn.get("TimeControl"))) {
     tstring t;
     t.set((char *) cp);
     STime = t.tokenvalue(tcsep);
     Inc   = t.tokenvalue(tcsep);
   }
 
-  if (cp=pgn.get("Variant")) {
+  if ((cp=pgn.get("Variant"))) {
     Variant = variantFromName((char *)cp);
   }
 }
@@ -677,7 +701,7 @@ void ChessGame::guessInfoFromPGN() {
 void ChessGame::guessPGNFromInfo() {
   time_t now;
   struct tm *snow;
-  char z[64];
+  char z[64],y[256],x[128];
   Position startpos;
 
   char rz[64];
@@ -687,9 +711,35 @@ void ChessGame::guessPGNFromInfo() {
   snow=localtime(&now);
 
   pgn.set("Event","?");
+  switch(source) {
+  case GS_ICS:
+    strcpy(x,variantName(Variant));
+    x[0] = toupper(x[0]);
+
+    // ICS Rated Chess Match
+    sprintf(y,"ICS %s %s Match",
+	    Rated?"Rated":"Unrated",
+	    x);
+    pgn.set("Event",y);
+    break;
+  case GS_Engine:
+    strcpy(x,variantName(Variant));
+    x[0] = toupper(x[0]);
+
+    // Engine Chess Match
+    sprintf(y,"Engine %s Match",x);
+    pgn.set("Event",y);
+    break;
+  case GS_Other:
+  case GS_PGN_File:
+  default:    
+    pgn.set("Event","?");
+    break;
+  }
+
   pgn.set("Site","?");
   
-  sprintf(z,"%d.%0.2d.%0.2d",1900+snow->tm_year,
+  sprintf(z,"%d.%.2d.%.2d",1900+snow->tm_year,
 	  1+snow->tm_mon,snow->tm_mday);
   
   pgn.set("Date",z);
@@ -738,7 +788,7 @@ void ChessGame::guessPGNFromInfo() {
     pgn.set("FEN", moves.front().getFEN() );
 }
 
-bool ChessGame::savePGN(char *filename, bool append=false) {
+bool ChessGame::savePGN(char *filename, bool append) {
   char z[512];
   const char *cp;
   list<Position>::iterator li;
@@ -747,7 +797,7 @@ bool ChessGame::savePGN(char *filename, bool append=false) {
   string sp, path, *p;
 
   if (moves.size() < 4) {
-    global.status->setText("savePGN failed: Won't save game with less than 2 moves");
+    global.status->setText(_("savePGN failed: Won't save game with less than 2 moves"),5);
     return false;
   }
 
@@ -762,8 +812,8 @@ bool ChessGame::savePGN(char *filename, bool append=false) {
   ofstream f(path.c_str(),append? ios::app : ios::out);
 
   if (!f) {
-    sprintf(z,"savePGN failed: %s",filename);
-    global.status->setText(z);
+    sprintf(z,_("savePGN failed: %s"),filename);
+    global.status->setText(z,10);
     return false;
   }
 
@@ -816,19 +866,19 @@ bool ChessGame::savePGN(char *filename, bool append=false) {
 
   f.close();
 
-  sprintf(z,"--- %s game to PGN file %s",
-	  append?"Appended":"Wrote",filename);
+  sprintf(z,_("--- %s game to PGN file %s"),
+	  append?_("Appended"):_("Wrote"),filename);
   global.output->append(z,0xc0ff00);
   return true;
 }
 
 // indexonly: do not load the move text now
-bool ChessGame::ParsePgnGame(istream &f, 
+bool ChessGame::ParsePgnGame(zifstream &f, 
 			     char * filename,
 			     bool indexonly, 
 			     int gameid,
-			     variant v = REGULAR,
-			     ChessGame *updatee = NULL) {
+			     variant v,
+			     ChessGame *updatee) {
   /* 
      ok, now (0.4.x) we do this in a DFA-like manner
      0  = state 0 (whitespace before movetext)
@@ -846,27 +896,29 @@ bool ChessGame::ParsePgnGame(istream &f,
 
      10 = initial state ;-)
           whitespace (and comments like in twic pgns) between games
+     11 = inside alternate line
   */
 
   int   state    = 10;
   int   halfmove = 0;
+  int   altmoves = 0;
   piece color    = WHITE;
+  tstring t;
 
   list<Position> gl;
-  list<Position>::iterator gli;
   ChessGame *game=0;
 
   ExtPatternMatcher Legacy0, Legacy1, Legacy2, Result;
   PatternBinder Binder, Binder2;
 
   char alpha[256], beta[256], omega[256];
-  int ap, bp;
+  int ap=0, bp=0;
   char line[512], *p;
   long pstart;
   GameResult r;
 
   static char *whitespace = " \t\n\r";
-  static char *token  =     "1234567890*/KQBNRP@kqbnrp=+#abcdefgh-Ox";
+  static char *token  =     "1234567890*/KQBNRP@kqbnrp=+#abcdefgh-Ox$";
   static char *digits =     "1234567890";
   static char *resultset =  "01*";
 
@@ -878,6 +930,8 @@ bool ChessGame::ParsePgnGame(istream &f,
   Result.set("*%N-%N*");
   Binder2.add(&Result,0);
 
+  ChessGame::initGlyphs();
+
   pstart=f.tellg();
   if (!updatee) {
     game = new ChessGame();
@@ -952,6 +1006,8 @@ bool ChessGame::ParsePgnGame(istream &f,
       case 8:
 	if (*p == '{')         { state=7; memset(alpha,0,256); ap=0; break; }
 
+	if (*p == '(')         { state=11; ap=0; altmoves++; break; }
+
 	if (strchr(token, *p)) { 
 	  state=6; 
 	  memset(alpha,0,256);
@@ -1009,6 +1065,11 @@ bool ChessGame::ParsePgnGame(istream &f,
 	  // move number token (1. , 1...)
 	  if ( strchr(digits, alpha[0]) && strchr(alpha, '.') ) {
 	    state=8; 
+            // Account for PGNs where Black moves first
+            if (strstr(alpha, "...") && !halfmove) {
+              color = BLACK;
+              halfmove = 1;
+            }
 #ifdef PGN_DEBUG
 	    cerr << "v: movenumber\n";
 #endif
@@ -1040,6 +1101,29 @@ bool ChessGame::ParsePgnGame(istream &f,
 	      break;
 	    }
 
+	    if (alpha[0]=='$') {
+	      int i;
+              t.set((char *) alpha);
+              if (! gl.empty() ) {
+                if (! global.annotator.isOpen() ) {
+		  i = t.tokenvalue("$");
+		  if (i < Glyphs.size()) {
+		    gl.back().addAnnotation( global.annotator.open() );
+		    global.annotator.append(* (Glyphs[i]));
+		    global.annotator.close();
+		  }
+                } else {
+		  i = t.tokenvalue("$");
+		  if (i < Glyphs.size()) {
+		    global.annotator.append(* (Glyphs[i]));
+		  }
+                }
+              }
+
+              state = 8;
+              break;
+            }
+
 #ifdef PGN_DEBUG
 	  cerr << "v: move text\n";
 #endif
@@ -1081,6 +1165,7 @@ bool ChessGame::ParsePgnGame(istream &f,
 	      gl.clear();
 
 	      delete game;
+	      game=0;
 	      return(ParsePgnGame(f,filename,indexonly,gameid,CRAZYHOUSE));
 	    }
 
@@ -1109,12 +1194,27 @@ bool ChessGame::ParsePgnGame(istream &f,
 	  return true;
 	}
 	break; // case 9
+
+      case 11: // Temporarily ignore alternate moves
+	if (*p == ')')
+	  if (!(--altmoves))
+             state = 8;
+	if (*p == '(')
+           altmoves++;
+	break;
+
       } // switch state
 
     } // for p
 
   } // while getline
 
+  // avoid issues with files whose last char belongs to a token
+  if (state == 9) {
+    gl.clear();
+    return true;
+  }
+  
   if (!gl.empty())
     gl.clear();
 
@@ -1126,23 +1226,23 @@ bool ChessGame::loadMoves() {
   char z[512];
   if (Loaded) return true;
 
-  ifstream f(PGNSource);
+  zifstream f(PGNSource);
 
   if (!f) {
-    sprintf(z,"can't load PGN move text from %s (error opening file)",PGNSource);
+    sprintf(z,_("can't load PGN move text from %s (error opening file)"),PGNSource);
     global.output->append(z,0xc0ff00);
     return false;
   }
 
   if (!f.seekg(SourceOffset)) {
-    sprintf(z,"can't seek to offset %lu of %s",SourceOffset,PGNSource);
+    sprintf(z,_("can't seek to offset %lu of %s"),SourceOffset,PGNSource);
     global.output->append(z,0xc0ff00);
     f.close();
     return false;
   }
   
   if (!ParsePgnGame(f,PGNSource,false,-1,Variant,this)) {
-    strcpy(z,"error parsing PGN data");
+    strcpy(z,_("error parsing PGN data"));
     global.output->append(z,0xc0ff00);
     f.close();
     return false;
@@ -1162,12 +1262,12 @@ void ChessGame::LoadPGN(char *filename) {
   list<ChessGame *>::iterator gi;
   bool id_space_empty=false;
 
-  ifstream f(filename);
+  zifstream f(filename);
   if (!f)
     return;
 
   //  cerr << "reading " << filename << endl;
-  pw=new UnboundedProgressWindow("%d games read");
+  pw=new UnboundedProgressWindow(_("%d games read"));
 
   // grab a list of existent game ids
   for(gi=global.GameList.begin();gi!=global.GameList.end();gi++)
@@ -1225,6 +1325,8 @@ variant ChessGame::variantFromName(char *p) {
 	  if (!strcmp(p,variantName(LOSERS)))     v=LOSERS;
 	  else
 	    if (!strcmp(p,variantName(GIVEAWAY)))   v=GIVEAWAY;
+	    else
+	      if (!strcmp(p,variantName(ATOMIC)))     v=ATOMIC;
   return v;
 }
 
@@ -1237,6 +1339,7 @@ const char * ChessGame::variantName(variant v) {
   case WILD:       return("wild");
   case LOSERS:     return("losers");
   case GIVEAWAY:   return("giveaway");
+  case ATOMIC:     return("atomic");
   default: return("unknown");
   }
 }
@@ -1332,7 +1435,7 @@ const char * PGNheader::get(const char *n) {
 // -- PGN info edit dialog ------
 
 PGNEditInfoDialog::PGNEditInfoDialog(ChessGame *src) :
-  ModalDialog("PGN Headers")
+  ModalDialog(N_("PGN Headers"))
 {
   GtkWidget *v,*sw,*hb,*hb2,*l[2],*setb,*hs,*bb,*closeb;
   obj=src;
@@ -1348,8 +1451,8 @@ PGNEditInfoDialog::PGNEditInfoDialog(ChessGame *src) :
   clist=gtk_clist_new(2);
   gtk_clist_set_shadow_type(GTK_CLIST(clist),GTK_SHADOW_IN);
   gtk_clist_set_selection_mode(GTK_CLIST(clist),GTK_SELECTION_SINGLE);
-  gtk_clist_set_column_title(GTK_CLIST(clist),0,"Key");
-  gtk_clist_set_column_title(GTK_CLIST(clist),1,"Value");
+  gtk_clist_set_column_title(GTK_CLIST(clist),0,_("Key"));
+  gtk_clist_set_column_title(GTK_CLIST(clist),1,_("Value"));
   gtk_clist_column_titles_passive(GTK_CLIST(clist));
   gtk_clist_column_titles_show(GTK_CLIST(clist));
   gtk_clist_set_column_min_width(GTK_CLIST(clist),0,64);
@@ -1362,17 +1465,17 @@ PGNEditInfoDialog::PGNEditInfoDialog(ChessGame *src) :
   gtk_container_add(GTK_CONTAINER(sw),clist);
 
   hb=gtk_hbox_new(FALSE,0);
-  del=gtk_button_new_with_label(" Remove Field ");
+  del=gtk_button_new_with_label(_(" Remove Field "));
   gtk_box_pack_end(GTK_BOX(hb),del,FALSE,FALSE,0);
   gtk_box_pack_start(GTK_BOX(v),hb,FALSE,FALSE,4);
 
   hb2=gtk_hbox_new(FALSE,0);
-  l[0]=gtk_label_new("Key:");
-  l[1]=gtk_label_new("Value:");
+  l[0]=gtk_label_new(_("Key:"));
+  l[1]=gtk_label_new(_("Value:"));
   en[0]=gtk_entry_new();
   en[1]=gtk_entry_new();
   
-  setb=gtk_button_new_with_label(" Set ");
+  setb=gtk_button_new_with_label(_(" Set "));
 
   gtk_box_pack_start(GTK_BOX(hb2),l[0],FALSE,FALSE,4);
   gtk_box_pack_start(GTK_BOX(hb2),en[0],TRUE,TRUE,4);
@@ -1389,26 +1492,13 @@ PGNEditInfoDialog::PGNEditInfoDialog(ChessGame *src) :
   gtk_button_box_set_spacing(GTK_BUTTON_BOX(bb), 5);
   gtk_box_pack_start(GTK_BOX(v),bb,FALSE,FALSE,2);  
 
-  closeb=gtk_button_new_with_label("Close");
+  closeb=gtk_button_new_with_label(_("Close"));
   GTK_WIDGET_SET_FLAGS(closeb,GTK_CAN_DEFAULT);
   gtk_box_pack_start(GTK_BOX(bb),closeb,TRUE,TRUE,0);
   gtk_widget_grab_default(closeb);
 
-  gtk_widget_show(closeb);
-  gtk_widget_show(bb);
-  gtk_widget_show(hs);
-  gtk_widget_show(l[0]);
-  gtk_widget_show(l[1]);
-  gtk_widget_show(en[0]);
-  gtk_widget_show(en[1]);
-  gtk_widget_show(setb);
-  gtk_widget_show(hb2);
-
-  gtk_widget_show(hb);
-  gtk_widget_show(del);
-  gtk_widget_show(sw);
-  gtk_widget_show(clist);
-  gtk_widget_show(v);
+  Gtk::show(closeb,bb,hs,l[0],l[1],en[0],en[1],setb,hb2,
+	    hb,del,sw,clist,v,0);
   setDismiss(GTK_OBJECT(closeb),"clicked");
   
   populate();
@@ -1491,3 +1581,47 @@ void pgnedit_rowunsel(GtkCList *w, gint row, gint col,
   me->Selection=-1;
   gtk_widget_set_sensitive(me->del,FALSE);
 }
+
+void ChessGame::initGlyphs() {
+  if (GlyphsInited)
+    return;
+
+  EboardFileFinder eff;
+  string nag, nagpath;
+  char line[512];
+  int i;
+
+  // fix to look for NLS versions
+  nag = "NAG.en.txt";
+
+  if (!eff.find(nag, nagpath)) {
+    failGlyphs();
+    return;
+  }
+
+  ifstream nagf(nagpath.c_str());
+
+  if (!nagf) {
+    failGlyphs();
+    return;
+  }
+
+  memset(line,0,512);
+  while(nagf.getline(line,511,'\n')) {
+    if (!strlen(line))
+      break;
+    Glyphs.push_back(new string(line));
+  }
+
+  nagf.close();
+  GlyphsInited = true;
+}
+
+void ChessGame::failGlyphs() {
+  int i;
+  string *oops;
+  oops = new string("PGN NAG file missing");
+  for(i=0;i<140;i++)
+    Glyphs.push_back(oops);
+  GlyphsInited = true;
+}
