@@ -1,4 +1,4 @@
-/* $Id: dlg_gamelist.cc,v 1.20 2002/04/24 01:16:33 bergo Exp $ */
+/* $Id: dlg_gamelist.cc,v 1.27 2003/06/30 16:55:42 bergo Exp $ */
 
 /*
 
@@ -23,10 +23,11 @@
 
 */
 
-#include <iostream.h>
+#include <iostream>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include "eboard.h"
 #include "global.h"
 #include "chess.h"
 #include "dlg_gamelist.h"
@@ -63,7 +64,7 @@ GameListDialog::GameListDialog(GameListListener *someone) {
 
   widget=gtk_window_new(GTK_WINDOW_TOPLEVEL);
   gtk_window_set_default_size(GTK_WINDOW(widget),600,400);
-  gtk_window_set_title(GTK_WINDOW(widget),"Game List");
+  gtk_window_set_title(GTK_WINDOW(widget),_("Game List"));
   gtk_window_set_position(GTK_WINDOW(widget),GTK_WIN_POS_CENTER);
   gtk_container_set_border_width(GTK_CONTAINER(widget),4);
   gtk_widget_realize(widget);
@@ -75,18 +76,18 @@ GameListDialog::GameListDialog(GameListListener *someone) {
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
 				 GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
 
-  style=gtk_style_new();
+  style=gtk_style_copy( gtk_widget_get_default_style() );
   fixed=gdk_font_load("fixed");
   if (fixed)
     style->font=fixed;
   else
-    cerr << "<GameListDialog::GameListDialog> Warning: couldn't load fixed font, this dialog's content will look awful.\n\n";
+    cerr << _("<GameListDialog::GameListDialog> Warning: couldn't load fixed font, this dialog's content will look awful.\n\n");
 
   clist=gtk_clist_new(2);
   gtk_clist_set_shadow_type(GTK_CLIST(clist),GTK_SHADOW_IN);
   gtk_clist_set_selection_mode(GTK_CLIST(clist),GTK_SELECTION_SINGLE);
   gtk_clist_set_column_title(GTK_CLIST(clist),0,"#");
-  gtk_clist_set_column_title(GTK_CLIST(clist),1,"Game Description");
+  gtk_clist_set_column_title(GTK_CLIST(clist),1,_("Game Description"));
   gtk_clist_column_titles_passive(GTK_CLIST(clist));
   gtk_clist_column_titles_show(GTK_CLIST(clist));
 
@@ -103,22 +104,19 @@ GameListDialog::GameListDialog(GameListListener *someone) {
   bh=gtk_hbox_new(TRUE,0);
   gtk_box_pack_start(GTK_BOX(v),bh,FALSE,FALSE,0);
 
-  b[0]=gtk_button_new_with_label("Refresh List");
-  b[1]=gtk_button_new_with_label("Observe");
+  b[0]=gtk_button_new_with_label(_("Refresh List"));
+  b[1]=gtk_button_new_with_label(_("Observe"));
 
   for(i=0;i<2;i++) {
     gtk_box_pack_start(GTK_BOX(bh),b[i],FALSE,TRUE,0);
-    gtk_widget_show(b[i]);
+    gshow(b[i]);
   }
 
   gtk_widget_set_sensitive(b[1],FALSE);
 
-  setIcon(icon_games_xpm,"Games");
+  setIcon(icon_games_xpm,_("Games"));
 
-  gtk_widget_show(bh);
-  gtk_widget_show(clist);
-  gtk_widget_show(sw);
-  gtk_widget_show(v);
+  Gtk::show(bh,clist,sw,v,0);
 
   gtk_signal_connect(GTK_OBJECT(widget),"delete_event",
 		     GTK_SIGNAL_FUNC(gamelist_delete),(gpointer)(&canclose));
@@ -146,7 +144,7 @@ void GameListDialog::appendGame(int gamenum, char *desc) {
 
 void GameListDialog::endOfList() {
   gtk_widget_set_sensitive(b[0],TRUE);
-  gtk_window_set_title(GTK_WINDOW(widget),"Game List");
+  gtk_window_set_title(GTK_WINDOW(widget),_("Game List"));
   gtk_clist_columns_autosize(GTK_CLIST(clist));
   canclose=1;
 }
@@ -172,7 +170,7 @@ void GameListDialog::refresh() {
     gtk_clist_clear(GTK_CLIST(clist));
     gtk_widget_set_sensitive(b[0],FALSE);
     gtk_widget_set_sensitive(b[1],FALSE);
-    gtk_window_set_title(GTK_WINDOW(widget),"Game List (refreshing...)");
+    gtk_window_set_title(GTK_WINDOW(widget),_("Game List (refreshing...)"));
   }
 }
 
@@ -233,7 +231,7 @@ StockListDialog::StockListDialog(StockListListener *someone) {
 
   widget=gtk_window_new(GTK_WINDOW_TOPLEVEL);
   gtk_window_set_default_size(GTK_WINDOW(widget),650,400);
-  gtk_window_set_title(GTK_WINDOW(widget),"Local Game List");
+  gtk_window_set_title(GTK_WINDOW(widget),_("Local Game List"));
   gtk_window_set_position(GTK_WINDOW(widget),GTK_WIN_POS_CENTER);
   gtk_container_set_border_width(GTK_CONTAINER(widget),4);
   gtk_widget_realize(widget);
@@ -270,25 +268,25 @@ StockListDialog::StockListDialog(StockListListener *someone) {
   left=gtk_vbox_new(FALSE,0);
   gtk_box_pack_start(GTK_BOX(h),left,FALSE,TRUE,0);
 
-  b[3]=PixButton(goc_open_xpm,"Load PGN...");
+  b[3]=PixButton(goc_open_xpm,_("Load PGN..."));
   gtk_box_pack_start(GTK_BOX(left),b[3],FALSE,TRUE,0);
 
-  b[0]=PixButton(goc_refresh_xpm,"Refresh List");
+  b[0]=PixButton(goc_refresh_xpm,_("Refresh List"));
   gtk_box_pack_start(GTK_BOX(left),b[0],FALSE,TRUE,0);
 
-  b[6]=PixButton(goc_discardall_xpm,"Discard All");
+  b[6]=PixButton(goc_discardall_xpm,_("Discard All"));
   gtk_box_pack_start(GTK_BOX(left),b[6],FALSE,TRUE,0);
 
-  b[1]=PixButton(goc_display_xpm,"Display Game");
+  b[1]=PixButton(goc_display_xpm,_("Display Game"));
   gtk_box_pack_start(GTK_BOX(left),b[1],FALSE,TRUE,0);
 
-  b[4]=PixButton(goc_save_xpm,"Save Game...");
+  b[4]=PixButton(goc_save_xpm,_("Save Game..."));
   gtk_box_pack_start(GTK_BOX(left),b[4],FALSE,TRUE,0);
 
-  b[2]=PixButton(goc_discard_xpm,"Discard Game");
+  b[2]=PixButton(goc_discard_xpm,_("Discard Game"));
   gtk_box_pack_start(GTK_BOX(left),b[2],FALSE,TRUE,0);
 
-  b[5]=PixButton(goc_edit_xpm,"Edit Game Info");
+  b[5]=PixButton(goc_edit_xpm,_("Edit Game Info"));
   gtk_box_pack_start(GTK_BOX(left),b[5],FALSE,TRUE,0);
 
   // -------
@@ -300,24 +298,24 @@ StockListDialog::StockListDialog(StockListListener *someone) {
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
 				 GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
 
-  style=gtk_style_new();
+  style=gtk_style_copy( gtk_widget_get_default_style() );
   fixed=gdk_font_load("fixed");
   if (fixed)
     style->font=fixed;
   else
-    cerr << "<StockListDialog::StockListDialog> Warning: couldn't load fixed font, this dialog's content will look awful.\n\n";
+    cerr << _("<StockListDialog::StockListDialog> Warning: couldn't load fixed font, this dialog's content will look awful.\n\n");
   
   // number(tree) displayed white black variant result
 
   clist=gtk_ctree_new(7,0);
   gtk_clist_set_shadow_type(GTK_CLIST(clist),GTK_SHADOW_IN);
   gtk_clist_set_selection_mode(GTK_CLIST(clist),GTK_SELECTION_SINGLE);
-  gtk_clist_set_column_title(GTK_CLIST(clist),0,"Game #");
-  gtk_clist_set_column_title(GTK_CLIST(clist),1,"Displayed");
-  gtk_clist_set_column_title(GTK_CLIST(clist),2,"White");
-  gtk_clist_set_column_title(GTK_CLIST(clist),3,"Black");
-  gtk_clist_set_column_title(GTK_CLIST(clist),4,"Variant");
-  gtk_clist_set_column_title(GTK_CLIST(clist),5,"Result");
+  gtk_clist_set_column_title(GTK_CLIST(clist),0,_("Game #"));
+  gtk_clist_set_column_title(GTK_CLIST(clist),1,_("Displayed"));
+  gtk_clist_set_column_title(GTK_CLIST(clist),2,_("White"));
+  gtk_clist_set_column_title(GTK_CLIST(clist),3,_("Black"));
+  gtk_clist_set_column_title(GTK_CLIST(clist),4,_("Variant"));
+  gtk_clist_set_column_title(GTK_CLIST(clist),5,_("Result"));
 
   gtk_clist_set_column_title(GTK_CLIST(clist),6,"ctree is buggy it seems");
   gtk_clist_set_column_visibility(GTK_CLIST(clist),6,FALSE);
@@ -339,7 +337,7 @@ StockListDialog::StockListDialog(StockListListener *someone) {
   // ----------------------------------
 
   for(i=0;i<7;i++)
-    gtk_widget_show(b[i]);
+    gshow(b[i]);
 
   gtk_widget_set_sensitive(b[0],TRUE);
   gtk_widget_set_sensitive(b[1],FALSE);
@@ -349,14 +347,9 @@ StockListDialog::StockListDialog(StockListListener *someone) {
   gtk_widget_set_sensitive(b[5],FALSE);
   gtk_widget_set_sensitive(b[6],TRUE);
 
-  setIcon(icon_local_xpm,"Local");
+  setIcon(icon_local_xpm,_("Local"));
 
-  gtk_widget_show(clist);
-  gtk_widget_show(sw);
-  gtk_widget_show(v);
-
-  gtk_widget_show(left);
-  gtk_widget_show(h);
+  Gtk::show(clist,sw,v,left,h,0);
 
   gtk_signal_connect(GTK_OBJECT(widget),"delete_event",
 		     GTK_SIGNAL_FUNC(gamelist_delete),(gpointer)(&canclose));
@@ -389,7 +382,8 @@ void StockListDialog::refresh() {
   list<ChessGame *>::iterator gli;
   char x[16],y[8],z[64],a[64],b[32],c[16];
   char *zp[7];
-  int i,j;
+  unsigned int i;
+  int j;
 
   char ka[256], kb[256];
   string *kp;
@@ -402,9 +396,6 @@ void StockListDialog::refresh() {
   GtkCTreeNode *myfile;
   tstring t;
 
-  static char *e="e";
-  static char *es="es";
-
   SelectedRow=-1;
   gtk_clist_freeze(GTK_CLIST(clist));
   gtk_clist_clear(GTK_CLIST(clist));
@@ -428,7 +419,7 @@ void StockListDialog::refresh() {
   for(gli=global.GameList.begin();gli!=global.GameList.end();gli++) {
     sprintf(x,"%d",(*gli)->GameNumber);
 
-    strcpy(y,(*gli)->getBoard()?"Yes":"No");
+    strcpy(y,(*gli)->getBoard()?_("Yes"):_("No"));
 
     strcpy(z,(*gli)->PlayerName[0]);
     strcpy(a,(*gli)->PlayerName[1]);
@@ -436,7 +427,7 @@ void StockListDialog::refresh() {
     strcpy(b,ChessGame::variantName((*gli)->Variant));
 
     if (!(*gli)->isOver())
-      strcpy(c,"in progress");
+      strcpy(c,_("in progress"));
     else {
       GameResult gr=(*gli)->getResult();
       switch(gr) {
@@ -460,6 +451,7 @@ void StockListDialog::refresh() {
 	myfile=gtk_ctree_insert_node(GTK_CTREE(clist), toplevel[1], 0,
 				     0, 2, icons[4],masks[4], 
 				     icons[4],masks[4], FALSE, TRUE);
+	gtk_ctree_node_set_text(GTK_CTREE(clist), myfile, 6, "gtk_is_buggy");
 	filenames.push_back( (*gli)->source_data );
 	filenodes.push_back( myfile );
 	count.push_back(1);
@@ -497,7 +489,8 @@ void StockListDialog::refresh() {
       }
     } while(kp);
 
-    snprintf(kb,256,"%s (%d gam%s)", ka, count[i], count[i]==1?e:es);
+    snprintf(kb,256,"%s (%d %s)", ka, count[i], 
+	     count[i]==1?_("game"):_("games"));
     kb[255]=0;
       
     gtk_ctree_node_set_pixtext(GTK_CTREE(clist) , filenodes[i], 0, kb,
@@ -507,19 +500,19 @@ void StockListDialog::refresh() {
   for(i=0;i<4;i++)
     gtk_ctree_node_set_text(GTK_CTREE(clist), toplevel[i], 6, "gtk-is-buggy");
 
-  sprintf(kb,"From ICS (%d gam%s)", kids[0], kids[0]==1?e:es);
+  sprintf(kb,"From ICS (%d %s)", kids[0], kids[0]==1?_("game"):_("games"));
   gtk_ctree_node_set_pixtext(GTK_CTREE(clist), toplevel[0], 0, kb, 
 			     4, icons[0],masks[0]);
 
-  sprintf(kb,"From PGN Files (%d gam%s)", kids[1], kids[1]==1?e:es);
+  sprintf(kb,"From PGN Files (%d %s)", kids[1], kids[1]==1?_("game"):_("games"));
   gtk_ctree_node_set_pixtext(GTK_CTREE(clist), toplevel[1], 0, kb,
 			     4, icons[1], masks[1]);
 
-  sprintf(kb,"From Engines (%d gam%s)", kids[2], kids[2]==1?e:es);
+  sprintf(kb,"From Engines (%d %s)", kids[2], kids[2]==1?_("game"):_("games"));
   gtk_ctree_node_set_pixtext(GTK_CTREE(clist), toplevel[2], 0, kb,
 			     4, icons[2], masks[2]);
 
-  sprintf(kb,"From Elsewhere (%d gam%s)", kids[3], kids[3]==1?e:es);
+  sprintf(kb,"From Elsewhere (%d %s)", kids[3], kids[3]==1?_("game"):_("games"));
   gtk_ctree_node_set_pixtext(GTK_CTREE(clist), toplevel[3], 0, kb,
 			     4, icons[3], masks[3]);
 
@@ -549,7 +542,7 @@ void StockListDialog::open() {
   b->walkBackAll();  
   cg->acknowledgeInfo();
   
-  sprintf(tab,"Game #%d",cg->GameNumber);
+  sprintf(tab,_("Game #%d"),cg->GameNumber);
   global.ebook->addPage(b->widget,tab,cg->GameNumber);
   b->setNotebook(global.ebook,cg->GameNumber);
   b->pop();
@@ -678,12 +671,12 @@ stocklist_savepgn (GtkWidget * w, gpointer data)
   ChessGame *cg;
 
   me=(StockListDialog *)data;
-  fd=new FileDialog("Save as PGN");
+  fd=new FileDialog(_("Save as PGN"));
   
   if (fd->run()) {
     cg=global.getGame(me->SelectedRow);
     if (cg)
-      cg->savePGN(fd->FileName);
+      cg->savePGN(fd->FileName, true);
   }
 
   delete fd;
@@ -696,7 +689,7 @@ stocklist_loadpgn (GtkWidget * w, gpointer data)
   FileDialog *fd;
 
   me=(StockListDialog *)data;
-  fd=new FileDialog("Load PGN");
+  fd=new FileDialog(_("Load PGN"));
 
   if (fd->run()) {
     ChessGame::LoadPGN(fd->FileName);
@@ -797,7 +790,7 @@ AdListDialog::AdListDialog(AdListListener *someone) {
 
   widget=gtk_window_new(GTK_WINDOW_TOPLEVEL);
   gtk_window_set_default_size(GTK_WINDOW(widget),600,400);
-  gtk_window_set_title(GTK_WINDOW(widget),"Ad List");
+  gtk_window_set_title(GTK_WINDOW(widget),_("Ad List"));
   gtk_window_set_position(GTK_WINDOW(widget),GTK_WIN_POS_CENTER);
   gtk_container_set_border_width(GTK_CONTAINER(widget),4);
   gtk_widget_realize(widget);
@@ -809,18 +802,18 @@ AdListDialog::AdListDialog(AdListListener *someone) {
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
 				 GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
 
-  style=gtk_style_new();
+  style=gtk_style_copy( gtk_widget_get_default_style() );
   fixed=gdk_font_load("fixed");
   if (fixed)
     style->font=fixed;
   else
-    cerr << "<AdListDialog::AdListDialog> Warning: couldn't load fixed font, this dialog's content will look awful.\n\n";
+    cerr << _("<AdListDialog::AdListDialog> Warning: couldn't load fixed font, this dialog's content will look awful.\n\n");
 
   clist=gtk_clist_new(2);
   gtk_clist_set_shadow_type(GTK_CLIST(clist),GTK_SHADOW_IN);
   gtk_clist_set_selection_mode(GTK_CLIST(clist),GTK_SELECTION_SINGLE);
   gtk_clist_set_column_title(GTK_CLIST(clist),0,"#");
-  gtk_clist_set_column_title(GTK_CLIST(clist),1,"Ad Description");
+  gtk_clist_set_column_title(GTK_CLIST(clist),1,_("Ad Description"));
   gtk_clist_column_titles_passive(GTK_CLIST(clist));
   gtk_clist_column_titles_show(GTK_CLIST(clist));
 
@@ -837,22 +830,19 @@ AdListDialog::AdListDialog(AdListListener *someone) {
   bh=gtk_hbox_new(TRUE,0);
   gtk_box_pack_start(GTK_BOX(v),bh,FALSE,FALSE,0);
 
-  b[0]=gtk_button_new_with_label("Refresh List");
-  b[1]=gtk_button_new_with_label("Play");
+  b[0]=gtk_button_new_with_label(_("Refresh List"));
+  b[1]=gtk_button_new_with_label(_("Play"));
 
   for(i=0;i<2;i++) {
     gtk_box_pack_start(GTK_BOX(bh),b[i],FALSE,TRUE,0);
-    gtk_widget_show(b[i]);
+    gshow(b[i]);
   }
 
   gtk_widget_set_sensitive(b[1],FALSE);
 
-  setIcon(icon_ads_xpm,"Ads");
+  setIcon(icon_ads_xpm,_("Ads"));
 
-  gtk_widget_show(bh);
-  gtk_widget_show(clist);
-  gtk_widget_show(sw);
-  gtk_widget_show(v);
+  Gtk::show(bh,clist,sw,v,0);
 
   gtk_signal_connect(GTK_OBJECT(widget),"delete_event",
 		     GTK_SIGNAL_FUNC(gamelist_delete),(gpointer)(&canclose));
@@ -880,7 +870,7 @@ void AdListDialog::appendGame(int gamenum, char *desc) {
 
 void AdListDialog::endOfList() {
   gtk_widget_set_sensitive(b[0],TRUE);
-  gtk_window_set_title(GTK_WINDOW(widget),"Ad List");
+  gtk_window_set_title(GTK_WINDOW(widget),_("Ad List"));
   gtk_clist_columns_autosize(GTK_CLIST(clist));
   canclose=1;
 }
@@ -893,7 +883,7 @@ void AdListDialog::refresh() {
     gtk_clist_clear(GTK_CLIST(clist));
     gtk_widget_set_sensitive(b[0],FALSE);
     gtk_widget_set_sensitive(b[1],FALSE);
-    gtk_window_set_title(GTK_WINDOW(widget),"Ad List (refreshing...)");
+    gtk_window_set_title(GTK_WINDOW(widget),_("Ad List (refreshing...)"));
   }
 }
 
