@@ -1,4 +1,4 @@
-/* $Id: dlg_prefs.h,v 1.20 2002/04/27 14:43:42 bergo Exp $ */
+/* $Id: dlg_prefs.h,v 1.24 2003/01/20 03:30:20 bergo Exp $ */
 
 /*
 
@@ -40,15 +40,19 @@ class PreferencesDialog : public ModalDialog,
   void SoundEventChanged();
 
  private:
-  GtkWidget *tabposb[4],*efont[4],*fontdlg,*plainb,
-    *showratb, *autologb, *sbacke, *snddesc, *sndb[2],
+  GtkWidget *tabposb[4],*efont[NFONTS],*fontdlg,*plainb,
+    *showratb, *autologb, *sbacke, /* *allob[2], */ 
     *asp, *aso, *afn, *sgb, *hsb, *chsb, *coct, *dhsb, *aqbar, *lowtime,
-    *wget, *smoothb;
-  BoxedLabel *fm[4];
+    *wget, *smoothb, *xfont[NFONTS], 
+    *sndon[N_SOUND_EVENTS],*sndtest[N_SOUND_EVENTS],*sndedit[N_SOUND_EVENTS],
+    *sndd[N_SOUND_EVENTS];
+  BoxedLabel *fm[NFONTS];
   SoundEvent sndcopy[N_SOUND_EVENTS];
   ColorButton *lsq, *dsq, *textcb[11];
   TextPreview *preview;
-  int selected_sound_event;
+  int FontBeingEdited;
+
+  static char *FontSample[NFONTS];
 
   void ResolveFont(char *src,char *dest);
 
@@ -61,20 +65,11 @@ class PreferencesDialog : public ModalDialog,
   friend void prefs_apply(GtkWidget *w,gpointer data);
 
   friend void prefs_frevert(GtkWidget *w,gpointer data);
-  friend void prefs_cfont1(GtkWidget *w,gpointer data);
-  friend void prefs_cfont2(GtkWidget *w,gpointer data);
-  friend void prefs_cfont3(GtkWidget *w,gpointer data);
-  friend void prefs_cfont4(GtkWidget *w,gpointer data);
-
-  friend void prefs_fok1(GtkWidget *w,gpointer data);
-  friend void prefs_fok2(GtkWidget *w,gpointer data);
-  friend void prefs_fok3(GtkWidget *w,gpointer data);
-  friend void prefs_fok4(GtkWidget *w,gpointer data);
+  friend void prefs_cfont(GtkWidget *w,gpointer data);
+  friend void prefs_fok(GtkWidget *w,gpointer data);
 
   friend void prefs_sndtest(GtkWidget *w,gpointer data);
   friend void prefs_sndedit(GtkWidget *w,gpointer data);
-  friend void prefs_sndsel(GtkCList *c,gint row, gint col, GdkEventButton *eb, gpointer data);
-  friend void prefs_sndunsel(GtkCList *c,gint row, gint col, GdkEventButton *eb, gpointer data);
 
   friend void prefs_defcolor(GtkWidget *w,gpointer data);
 };
