@@ -1,10 +1,10 @@
-/* $Id: global.cc,v 1.66 2002/06/26 01:06:59 bergo Exp $ */
+/* $Id: global.cc,v 1.80 2003/07/03 19:47:53 bergo Exp $ */
 
 /*
 
     eboard - chess client
     http://eboard.sourceforge.net
-    Copyright (C) 2000-2002 Felipe Paulo Guazzi Bergo
+    Copyright (C) 2000-2003 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -23,7 +23,7 @@
 
 */
 
-#include <iostream.h>
+#include <iostream>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -51,6 +51,7 @@
 #include "board.h"
 #include "quickbar.h"
 #include "pieces.h"
+#include "eboard.h"
 
 Global global;
 
@@ -64,7 +65,17 @@ ostream & operator<<(ostream &s, WindowGeometry w) {
 
 ostream & operator<<(ostream &s, Desktop d) {
   s << d.wMain << ',' << d.wGames << ',' << d.wLocal << ',';
-  s << d.wAds;
+  s << d.wAds << ',' << d.PanePosition                          ;
+  return(s);
+}
+
+ostream & operator<<(ostream &s, TimeControl tc) {
+  TimeControl x;
+  x = tc;
+  if (x.mode == TC_SPM) x.value[1] = 0;
+  if (x.mode == TC_NONE) x.value[0] = x.value[1] = 0;
+  s << '/' << ((int)(x.mode)) << '/' << x.value[0];
+  s << '/' << x.value[1];
   return(s);
 }
 
@@ -73,7 +84,7 @@ ostream & operator<<(ostream &s, EngineBookmark b) {
   if (b.directory.empty()) s << "NULL"; else s << b.directory;
   s << '^';
   if (b.cmdline.empty()) s << "NULL"; else s << b.cmdline;
-  s << '^' << b.humanwhite << '^' << b.secspermove << '^';
+  s << '^' << b.humanwhite << '^' << b.timecontrol << '^';
   s << b.maxply  << '^' << b.think << '^';
   s << b.proto << '^' << ((int)b.mode);
   return(s);
@@ -97,6 +108,8 @@ ostream & operator<<(ostream &s, TerminalColor t) {
 }
 
 Global::Global() {
+  int i;
+  
   input=0;
   output=0;
   network=0;
@@ -168,6 +181,17 @@ Global::Global() {
 
   SmootherAnimation=0;
 
+  // chess machine had this and FICS admins didn't like it
+  // if you turn it on, you may be committing abuse.
+  // this is DAV's fault for not implementing a decent
+  // way of tracking game observers.
+  // I'm not going to delete the code. I'm just removing the
+  // user-friendly controls from the config dialog and setting
+  // the defaults to safe values.
+
+  IcsAllObPlayed   = 0;
+  IcsAllObObserved = 0;
+
   ShowQuickbar=1;
   LowTimeWarningLimit=5;
   RetrieveChannelNames=1;
@@ -185,10 +209,20 @@ Global::Global() {
   sndevents[4].Pitch=740;
   sndevents[4].Duration=120;
 
+  for(i=5;i<N_SOUND_EVENTS;i++) {
+    sndevents[i].Pitch=610;
+    sndevents[i].Duration=60;
+    sndevents[i].enabled=false;
+  }
+
   strcpy(ClockFont,DEFAULT_FONT_CLOK);
   strcpy(PlayerFont,DEFAULT_FONT_PLYR);
   strcpy(InfoFont,DEFAULT_FONT_INFO);
   strcpy(ConsoleFont,DEFAULT_FONT_CONS);
+  strcpy(SeekFont,DEFAULT_FONT_SEEK);
+
+  memset(P2PName,0,64);
+  env.User.copy(P2PName,63);
 
   RCKeys.push_back("HilightLastMove"); //0
   RCKeys.push_back("AnimateMoves");
@@ -233,12 +267,16 @@ Global::Global() {
   RCKeys.push_back("RetrieveChannelNames"); //40
   RCKeys.push_back("SmootherAnimation");
   RCKeys.push_back("EngineBookmark");  // 42
+  RCKeys.push_back("SeekFont");
+  RCKeys.push_back("IcsAllObPlayed");  // 44
+  RCKeys.push_back("IcsAllObObserved");
+  RCKeys.push_back("P2PName");         // 46
 
   PopupHelp = false;
 }
 
 void Global::dropQuickbarButtons() {
-  int i;
+  unsigned int i;
   for(i=0;i<QuickbarButtons.size();i++)
     delete(QuickbarButtons[i]);
   QuickbarButtons.clear();
@@ -260,11 +298,11 @@ void Global::deleteGame(ChessGame *cg) {
     }
 }
 
-void Global::appendGame(ChessGame *cg,bool RenumberDupes=true) {
+void Global::appendGame(ChessGame *cg,bool RenumberDupes) {
   if (RenumberDupes) clearDupes(cg);
   GameList.push_back(cg);
 }
-void Global::prependGame(ChessGame *cg, bool RenumberDupes=true) {
+void Global::prependGame(ChessGame *cg, bool RenumberDupes) {
   if (RenumberDupes) clearDupes(cg);
   GameList.push_front(cg);
 }
@@ -306,7 +344,7 @@ void Global::ensureDirectories() {
   DIR *tdir;
 
   if (env.Home.empty()) {
-    cerr << "[eboard] ** no $HOME" << endl;
+    cerr << _("[eboard] ** no $HOME") << endl;
     return;
   }
   sprintf(z,"%s/.eboard",env.Home.c_str());
@@ -333,7 +371,7 @@ int Global::createDir(char *z) {
     closedir(tdir);
   else
     if (mkdir(z,0755)) {
-      cerr << "[eboard] ** failed to create directory " << z << endl;
+      cerr << _("[eboard] ** failed to create directory ") << z << endl;
       return -1;
     }
   return 0;
@@ -380,7 +418,7 @@ void Global::readRC() {
     if (!p) continue;
     if (p->at(0)=='#') continue;
 
-    for(j=0;j<=42;j++) {
+    for(j=0;j<=46;j++) {
       if (! p->compare(RCKeys[j]) ) {
 	switch(j) {
 	case  0: HilightLastMove =t.tokenvalue(sep); break;
@@ -445,9 +483,13 @@ void Global::readRC() {
 	case 41: SmootherAnimation   =t.tokenvalue(sep); break;
 	case 42: ebm=new EngineBookmark(); ebm->read(t);
 	         EnginePresets.push_back(ebm); break;
-
+	case 43: p=t.token(sep2); memset(SeekFont,0,96);
+	         p->copy(SeekFont,95); break;
 		 //	default: cerr << "ignored [" << (*p) << "]\n";
-
+	case 44: IcsAllObPlayed      =t.tokenvalue(sep); break;
+	case 45: IcsAllObObserved    =t.tokenvalue(sep); break;
+	case 46: p=t.token(sep2); memset(P2PName,0,64);
+                 p->copy(P2PName,63); break;
 	} // switch
       } // compare
     } // for j 0..41
@@ -463,7 +505,7 @@ void Global::writeRC() {
   list<HostBookmark *>::iterator bi;
   list<EngineBookmark *>::iterator ei;
   static char *tabpos="RLTB";
-  int i;
+  unsigned int i;
 
   if (env.Config.empty())
     return;
@@ -537,6 +579,12 @@ void Global::writeRC() {
   for(ei=EnginePresets.begin();ei!=EnginePresets.end();ei++)    
     rc << RCKeys[42] << '^' << (*(*ei)) << endl;
 
+  rc << RCKeys[43] << div << SeekFont << endl;
+
+  rc << RCKeys[44] << div << IcsAllObPlayed << endl;
+  rc << RCKeys[45] << div << IcsAllObObserved << endl;
+  rc << RCKeys[46] << div << P2PName << endl;
+
   rc.close();
 }
 
@@ -574,6 +622,7 @@ void Global::WrappedMainQuit() {
 
 void Global::addAgent(NetConnection *ag) {
   Agents.push_back(ag);
+  ag->notifyReadReady(iowatcher);
 }
 
 void Global::removeAgent(NetConnection *ag) {
@@ -607,7 +656,7 @@ int  Global::receiveAgentLine(char *dest,int limit) {
 }
 
 void Global::opponentMoved() {
-  if (BeepWhenOppMoves) {
+  if (BeepWhenOppMoves && sndevents[0].enabled) {
     if (AnimateMoves)
       SoundStack.push(0);
     else
@@ -629,24 +678,28 @@ void Global::flushSound() {
   }
 }
 
-void Global::drawOffered() {
-  if (EnableSounds)
-    sndevents[1].safePlay();
+void Global::drawOffered()    { playOther(1); }
+void Global::privatelyTold()  { playOther(2); }
+void Global::challenged()     { playOther(3); }
+void Global::timeRunningOut() { playOther(4); }
+void Global::gameWon()        { playOther(5); }
+void Global::gameLost()       { playOther(6); }
+void Global::gameStarted()    { playOther(7); }
+void Global::gameFinished()   { playOther(8); }
+
+void Global::moveMade() {
+  if (EnableSounds && sndevents[9].enabled) {
+    if (AnimateMoves)
+      SoundStack.push(9);
+    else
+      sndevents[9].safePlay();
+  }
 }
 
-void Global::privatelyTold() {
-  if (EnableSounds)
-    sndevents[2].safePlay();
-}
-
-void Global::challenged() {
-  if (EnableSounds)
-    sndevents[3].safePlay();
-}
-
-void Global::timeRunningOut() {
-  if (EnableSounds)
-    sndevents[4].safePlay();
+void Global::playOther(int i) {
+  if (i>=N_SOUND_EVENTS) return;
+  if (EnableSounds && sndevents[i].enabled)
+    sndevents[i].safePlay();
 }
 
 void Global::repaintAllBoards() {
@@ -669,7 +722,7 @@ void Global::setPasswordMode(int pm) {
     (*i)->setPasswordMode(pm);
 }
 
-void Global::debug(char *klass,char *method,char *data=0) {
+void Global::debug(char *klass,char *method,char *data) {
   char z[256];
   time_t now;
   string rm;
@@ -708,7 +761,7 @@ void Global::LogAppend(char *msg) {
     return;
 
   if (PauseLog)
-    msg="(message obfuscated -- password mode ?)";
+    msg=_("(message obfuscated -- password mode ?)");
 
   if (CommLog) {
     sprintf(z,"%s/LOG.eboard",env.Home.c_str());
@@ -919,7 +972,7 @@ int HostBookmark::operator==(HostBookmark *hbm) {
 
 int EngineBookmark::operator==(EngineBookmark *ebm) {
   if (humanwhite != ebm->humanwhite) return 0;
-  if (secspermove != ebm->secspermove) return 0;
+  if (timecontrol != ebm->timecontrol) return 0;
   if (maxply != ebm->maxply) return 0;
   if (think != ebm->think) return 0;
   if (proto != ebm->proto) return 0;
@@ -931,12 +984,17 @@ int EngineBookmark::operator==(EngineBookmark *ebm) {
 
 void EngineBookmark::read(tstring &t) {
   static char *sep="^\n\r";
+  string *p;
 
   caption     = *(t.token(sep));
   directory   = *(t.token(sep));
   cmdline     = *(t.token(sep));
   humanwhite  = t.tokenvalue(sep);
-  secspermove = t.tokenvalue(sep);
+
+  p = t.token(sep);
+  if (p)
+    timecontrol.fromSerialization(p->c_str());
+
   maxply      = t.tokenvalue(sep);
   think       = t.tokenvalue(sep);
   proto       = t.tokenvalue(sep);
@@ -1068,7 +1126,27 @@ void ChannelSplitter::createPane(int ch) {
   panes.push_back(op);
 }
 
-void ChannelSplitter::appendToChannel(int ch,char *msg,int color,Importance im=IM_NORMAL) {
+void ChannelSplitter::channelPageUp(int ch) {
+  int i,j;
+  j=panes.size();
+  for(i=0;i<j;i++)
+    if (numbers[i] == ch) {
+      panes[i]->pageUp();
+      return;
+    }
+}
+
+void ChannelSplitter::channelPageDown(int ch) {
+  int i,j;
+  j=panes.size();
+  for(i=0;i<j;i++)
+    if (numbers[i] == ch) {
+      panes[i]->pageDown();
+      return;
+    }
+}
+
+void ChannelSplitter::appendToChannel(int ch,char *msg,int color,Importance im) {
   int i,j;
   ensurePane(ch);
   j=panes.size();
@@ -1195,10 +1273,12 @@ void Desktop::clear() {
 }
 
 void Desktop::read(tstring &t) {
+  static char *sep=":,()\n\t\r ";
   wMain.read(t);
   wGames.read(t);
   wLocal.read(t);
   wAds.read(t);
+  global.Desk.PanePosition = t.tokenvalue(sep); /* BUG */
 }
 
 void Desktop::writeConsoles(ostream &s, const char *key) {
@@ -1269,7 +1349,8 @@ void ZombieHunter::add(int pid, SigChildHandler *sigh) {
 
 void ZombieHunter::handleSigChild() {
   pid_t epid;
-  int i,s;
+  unsigned int i;
+  int s;
   
   while ( ( epid = waitpid(-1,&s,WNOHANG) ) > 0 ) {
     for(i=0;i<pids.size();i++)
@@ -1287,55 +1368,6 @@ void zh_sigchild_handler(int sig) {
     global.zombies.handleSigChild();
 }
 
-// ---------
-
-void ObjectCounter::create(const char * k) {
-  int i;
-  i=getIndex(k);
-  if (i>=0) {
-    N[i]++;
-    T[i]++;
-  } else {
-    names.push_back(string(k));
-    N.push_back(1);
-    T.push_back(1);
-  }
-  // cout << "+" << k << endl;
-}
-
-void ObjectCounter::destroy(const char * k) {
-  int i;
-  i=getIndex(k);
-  if (i>=0)
-    N[i]--;
-  else {
-    names.push_back(string(k));
-    N.push_back(-1);
-    T.push_back(0);
-  }
-  //  cout << "-" << k << endl;
-}
-
-void ObjectCounter::report() {
-  int i,j;
-  j=names.size();
-
-  cout << "object report:\n";
-  cout << "--------------------\n";
-  for(i=0;i<j;i++)
-    cout << names[i] << "\ttotal " << T[i] << ", rest " << N[i] << endl;
-  cout << "--------------------\n";
-}
-
-int ObjectCounter::getIndex(const char *k) {
-  int i,j;
-  j=names.size();
-  for(i=0;i<j;i++)
-    if (!strcmp(k,names[i].c_str()))
-      return i;
-  return -1;
-}
-
 Environment::Environment() {
   char *p;
 
@@ -1344,14 +1376,14 @@ Environment::Environment() {
     Home=p;    
   } else {
     Home.erase();
-    cerr << "** eboard ** warning: HOME environment variable not set\n";
+    cerr << _("** eboard ** warning: HOME environment variable not set\n");
   }
 
   p=getenv("USER");
   if (p) {
     User=p;
   } else {
-    User="Human";
+    User=_("Human");
   }
 
   if (!Home.empty()) {
