@@ -1,4 +1,4 @@
-/* $Id: help.cc,v 1.12 2002/06/26 01:06:59 bergo Exp $ */
+/* $Id: help.cc,v 1.19 2003/01/02 14:58:25 bergo Exp $ */
 
 /*
 
@@ -46,10 +46,69 @@
 #include "global.h"
 #include "help.h"
 #include "mainwindow.h"
+#include "eboard.h"
+#include "widgetproxy.h"
+
+#include "gnupiece.xpm"
 
 namespace Help {
 
-  KeysDialog::KeysDialog() : ModalDialog("Help: Keys") {
+  AboutDialog::AboutDialog() : ModalDialog(N_("About eboard")) {
+    GtkWidget *v,*hb,*label,*bhb,*hs,*ok,*gnup;
+    GtkStyle *style;
+    GdkPixmap *gnud;
+    GdkBitmap *gnub;
+    char z[1024];
+
+    v=gtk_vbox_new(FALSE,4);
+    gtk_container_add(GTK_CONTAINER(widget),v);
+
+    hb=gtk_hbox_new(FALSE,4);
+    gtk_box_pack_start(GTK_BOX(v),hb,TRUE,TRUE,0);
+    
+    style=gtk_widget_get_style(widget);
+    gnud=gdk_pixmap_create_from_xpm_d(widget->window,&gnub,
+				      &style->bg[GTK_STATE_NORMAL],
+				      (gchar **)gnupiece_xpm);
+    gnup=gtk_pixmap_new(gnud,gnub);
+    
+    gtk_box_pack_start(GTK_BOX(hb),gnup,FALSE,TRUE,4);
+    
+    sprintf(z,
+	    _("eboard version %s\n"\
+	      "(c) 2000-2002 Felipe Bergo\n"\
+	      "<bergo@seul.org>\n"\
+	      "http://eboard.sourceforge.net\n\n"\
+	      "This program is free software; you can redistribute\n"\
+	      "it and/or modify it under the terms of the GNU General\n"\
+	      "Public License as published by the Free Software\n"\
+	      "Foundation; either version 2 of the License, or\n"\
+	      "(at your option) any later version.\n"),
+	    global.Version);
+    
+    label=gtk_label_new(z);
+    gtk_label_set_justify(GTK_LABEL(label),GTK_JUSTIFY_LEFT);
+    gtk_box_pack_start(GTK_BOX(hb),label,TRUE,TRUE,4);
+    
+    hs=gtk_hseparator_new();
+    gtk_box_pack_start(GTK_BOX(v),hs,FALSE,FALSE,4);
+    
+    bhb=gtk_hbutton_box_new();
+    gtk_button_box_set_layout(GTK_BUTTON_BOX(bhb), GTK_BUTTONBOX_END);
+    gtk_button_box_set_spacing(GTK_BUTTON_BOX(bhb), 5);
+    
+    gtk_box_pack_start(GTK_BOX(v),bhb,FALSE,FALSE,4);
+    
+    ok=gtk_button_new_with_label(_("Close"));
+    GTK_WIDGET_SET_FLAGS(ok,GTK_CAN_DEFAULT);
+    gtk_box_pack_start(GTK_BOX(bhb),ok,TRUE,TRUE,0);
+    
+    gtk_widget_grab_default(ok);   
+    Gtk::show(ok,bhb,hs,label,gnup,hb,v,0);
+    setDismiss(GTK_OBJECT(ok),"clicked");
+  }
+
+  KeysDialog::KeysDialog() : ModalDialog(N_("Help: Keys")) {
     GtkWidget *v,*label,*bhb,*hs,*ok;
     char z[1024];
     
@@ -57,13 +116,13 @@ namespace Help {
     gtk_container_add(GTK_CONTAINER(widget),v);
     
     strcpy(z,
-	   "Anywhere:\n"\
+	   _("Anywhere:\n"\
 	   "F3: Go to previous pane.\n"\
 	   "F4: Go to next pane.\n"\
 	   "F5: Go to the main board pane.\n"\
 	   "F6: Go to the console pane.\n"\
 	   "F7: Go to the seek graph pane (if available).\n"\
-	   "F8: Input focus on input box.\n"\
+	   "F8: Toggle Shortcut Bar visilibity.\n"\
 	   "Page Up/Page Down: scrolls the text console (must be visible)\n"\
 	   "Ctrl+(Left Arrow): Backward 1 halfmove\n"\
 	   "Ctrl+(Right Arrow): Forward 1 halfmove\n"\
@@ -72,7 +131,11 @@ namespace Help {
 	   "\nInput box:\n"\
 	   "Up/Down (arrows): move thru input history\n"\
 	   "Enter: send text line\n"\
-	   "Esc: switch Chat/Command Mode");
+	   "Esc: switch Chat/Command Mode\n"\
+	   "\nSyntaxisms:\n"\
+	   "In main window's input box:\n"\
+	   ".. entering %prefix text will set the chat prefix to text.\n"\
+	   ".. entering %do scriptname will run scriptname."));
     
     label=gtk_label_new(z);
     gtk_label_set_justify(GTK_LABEL(label),GTK_JUSTIFY_LEFT);
@@ -87,21 +150,17 @@ namespace Help {
     
     gtk_box_pack_start(GTK_BOX(v),bhb,FALSE,FALSE,4);
 
-    ok=gtk_button_new_with_label("Close");
+    ok=gtk_button_new_with_label(_("Close"));
     GTK_WIDGET_SET_FLAGS(ok,GTK_CAN_DEFAULT);
     gtk_box_pack_start(GTK_BOX(bhb),ok,TRUE,TRUE,0);
     
     gtk_widget_grab_default(ok);
     
-    gtk_widget_show(ok);
-    gtk_widget_show(bhb);
-    gtk_widget_show(hs);
-    gtk_widget_show(label);
-    gtk_widget_show(v);
+    Gtk::show(ok,bhb,hs,label,v,0);
     setDismiss(GTK_OBJECT(ok),"clicked");
   }
 
-  DebugDialog::DebugDialog() : ModalDialog("Help: Debug Info") {
+  DebugDialog::DebugDialog() : ModalDialog(N_("Help: Debug Info")) {
     GtkWidget *v,*label,*hs,*bhb,*ok,*txt;
     char z[2048],y[256],x[256],w[256];
     struct stat s;
@@ -149,10 +208,10 @@ namespace Help {
 
     if (lstat(y,&s)==0) {
       sprintf(w,"Found %s\n",y); strcpy(x,w);
-      sprintf(w,"Size %d bytes, Last Modified %s",
+      sprintf(w,"Size %lu bytes, Last Modified %s",
 	      s.st_size, ctime(&(s.st_mtime)) );
       strcat(x,w);
-      sprintf(w,"Mode %0.4o. (seems it i%s enough to be run by eboard)\n",
+      sprintf(w,"Mode %.4o. (seems it i%s enough to be run by eboard)\n",
 	      s.st_mode,
 	      ( S_ISREG(s.st_mode)
 		&&
@@ -187,8 +246,8 @@ namespace Help {
     
     gtk_box_pack_start(GTK_BOX(v),bhb,FALSE,FALSE,4);
 
-    ok=gtk_button_new_with_label("Close");
-    txt=gtk_button_new_with_label("Write to Console");
+    ok=gtk_button_new_with_label(_("Close"));
+    txt=gtk_button_new_with_label(_("Write to Console"));
     GTK_WIDGET_SET_FLAGS(ok,GTK_CAN_DEFAULT);
     GTK_WIDGET_SET_FLAGS(txt,GTK_CAN_DEFAULT);
     gtk_box_pack_start(GTK_BOX(bhb),txt,TRUE,TRUE,0);
@@ -196,12 +255,7 @@ namespace Help {
     
     gtk_widget_grab_default(ok);
     
-    gtk_widget_show(ok);
-    gtk_widget_show(txt);
-    gtk_widget_show(bhb);
-    gtk_widget_show(hs);
-    gtk_widget_show(label);
-    gtk_widget_show(v);
+    Gtk::show(ok,txt,bhb,hs,label,v,0);
     setDismiss(GTK_OBJECT(ok),"clicked");
 
     gtk_signal_connect(GTK_OBJECT(txt),"clicked",
@@ -247,10 +301,10 @@ namespace Help {
   }
 
 
-  GettingStarted::GettingStarted() : NonModalDialog("Help: Getting Started ") {
+  GettingStarted::GettingStarted() : NonModalDialog(N_("Help: Getting Started ")) {
     GtkWidget *v, *bhb, *ok;
 
-    gtk_window_set_default_size(GTK_WINDOW(widget),580,500);
+    gtk_window_set_default_size(GTK_WINDOW(widget),580,520);
     v=gtk_vbox_new(FALSE,4);
     gtk_container_add(GTK_CONTAINER(widget),v);
 
@@ -263,16 +317,13 @@ namespace Help {
     
     gtk_box_pack_start(GTK_BOX(v),bhb,FALSE,FALSE,4);
 
-    ok=gtk_button_new_with_label("Close");
+    ok=gtk_button_new_with_label(_("Close"));
     GTK_WIDGET_SET_FLAGS(ok,GTK_CAN_DEFAULT);
     gtk_box_pack_start(GTK_BOX(bhb),ok,TRUE,TRUE,0);
     
     gtk_widget_grab_default(ok);
     
-    gtk_widget_show(ok);
-    gtk_widget_show(bhb);
-    gtk_widget_show(text);
-    gtk_widget_show(v);
+    Gtk::show(ok,bhb,text,v,0);
     setDismiss(GTK_OBJECT(ok),"clicked");
 
     compose();
@@ -282,6 +333,7 @@ namespace Help {
     GdkFont *fl, *fm, *fs, *f;
     GdkColor blue, *pc;
     char *T,*p,*q;
+    static char *tags="\200\201\202\203\204";
 
     blue.red=0;
     blue.green=0;
@@ -293,7 +345,7 @@ namespace Help {
 
     // codes: 200 large  201 medium  202 small  203 blue  204 black
 
-    T="\200Getting Started\n"\
+    T=_("\200Getting Started\n"\
       "\201Common Tasks in eboard\n\n"\
       "Playing against the computer\n"\
       "\202eboard does not \"play chess\" itself, but rather is works as interface to programs that do,\n"\
@@ -316,17 +368,17 @@ namespace Help {
       "\203Games on Client\204. In the Local Game List dialog, click \203Load PGN...\204. To browse a\n"\
       "game, \203select it\204 and click \203Display\204. A new tab will be created in the main window\n"\
       "with the game."\
-      "\200";
+      "\200");
 
     q=T;
     f=fs;
     pc=0;
     for(p=T;*p;p++) {
 
-      if (*p < 0) {
+      if (strchr(tags,*p)) {
 
 	if (p-q) gtk_text_insert(GTK_TEXT(text),f,pc,0,q,p-q);
-	for(q=p;*q < 0;q++) ;
+	for(q=p;!strchr(tags,*q);q++) ;
 
 	switch(*p) {
 	case '\200': f=fl; break;
