@@ -1,4 +1,4 @@
-/* $Id: main.cc,v 1.12 2002/04/23 07:55:09 bergo Exp $ */
+/* $Id: main.cc,v 1.16 2003/01/19 19:24:02 bergo Exp $ */
 
 /*
 
@@ -24,7 +24,7 @@
 */
 
 
-#include <iostream.h>
+#include <iostream>
 #include <stdlib.h>
 #include <gtk/gtk.h>
 #include <gdk_imlib.h>
@@ -34,6 +34,8 @@
 #include "global.h"
 #include "sound.h"
 #include "help.h"
+#include "eboard.h"
+#include "config.h"
 
 static void dump_tables(int signum) {
   // dump all internal tables to stderr when SIGUSR1 is received
@@ -42,7 +44,6 @@ static void dump_tables(int signum) {
   global.dumpBoards();
   global.dumpPanes();
   */
-  global.report();
 }
 
 int main(int argc,char **argv) {
@@ -62,6 +63,15 @@ int main(int argc,char **argv) {
   signal(SIGPIPE,SIG_IGN);
   signal(SIGUSR1,dump_tables); // for debugging
 
+
+#ifdef HAVE_GETTEXT
+  setlocale (LC_ALL, "");
+  bindtextdomain ("eboard", LOCALE_DIR);
+  textdomain ("eboard");
+#endif
+  gtk_set_locale ();
+
+
   gtk_init(&argc,&argv);
   gdk_rgb_init();
   gdk_imlib_init();
