@@ -1,10 +1,10 @@
-/* $Id: network.cc,v 1.32 2002/07/06 21:05:11 bergo Exp $ */
+/* $Id: network.cc,v 1.41 2003/07/03 19:47:53 bergo Exp $ */
 
 /*
 
     eboard - chess client
     http://eboard.sourceforge.net
-    Copyright (C) 2000-2002 Felipe Paulo Guazzi Bergo
+    Copyright (C) 2000-2003 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -25,7 +25,7 @@
 
 
 
-#include <iostream.h>
+#include <iostream>
 #include <time.h>
 #include <sys/time.h>
 #include <sys/types.h>
@@ -36,6 +36,8 @@
 #include "global.h"
 #include "status.h"
 #include "util.h"
+#include "config.h"
+#include "eboard.h"
 
 // ===================================================================
 // PROCESS CONTROL
@@ -55,11 +57,13 @@ void PidRing::add(PidIssuer *po, int pid) {
 
 void PidRing::remove(PidIssuer *po) {
   list<Parent *>::iterator i;
+  Parent *x;
 
   for(i=parents.begin();i!=parents.end();i++)
     if ( (*i)->issuer == po ) {
+       x = *i;
       parents.erase(i);
-      delete(*i);
+      delete(x);
       return;
     }
 }
@@ -91,6 +95,8 @@ NetConnection::NetConnection() {
   strcpy(HostName,"abstract");
   strcpy(HostAddress,"abstract");
   TimeGuard=0;
+  TagRead=-1;
+  listener = NULL;
 }
 
 int NetConnection::isConnected() {
@@ -98,7 +104,32 @@ int NetConnection::isConnected() {
 }
 
 NetConnection::~NetConnection() {
+  notifyReadReady(NULL);
+}
 
+void NetConnection::notifyReadReady(IONotificationInterface *target) {
+  if (TagRead>=0)
+    gdk_input_remove(TagRead);
+  TagRead = -1;
+  listener = NULL;
+  if (target != NULL) {
+    listener = target;
+    TagRead = gdk_input_add(getReadHandle(), GDK_INPUT_READ, 
+			    (GdkInputFunction) netconn_read_notify,
+			    (gpointer) this );
+  }
+}
+
+void netconn_read_notify(gpointer data, gint source, 
+			 GdkInputCondition cond) {
+  NetConnection *me = (NetConnection *) data;
+  if (me->listener != NULL)
+    me->listener->readAvailable(source);
+}
+
+void NetConnection::sendReadNotify() {
+  if (listener != NULL)
+    listener->readAvailable(getReadHandle());
 }
 
 int  NetConnection::hasTimeGuard() {
@@ -135,7 +166,7 @@ int BufferedConnection::bufferMatch(char *match) {
   return(strstr(dump,match)!=0);
 }
 
-int BufferedConnection::consume(int handle, int amount=128) {
+int BufferedConnection::consume(int handle, int amount) {
   int i,j;
   char sm[2048];
   if (amount>2048) amount=2048;
@@ -165,9 +196,9 @@ int BufferedConnection::consume(int handle, int amount=128) {
 }
 
 int BufferedConnection::produce(char *tbuffer,int limit,int handle) {
-  int i,j;
+  int i;
   char c;
-  list<char>::iterator di,dj;
+  list<char>::iterator di;
 
   for(di=buffer.begin();di!=buffer.end();di++)
     if (*di=='\n')
@@ -211,7 +242,7 @@ int AltBufferedConnection::innerReadLine(char *tbuffer,int limit,int handle) {
   int i,j;
   char sm[128];
   char c;
-  list<char>::iterator di,dj;
+  list<char>::iterator di;
 
   fd_set mine,*mp;
   struct timeval tv; 
@@ -273,7 +304,8 @@ DirectConnection::DirectConnection(char *hostname,int port) {
   strcpy(HostName,hostname);
   strcpy(HostAddress,"???");
   Port=port;
-  strcpy(errorMessage,"No error.");
+  Connected=0;
+  strcpy(errorMessage,_("No error."));
 }
 
 int DirectConnection::open() {
@@ -286,12 +318,12 @@ int DirectConnection::open() {
     global.LogAppend(ls);
   }
 
-  sprintf(z,"Looking up host %s...",HostName);
-  global.status->setText(z);
+  sprintf(z,_("Looking up host %s..."),HostName);
+  global.status->setText(z,30);
   
   he=gethostbyname(HostName);
   if (he==NULL) {
-    sprintf(errorMessage,"Host not found: %s",HostName);
+    sprintf(errorMessage,_("Host not found: %s"),HostName);
     return(-1);
   }
 
@@ -302,40 +334,50 @@ int DirectConnection::open() {
 	  (guchar) he->h_addr_list[0][3]);
 
   netsocket=socket(AF_INET,SOCK_STREAM,0);
+
+#ifndef USE_NO_TCP_OPTIONS
+
   int nagle=1;
-  setsockopt(netsocket,SOL_TCP,TCP_NODELAY,&nagle,sizeof(nagle));
+
+  #ifdef USE_SOL_TCP
+    setsockopt(netsocket,SOL_TCP,TCP_NODELAY,&nagle,sizeof(nagle));
+  #elif defined USE_IPPROTO_TCP
+    setsockopt(netsocket,IPPROTO_TCP,TCP_NODELAY,&nagle,sizeof(nagle));
+  #endif
+
+#endif
 
   sa.sin_family=he->h_addrtype;
   sa.sin_port=htons(Port);
   memcpy(&sa.sin_addr,he->h_addr_list[0],he->h_length);
 
-  sprintf(z,"Connecting to %s...",HostAddress);
-  global.status->setText(z);
+  sprintf(z,_("Connecting to %s..."),HostAddress);
+  global.status->setText(z,30);
 
   i=::connect(netsocket,(struct sockaddr *)&sa,sizeof(sa));
 
   if (i!=0) {
-    sprintf(z,"Connection to %s:%d failed: ",HostName,Port);
+    sprintf(z,_("Connection to %s:%d failed: "),HostName,Port);
     switch(errno) {
-    case EBADF:        strcat(z,"Bad descriptor");         break;
-    case EFAULT:       strcat(z,"Wrong address space");    break;
-    case ENOTSOCK:     strcat(z,"Not a socket ?!?");       break;
-    case EISCONN:      strcat(z,"Already connected ?!?");  break;
-    case ECONNREFUSED: strcat(z,"Connection refused");     break;
-    case ETIMEDOUT:    strcat(z,"Timeout");                break;
-    case ENETUNREACH:  strcat(z,"Network is unreachable"); break;
-    case EADDRINUSE:   strcat(z,"Address already in use"); break;
+    case EBADF:        strcat(z,_("Bad descriptor"));         break;
+    case EFAULT:       strcat(z,_("Wrong address space"));    break;
+    case ENOTSOCK:     strcat(z,_("Not a socket ?!?"));       break;
+    case EISCONN:      strcat(z,_("Already connected ?!?"));  break;
+    case ECONNREFUSED: strcat(z,_("Connection refused"));     break;
+    case ETIMEDOUT:    strcat(z,_("Timeout"));                break;
+    case ENETUNREACH:  strcat(z,_("Network is unreachable")); break;
+    case EADDRINUSE:   strcat(z,_("Address already in use")); break;
     case EALREADY:
     case EINPROGRESS:  strcat(z,"EINPROGRESS");           break;
-    default:           strcat(z,"Unknown error");
+    default:           strcat(z,_("Unknown error"));
     }
     strcpy(errorMessage,z);
     return(-1);
   }
   Connected=1;
   fcntl(netsocket,F_SETFL,O_NONBLOCK);
-  sprintf(z,"Connected to %s (%s)",HostName,HostAddress);
-  global.status->setText(z);
+  sprintf(z,_("Connected to %s (%s)"),HostName,HostAddress);
+  global.status->setText(z,300);
 
   global.getChannels(HostAddress);
 
@@ -372,6 +414,135 @@ char * DirectConnection::getError() {
   return(errorMessage);
 }
 
+int DirectConnection::getReadHandle() {
+  return(Connected?netsocket:-1);
+}
+
+// ===================================================================
+// INCOMING CONNECTION (FOR P2P PLAYING)
+// ===================================================================
+
+IncomingConnection::IncomingConnection(int port) {
+  strcpy(HostName,"localhost");
+  strcpy(HostAddress,"???");
+  Port = port;
+  Connected = 0;
+  netsocket = 0;  
+  strcpy(errorMessage,_("No error."));
+}
+
+/* the first open call on this object only
+   creates the socket, binds and listens,
+   subsequent calls do a non-blocking accept() */
+int IncomingConnection::open() {
+  if (Connected) return 0;
+  if (!netsocket)
+    return(createSocket());
+  else
+    return(acceptConnection());
+}
+
+int IncomingConnection::createSocket() {
+  struct sockaddr_in sin;
+
+  netsocket = socket(AF_INET, SOCK_STREAM, 0);
+  if (netsocket == -1) {
+    strcpy(errorMessage,_("Unable to create socket."));
+    return -1;
+  }
+
+  memset(&sin, 0, sizeof(sin));
+  sin.sin_family = AF_INET;
+  sin.sin_addr.s_addr = INADDR_ANY;
+  sin.sin_port = htons(Port);
+    
+  if (bind(netsocket,(struct sockaddr *) &sin,sizeof(sin))==-1) {
+    sprintf(errorMessage,_("Unable to bind on port %d."),Port);
+    return(-1);
+  }
+  
+  fcntl(netsocket,F_SETFL,O_NONBLOCK);
+  if (listen(netsocket, 1)!=0) {
+    sprintf(errorMessage,"Unable to listen on port %d.",Port);
+    return(-1);
+  }
+  return 0;
+
+}
+
+int IncomingConnection::acceptConnection() {
+  int       sock;
+  socklen_t addrlen;
+  struct    sockaddr_in pin;
+  char z[128];
+
+  addrlen = (socklen_t) sizeof(struct sockaddr_in);
+  sock = accept(netsocket, (struct sockaddr *) &pin, &addrlen);
+  
+  if (sock == -1) {
+    switch(errno) {
+    case EAGAIN: strcpy(errorMessage,"Nobody called."); break;
+    default:     strcpy(errorMessage,"Something broke."); break;
+    }
+    return -1;
+  }
+
+  strcpy(HostName, inet_ntoa(pin.sin_addr));
+  strcpy(HostAddress, HostName);
+
+  ::close(netsocket); /* kill the listening socket */
+  netsocket = sock;
+
+  strcpy(errorMessage,_("No error."));
+  Connected = 1;
+
+  fcntl(netsocket,F_SETFL,O_NONBLOCK);
+  sprintf(z,_("Accepted incoming connection from %s"),HostName);
+  global.status->setText(z,300);
+
+  return 0;
+}
+
+void IncomingConnection::close() {
+  if (Connected) {
+    shutdown(netsocket,2);
+    Connected=0;
+    netsocket=0;
+    ::close(netsocket);
+  } else {
+    ::close(netsocket);
+    netsocket=0;
+  }
+}
+
+int IncomingConnection::readLine(char *tbuffer, int limit) {
+  return(Connected?innerReadLine(tbuffer,limit,netsocket):-1);
+}
+
+void IncomingConnection::writeLine(char *obuffer) {
+  if (Connected) {
+    write(netsocket,obuffer,strlen(obuffer));
+    write(netsocket,"\n",1);
+    if (global.CommLog) {
+      char z[4096];
+      sprintf(z,"WROTE: %s",obuffer);
+      global.LogAppend(z);
+    }
+  }
+}
+
+int  IncomingConnection::isConnected() {
+  return(Connected);
+}
+
+char * IncomingConnection::getError() {
+  return(errorMessage);
+}
+
+int IncomingConnection::getReadHandle() {
+  return(Connected?netsocket:-1);
+}
+
 // ===================================================================
 // PIPE
 // ===================================================================
@@ -462,7 +633,7 @@ int PipeConnection::open() {
   int n2h[2], h2n[2];
 
   char *arguments[6];
-  int i;
+  unsigned int i;
 
   if (Connected)
     return 0;
@@ -482,12 +653,12 @@ int PipeConnection::open() {
       arguments[i]=args[i];
     break;
   case 1:
-    sprintf(z,"Looking up host %s...",HostName);
+    sprintf(z,_("Looking up host %s..."),HostName);
     if (!Quiet)
-      global.status->setText(z);   
+      global.status->setText(z,30);
     he=gethostbyname(HostName);
     if (he==NULL) {
-      sprintf(errorMessage,"Host not found: %s",HostName);
+      sprintf(errorMessage,_("Host not found: %s"),HostName);
       return(-1);
     }
 
@@ -497,9 +668,9 @@ int PipeConnection::open() {
 	    (guchar) he->h_addr_list[0][2],
 	    (guchar) he->h_addr_list[0][3]);
     
-    sprintf(z,"Connecting to %s...",HostAddress);
+    sprintf(z,_("Connecting to %s..."),HostAddress);
     if (!Quiet)
-      global.status->setText(z);  
+      global.status->setText(z,30);
 
     sprintf(z,"%d",Port);
     arguments[0]=HelperBin;
@@ -511,14 +682,14 @@ int PipeConnection::open() {
   // step 2: run helper
 
   if (pipe(n2h)||pipe(h2n)) {
-    strcpy(errorMessage,"IPC pipe creation failed.");
+    strcpy(errorMessage,_("IPC pipe creation failed."));
     return -1;    
   }
   signal(SIGPIPE,SIG_IGN);
 
   pid=fork();
   if (pid < 0) {
-    strcpy(errorMessage,"process creation failed.");
+    strcpy(errorMessage,_("process creation failed."));
     return -1;
   }
   if (!pid) {
@@ -547,7 +718,7 @@ int PipeConnection::open() {
     // send xboard/protover before anything else, to handle
     // gnu chess 5.04
     if (! handshake.empty() ) {
-      int i,j,n;
+      int wr,j,n;
       char hs[256];
 
       strncpy(hs,handshake.c_str(),255);
@@ -556,10 +727,10 @@ int PipeConnection::open() {
       j=0;
 
       do {
-	i=write(n2h[1],&hs[j],n);
-	if (i<0) break;
-	n-=i;
-	j+=i;
+	wr=write(n2h[1],&hs[j],n);
+	if (wr<0) break;
+	n-=wr;
+	j+=wr;
       } while(n);      
     }
 
@@ -573,7 +744,7 @@ int PipeConnection::open() {
       if ( (last_dead_pid == pid) || 
            (waitpid(pid, 0, WNOHANG) != 0) ) {
 	last_dead_pid = 0;
-	strcpy(errorMessage,"Program exited too soon");
+	strcpy(errorMessage,_("Program exited too soon"));
 	::close(h2n[0]);
 	::close(n2h[1]);
 	return -1;
@@ -582,7 +753,7 @@ int PipeConnection::open() {
     }
     *p=0;
     if (!strcmp(firstline,"exec failed")) {
-      strcpy(errorMessage,"Failed to run helper program");
+      strcpy(errorMessage,_("Failed to run helper program"));
       ::close(h2n[0]);
       ::close(n2h[1]);
       return -1;
@@ -598,13 +769,13 @@ int PipeConnection::open() {
 
   switch(opmode) {
   case 0:
-    strcpy(z,"Engine running");
+    strcpy(z,_("Engine running"));
   case 1:
-    sprintf(z,"Connected to %s (%s)",HostName,HostAddress);
+    sprintf(z,_("Connected to %s (%s)"),HostName,HostAddress);
     break;
   }
   if (!Quiet)
-    global.status->setText(z);
+    global.status->setText(z,300);
 
   if (opmode==1)
     global.getChannels(HostAddress);
@@ -670,11 +841,16 @@ void PipeConnection::farewellPid(int dpid) {
   toid=gtk_timeout_add(2000,sched_close,(gpointer)this);
 }
 
+int PipeConnection::getReadHandle() {
+  return(Connected?pin:-1);
+}
+
 gboolean sched_close(gpointer data) {
   PipeConnection *pc;
   pc=(PipeConnection *)data;
   pc->close();
   pc->toid=-1;
+  pc->sendReadNotify(); // cause disconnection to be detected
   return FALSE;
 }
 
@@ -761,3 +937,19 @@ char * FallBackConnection::getError() {
   return(errorMessage);
 }
 
+int FallBackConnection::getReadHandle() {
+  if (Connected)
+    return( (*current)->getReadHandle() );
+  else
+    return -1;
+}
+
+void FallBackConnection::notifyReadReady(IONotificationInterface *target) {
+  if (Connected)
+    (*current)->notifyReadReady(target);
+}
+
+void FallBackConnection::sendReadNotify() {
+  if (current != candidates.end())
+    (*current)->sendReadNotify();
+}
