@@ -1,10 +1,10 @@
-/* $Id: proto_fics.cc,v 1.80 2002/06/30 21:57:47 bergo Exp $ */
+/* $Id: proto_fics.cc,v 1.96 2003/07/05 02:48:19 bergo Exp $ */
 
 /*
 
     eboard - chess client
     http://eboard.sourceforge.net
-    Copyright (C) 2000-2002 Felipe Paulo Guazzi Bergo
+    Copyright (C) 2000-2003 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -23,9 +23,12 @@
 
 */
 
-#include <iostream.h>
+#include <iostream>
 #include <stdlib.h>
 #include <string.h>
+#include <gtk/gtk.h>
+
+#include "config.h"
 
 #ifdef HAVE_STRINGS_H
 #include <strings.h>
@@ -37,6 +40,7 @@
 #include "chess.h"
 #include "status.h"
 #include "seekgraph.h"
+#include "eboard.h"
 
 // <b1> game 45 white [PPPPPNBBR] black [PPNN]
 
@@ -53,8 +57,10 @@ FicsProtocol::FicsProtocol() {
   PActions.push_back(new string("History"));
   PActions.push_back(new string("Ping"));
   PActions.push_back(new string("Log"));
+  PActions.push_back(new string("Variables"));
 
   AuthenticationGone=0;
+  InitializationSent=false;
   LectureBotSpecial=false;
   LastWasPrompt=0;
   LastWasCTell=-1;
@@ -69,6 +75,7 @@ FicsProtocol::FicsProtocol() {
   lastALC=0;
 
   PartnerGame = -1;
+  UserPlayingWithWhite = true;
 
   xxnext=0;
   xxplayer[0][0]=0;
@@ -86,9 +93,9 @@ FicsProtocol::FicsProtocol() {
   WelcomeMessage.append(sp=new PercSSetPat());
   WelcomeMessage.append(new KleeneStarPat());
 
-  CreateGame.set("{Game %n (%s vs. %s) Creating %s %r *}*");
-  CreateGame0.set("Creating: %s (*) %s (*)*");
-  ContinueGame.set("{Game %n (%s vs. %s) Continuing %s %r *}*");
+  CreateGame.set("{Game %n (%S vs. %S) Creating %s %r *}*");
+  CreateGame0.set("Creating: %S (*) %S (*)*");
+  ContinueGame.set("{Game %n (%S vs. %S) Continuing %s %r *}*");
 
   BeginObserve.set("You are now observing game %n.");
   EndObserve.set("*Removing game %n from observation list*");
@@ -97,25 +104,25 @@ FicsProtocol::FicsProtocol() {
 
   Style12.set("<12> *");
   Style12House.set("<b1> game %n white [*] black [*]*");
-  WhiteWon.set    ("{Game %n (%s vs. %s) *} 1-0*");
-  BlackWon.set    ("{Game %n (%s vs. %s) *} 0-1*");
-  Drawn.set       ("{Game %n (%s vs. %s) *} 1/2-1/2*");
-  Interrupted.set ("{Game %n (%s vs. %s) *} %**");
+  WhiteWon.set    ("{Game %n (%S vs. %S) *} 1-0*");
+  BlackWon.set    ("{Game %n (%S vs. %S) *} 0-1*");
+  Drawn.set       ("{Game %n (%S vs. %S) *} 1/2-1/2*");
+  Interrupted.set ("{Game %n (%S vs. %S) *} %**");
 
-  DrawOffer.set("%s offers you a draw.*");
+  DrawOffer.set("%S offers you a draw.*");
 
-  PrivateTell.set ("%a tells you:*");
+  PrivateTell.set ("%A tells you:*");
   //Index of new news items
   News.set("Index of new *news items:*");
 
   // Ns: 0=game 1=rat.white 2=rat.black
   // Ss: 0=whitep 1=blackp
   // *s: 1=game string
-  GameStatus.set("*%n%b%n%b%s%b%n%b%s%b[*]*");
+  GameStatus.set("*%n%b%n%b%S%b%n%b%S%b[*]*");
 
   // 2 (Exam. 2413 Species     1234 Pulga     ) [ br  3   1] W:  1
   // amazingly enough, the indexes are the same as in GameStatus
-  ExaGameStatus.set("*%n (Exam.%b%n %s%b%n %s%b)%b[*]*");
+  ExaGameStatus.set("*%n (Exam.%b%n %S%b%n %S%b)%b[*]*");
 
   GameListEntry.set("*%n *[*]*:*");
   EndOfGameList.set("*%n games displayed.*");
@@ -124,11 +131,11 @@ FicsProtocol::FicsProtocol() {
   EndOfAdList.set("*%n ad* displayed.*");
 
   //IceBox (1156) vs. damilasa (UNR) --- Sun Apr 22, 09:07 ??? 2001
-  MovesHeader.set("%s (*) vs. %s (*) --- *");
+  MovesHeader.set("%S (*) vs. %S (*) --- *");
   //Unrated blitz match, initial time: 2 minutes, increment: 12 seconds.
   MovesHeader2.set("%s %r match, initial time: *");
   //Move  IceBox             damilasa
-  MovesHeader3.set("Move%b%s%b%s*");
+  MovesHeader3.set("Move%b%S%b%S*");
   //----  ----------------   ----------------
   MovesHeader4.set("----  ---*");
   //  1.  d4      (0:00)     d5      (0:00)
@@ -139,14 +146,14 @@ FicsProtocol::FicsProtocol() {
   //      {Still in progress} *
   MovesEnd.set("%b{*}*");
   
-  Kibitz.set  ("%a*[%n]%bkibitzes:*");
-  Whisper.set ("%a*[%n]%bwhispers:*");
-  Say.set     ("%a[%n] says:*");
-  Say2.set     ("%a says:*");
-  ChannelTell.set("%a%n):*");
+  Kibitz.set  ("%A*[%n]%bkibitzes:*");
+  Whisper.set ("%A*[%n]%bwhispers:*");
+  Say.set     ("%A[%n] says:*");
+  Say2.set     ("%A says:*");
+  ChannelTell.set("*(%n):*");
   Seek.set("*(*) seeking*(*to respond)");
-  Shout.set ("%a shouts: *");
-  cShout.set ("%a c-shouts: *");
+  Shout.set ("%A shouts: *");
+  cShout.set ("%A c-shouts: *");
   It.set ("-->*");
   Notify.set("Notification:*");
 
@@ -155,11 +162,11 @@ FicsProtocol::FicsProtocol() {
 
   // <s> 8 w=visar ti=02 rt=2194  t=4 i=0 r=r tp=suicide c=? rr=0-9999 a=t f=f
   //    n0   s0       r0    n1  *0  n2  n3  s1    r1       r2    r3      s2  s3
-  SgAdd.set("<s>%b%n%bw=%s%bti=%r%brt=%n*%bt=%n%bi=%n%br=%s%btp=%r%bc=%r%brr=%r%ba=%s%bf=%s*");
-  SgAdd2.set("%b<s>%b%n%bw=%s%bti=%r%brt=%n*%bt=%n%bi=%n%br=%s%btp=%r%bc=%r%brr=%r%ba=%s%bf=%s*");
+  SgAdd.set("<s>%b%n%bw=%S%bti=%r%brt=%n*%bt=%n%bi=%n%br=%s%btp=%r%bc=%r%brr=%r%ba=%s%bf=%s*");
+  SgAdd2.set("%b<s>%b%n%bw=%S%bti=%r%brt=%n*%bt=%n%bi=%n%br=%s%btp=%r%bc=%r%brr=%r%ba=%s%bf=%s*");
 
   // bughouse stuff
-  PTell.set("%a (your partner) tells you: *");
+  PTell.set("%A (your partner) tells you: *");
 
   // sample output: Your partner is playing game 75 (aglup vs. cglup).
   GotPartnerGame.set("Your partner is playing game %n (*");
@@ -171,6 +178,16 @@ FicsProtocol::FicsProtocol() {
   LectureBotXTell.set(":LectureBot(TD)(67):*");
   LectureBotXTellContinued.set(":   *");
 
+  // AllOb parsing
+  AllObNone.set("No one is observing game %n.*");
+
+  // Observing 18 [Fullmer vs. knighttour]: #boffo #HummerESS TieFighter (3 users)
+  // Observing 62 [VABORIS vs. Fudpucker]: BillJr(TM) CrouchingTiger(U) #Firefly 
+  AllObFirstLine.set("Observing %n [*]: *");
+  AllObMidLine.set("\\*");
+  AllObState = 0;
+  memset(AllObAcc,0,1024);
+
   memset(style12table,0,256*sizeof(piece));
   style12table['-']=EMPTY;
   style12table['P']=PAWN|WHITE;
@@ -198,7 +215,8 @@ FicsProtocol::FicsProtocol() {
   Binder.add(&Kibitz,&Whisper,&Say,&Shout,&cShout,&It,&ChannelTell,
 	     &Seek,&Notify,&SgClear,&SgAdd,&SgAdd2,&SgRemove,
 	     &PTell,&Say2,&Challenge,&LectureBotXTell,
-	     &LectureBotXTellContinued,&GotPartnerGame,0);
+	     &LectureBotXTellContinued,&GotPartnerGame,
+	     &AllObNone,&AllObFirstLine,&AllObMidLine,0);
 }
 
 FicsProtocol::~FicsProtocol() {
@@ -210,12 +228,15 @@ void FicsProtocol::finalize() {
   global.debug("FicsProtocol","finalize");
   if (global.network)
     global.network->writeLine("quit");  
+  if (global.IcsSeekGraph && global.skgraph)
+    global.skgraph->clear();
 }
 
 void FicsProtocol::receiveString(char *netstring) {
     int msgcolor=global.Colors.TextBright, ispersonal=0;
     char z[128];
     char pstring[1024];
+    ExtPatternMatcher *tm=0;
 
     global.debug("FicsProtocol","receiveString",netstring);
 
@@ -233,6 +254,14 @@ void FicsProtocol::receiveString(char *netstring) {
       // say the least
       if (strlen(Prompt.getStarToken(0))>9)
 	break;
+
+      if (!InitializationSent) {
+	UserName.reset();
+	UserName.append(new CIExactStringPat("IdontknowwhoIam/pleasefixme"));
+	strcpy(nick,"!idontknowwhoiam!");
+	sendInitialization();
+      }
+
       // this is to cope with the free (GPL) 1.6.2 version of FICS
       if (!AuthenticationGone) { 
 	AuthenticationGone=1;
@@ -267,16 +296,13 @@ void FicsProtocol::receiveString(char *netstring) {
 	global.input->setPasswordMode(0);
 	AuthenticationGone=1;
 	strcpy(nick,WelcomeMessage.getToken(3));	
+	UserName.reset();
 	UserName.append(new KleeneStarPat());
 	UserName.append(new CIExactStringPat(nick));
 	UserName.append(new KleeneStarPat());
 
-	sprintf(z,"set interface eboard %s/%s\niset startpos 1\niset defprompt 1",
-		global.Version,global.SystemType);
-	global.network->writeLine(z);
-	if (global.IcsSeekGraph)
-	  global.network->writeLine("iset seekinfo 1\niset seekremove 1\nset seek 1");
-	global.network->writeLine("iset lock 1\nstyle 12");
+	if (!InitializationSent)
+	  sendInitialization();
       }
     }
 
@@ -454,6 +480,45 @@ void FicsProtocol::receiveString(char *netstring) {
     if (GotPartnerGame.match())
       prepareBugPane();
 
+    if (!AllObReqs.empty() || AllObState!=0) {
+      switch(AllObState) {
+      case 0:
+	if (AllObNone.match()) {
+	  if (atoi(AllObNone.getNToken(0)) == AllObReqs.back()) {
+	    sprintf(AllObAcc,"Game %d: no observers.",AllObReqs.back());
+	    AllObReqs.pop_back();
+	    global.status->setText(AllObAcc,20);
+	    return;
+	  } else {
+	    goto just_output;
+	  }
+	}
+	if (AllObFirstLine.match())
+	  if (doAllOb1()) return; else goto just_output;
+
+	break; /* case 0 */	
+      case 1:
+	if (AllObMidLine.match()) {
+	  doAllOb2();
+	  return;
+	}
+	break; /* case 1 */
+      case 2:
+	if (strstr(netstring, "user")!=0)
+	  return;
+	if (netstring[0] == 0) {
+	  AllObState = 3;
+	  return;
+	}
+      case 3:
+	if (strstr(netstring,"1 game displayed")!=0) {
+	  AllObState = 0;
+	  return;
+	}
+	break;
+      }
+    }
+
     msgcolor=global.Colors.TextDefault;
     
     if (netstring[0]=='\\')
@@ -466,7 +531,6 @@ void FicsProtocol::receiveString(char *netstring) {
     }
 
     bool gottell;
-    ExtPatternMatcher *tm;
 
     gottell=false;
 
@@ -488,6 +552,7 @@ void FicsProtocol::receiveString(char *netstring) {
 
     if (UserName.match())    {
       msgcolor=global.Colors.PrivateTell; ispersonal=1;
+
       if (ChannelTell.match() || LectureBotSpecial || LectureBotXTell.match() )
 	goto it_is_a_chtell_too;
       else
@@ -575,7 +640,7 @@ void FicsProtocol::receiveString(char *netstring) {
     }
 
     if (ChannelTell.match()) {
-      if (!ispersonal)
+      if ((!ispersonal)&&(strchr(ChannelTell.getStarToken(0),' ')==0))
 	msgcolor = global.Colors.ChannelTell;
 
       if (global.SplitChannels) {
@@ -600,7 +665,7 @@ void FicsProtocol::receiveString(char *netstring) {
 void FicsProtocol::createExaminedGame(int gameid,char *whitep,char *blackp) {
   ChessGame *cg;
   Board *b;
-  char *p,z[64];
+  char z[64];
 
   global.debug("FicsProtocol","createExaminedGame");
 
@@ -622,7 +687,7 @@ void FicsProtocol::createExaminedGame(int gameid,char *whitep,char *blackp) {
   b->setCanMove(true);
   b->setSensitive(true);
 
-  sprintf(z,"Exam.Game #%d",cg->GameNumber);
+  sprintf(z,_("Exam.Game #%d"),cg->GameNumber);
   global.ebook->addPage(b->widget,z,cg->GameNumber);
   b->setNotebook(global.ebook,cg->GameNumber);
 
@@ -633,17 +698,35 @@ void FicsProtocol::createExaminedGame(int gameid,char *whitep,char *blackp) {
 
   cg->acknowledgeInfo();
 
-  sprintf(z,"game %d",cg->GameNumber);
+  sprintf(z,_("game %d"),cg->GameNumber);
   global.network->writeLine(z);
   PendingStatus++;
 }
 
+void FicsProtocol::sendInitialization() {
+  char z[128];
+  if (!InitializationSent) {
+    
+    sprintf(z,"set interface eboard %s/%s\niset startpos 1\niset defprompt 1",
+	    global.Version,global.SystemType);
+    global.network->writeLine(z);
+    if (global.Premove)
+      global.network->writeLine("iset premove 1\n");
+    else
+      global.network->writeLine("iset premove 0\n");
+    if (global.IcsSeekGraph)
+      global.network->writeLine("iset seekinfo 1\niset seekremove 1\nset seek 1");
+    global.network->writeLine("iset lock 1\nstyle 12");
+    InitializationSent = true;
+  }
+}
+
 // creates a non-moveable, non-editable position for sposition
 void FicsProtocol::createScratchPos(int gameid,char *whitep,char *blackp)
 {
   ChessGame *cg;
   Board *b;
-  char *p,z[64];
+  char z[64];
 
   global.debug("FicsProtocol","createScratchPos");
 
@@ -665,7 +748,7 @@ void FicsProtocol::createScratchPos(int gameid,char *whitep,char *blackp)
   b->setCanMove(false);
   b->setSensitive(false);
 
-  sprintf(z,"Pos: %s vs. %s",whitep,blackp);
+  sprintf(z,_("Pos: %s vs. %s"),whitep,blackp);
   global.ebook->addPage(b->widget,z,cg->GameNumber);
   b->setNotebook(global.ebook,cg->GameNumber);
 
@@ -704,10 +787,14 @@ void FicsProtocol::createGame(ExtPatternMatcher &pm) {
   if (p) strncpy(cg->Rating[1],p,32);
   clearRatingBuffer();
 
-  if (!strncmp(cg->PlayerName[0],nick,strlen(cg->PlayerName[0])))
+  if (!strncmp(cg->PlayerName[0],nick,strlen(cg->PlayerName[0]))) {
     cg->MyColor=WHITE;
-  if (!strncmp(cg->PlayerName[1],nick,strlen(cg->PlayerName[1])))
+    UserPlayingWithWhite = true;
+  }
+  if (!strncmp(cg->PlayerName[1],nick,strlen(cg->PlayerName[1]))) {
     cg->MyColor=BLACK;
+    UserPlayingWithWhite = false;
+  }
   
   if (!strcmp(pm.getSToken(2),"rated"))
     cg->Rated=1;
@@ -723,6 +810,8 @@ void FicsProtocol::createGame(ExtPatternMatcher &pm) {
     cg->Variant=SUICIDE;
   if (!strcmp(pm.getRToken(0),"losers"))
     cg->Variant=LOSERS;
+  if (!strcmp(pm.getRToken(0),"atomic"))
+    cg->Variant=ATOMIC;
   if (!strncmp(pm.getRToken(0),"wild",4))
     cg->Variant=WILD;
 
@@ -736,7 +825,8 @@ void FicsProtocol::createGame(ExtPatternMatcher &pm) {
   global.BoardList.front()->repaint();
 
   cg->acknowledgeInfo();
-  global.status->setText("Game started!");
+  global.status->setText(_("Game started!"),10);
+  global.gameStarted();
 }
 
 void FicsProtocol::addGame(ExtPatternMatcher &pm) {
@@ -772,7 +862,7 @@ void FicsProtocol::addGame(ExtPatternMatcher &pm) {
     cg->protodata[2] = 1;
   }
 
-  sprintf(tab,"Game #%d",cg->GameNumber);
+  sprintf(tab,_("Game #%d"),cg->GameNumber);
   global.ebook->addPage(b->widget,tab,cg->GameNumber);
   b->setNotebook(global.ebook,cg->GameNumber);
 
@@ -835,6 +925,10 @@ void FicsProtocol::removeGame(ExtPatternMatcher &pm) {
   global.debug("FicsProtocol","removeGame");
 
   gm=atoi(pm.getNToken(0));
+  
+  if (global.SmartDiscard && global.ebook->isNaughty(gm))
+    return;
+
   innerRemoveGame(gm);
 }
 
@@ -949,7 +1043,7 @@ void FicsProtocol::attachHouseStock() {
 void FicsProtocol::parseStyle12(char *data) {
   static char *sep=" \t\n";
   tstring t;
-  int i,j,n;
+  int i,j;
   string *sp;
   Position pos;
   int blacktomove;
@@ -961,7 +1055,9 @@ void FicsProtocol::parseStyle12(char *data) {
   char stra[64],strb[64],strc[64];
   char plyr[2][64];
   int ackp=0;
+  int stopclock = -1; /* -1: keep as it is, 0: stop 1: moving */
   bool spawnExamination;
+  bool someSound;
 
   global.debug("FicsProtocol","parseStyle12");
 
@@ -1052,7 +1148,12 @@ void FicsProtocol::parseStyle12(char *data) {
   t.token(sep)->copy(strb,63); // pretty print for previous move
 
   flip=t.tokenvalue(sep);    // flip board ?
- 
+
+  t.setFail(-1);
+  stopclock = t.tokenvalue(sep); // undocumented pause/unpause clock flag
+  if (stopclock != -1)
+    stopclock = 1-stopclock;
+
   refgame=global.getGame(game);
   if (!refgame) {
     cerr << "no such game: " << game << endl;
@@ -1118,8 +1219,13 @@ void FicsProtocol::parseStyle12(char *data) {
   if (rel==-2) // don't let it tick when observing examined games
     refgame->StopClock=1;
 
+  someSound = (rel==1 || rel==0 || rel==2 || rel==-2) && (movenum!=1 || blacktomove);
+
+  if (stopclock >= 0)
+    refgame->StopClock = stopclock;
+
   refgame->updatePosition(pos, (movenum-(blacktomove?0:1)), blacktomove,
-			  clk[0], clk[1], strc, (rel==1));
+			  clk[0], clk[1], strc, someSound);
 
   /*
   if ( ((rel==-1)&&(blacktomove)) || ((rel==1)&&(!blacktomove)) ) {
@@ -1131,12 +1237,28 @@ void FicsProtocol::parseStyle12(char *data) {
   }
   */
 
-  if (rel==1)
+  if (rel==1 && (movenum!=1 || blacktomove) )
     global.opponentMoved();
 
+  if (rel==0 || rel==-2 || rel==2 && (movenum!=1 || blacktomove))
+    global.moveMade();
+
   if (rel!=0) refgame->flipHint(flip);
   if (rel==1) refgame->enableMoving(true);
   if (rel==-1) refgame->enableMoving(false);
+
+  if (global.IcsAllObPlayed && (rel==1 || rel==-1) )
+    if (refgame->protodata[3] == 0) {
+      refgame->protodata[3] = gtk_timeout_add(30*1000, fics_allob, (void *) refgame);
+      fics_allob((gpointer) refgame);
+    }
+
+  if (global.IcsAllObObserved && (rel==0) )
+    if (refgame->protodata[3] == 0) {
+      refgame->protodata[3] = gtk_timeout_add(30*1000, fics_allob, (void *) refgame);
+      fics_allob((gpointer) refgame);
+    }
+
 }
 
 void FicsProtocol::updateGame(ExtPatternMatcher &pm) {
@@ -1149,8 +1271,8 @@ void FicsProtocol::updateGame(ExtPatternMatcher &pm) {
   // Ns: 0=game 1=rat.white 2=rat.black
   // Ss: 0=whitep 1=blackp
   // *s: 1=game string
-  //  GameStatus.set("*%n%b%n%b%s%b%n%b%s%b[*]*");
-  //  ExaGameStatus.set("*%n (Exam.%b%n %s%b%n %s%b)%b[*]*");
+  //  GameStatus.set("*%n%b%n%b%S%b%n%b%S%b[*]*");
+  //  ExaGameStatus.set("*%n (Exam.%b%n %S%b%n %S%b)%b[*]*");
 
   gm=atoi(pm.getNToken(0));
 
@@ -1186,6 +1308,9 @@ void FicsProtocol::updateGame(ExtPatternMatcher &pm) {
   case 'L':
     refgame->Variant=LOSERS;
     break;
+  case 'x':
+    refgame->Variant=ATOMIC;
+    break;
   }
 
   refgame->Rated=(p[2]=='r');
@@ -1224,10 +1349,30 @@ void FicsProtocol::gameOver(ExtPatternMatcher &pm, GameResult result) {
 
   refgame=global.getGame(game);
   if (!refgame) {
-    cerr << "no such game: " << game << endl;
+    cerr << _("no such game: ") << game << endl;
     return;
   }
 
+  // allob timeout
+  if (refgame->protodata[3]!=0) {
+    gtk_timeout_remove(refgame->protodata[3]);
+    refgame->protodata[3] = 0;
+  }
+
+  if (refgame->protodata[0]==0) {
+    if ( (   UserPlayingWithWhite  && result==WHITE_WIN) ||
+	 ( (!UserPlayingWithWhite) && result==BLACK_WIN) )
+      global.gameWon();
+
+    if ( (   UserPlayingWithWhite  && result==BLACK_WIN) ||
+	 ( (!UserPlayingWithWhite) && result==WHITE_WIN) )
+      global.gameLost();
+  }
+
+  if (refgame->protodata[0]==257) {
+    global.gameFinished();
+  }
+
   refgame->endGame(reason,result);
 
   if (((refgame->protodata[0]==0)&&(global.AppendPlayed))
@@ -1236,8 +1381,8 @@ void FicsProtocol::gameOver(ExtPatternMatcher &pm, GameResult result) {
     {
       if  (refgame->savePGN(global.AppendFile,true)) {
 	char z[128];
-	sprintf(z,"Game appended to %s",global.AppendFile);
-	global.status->setText(z);
+	sprintf(z,_("Game appended to %s"),global.AppendFile);
+	global.status->setText(z,5);
       }
     }
 
@@ -1270,27 +1415,25 @@ int  FicsProtocol::hasAuthenticationPrompts() {
 
 void FicsProtocol::resign() {
   global.network->writeLine("resign");
-  global.status->setText("Resigned.");
+  global.status->setText(_("Resigned."),10);
 }
 
 void FicsProtocol::draw() {
   global.network->writeLine("draw");
-  global.status->setText("<Fics Protocol> draw request sent");
+  global.status->setText(_("<Fics Protocol> draw request sent"),30);
 }
 
 void FicsProtocol::adjourn() {
   global.network->writeLine("adjourn");
-  global.status->setText("<Fics Protocol> adjourn request sent");
+  global.status->setText(_("<Fics Protocol> adjourn request sent"),30);
 }
 
 void FicsProtocol::abort() {
   global.network->writeLine("abort");
-  global.status->setText("<Fics Protocol> abort request sent");
+  global.status->setText(_("<Fics Protocol> abort request sent"),30);
 }
 
 void FicsProtocol::clearMoveBuffer() {
-  list<Position *>::iterator li;
-
   global.debug("FicsProtocol","clearMoveBuffer");
   MoveBuf.clear();
 }
@@ -1370,7 +1513,7 @@ void FicsProtocol::retrievef() {
 
   cg=global.getGame(gm);
   if (!cg) {
-    cerr << "no such game: " << gm << endl;
+    cerr << _("no such game: ") << gm << endl;
     return;
   }
   MoveRetrievals.pop_back();
@@ -1502,7 +1645,7 @@ void FicsProtocol::ensureSeekGraph() {
   if (!global.skgraph) {
     global.skgraph=new SeekGraph();
     global.skgraph->show();
-    global.ebook->addPage(global.skgraph->widget,"Seek Table",-3);
+    global.ebook->addPage(global.skgraph->widget,_("Seek Table"),-3);
     global.skgraph->setNotebook(global.ebook,-3);
   }
 }
@@ -1554,8 +1697,8 @@ void FicsProtocol::seekAdd(ExtPatternMatcher &pm) {
   p=pm.getRToken(2);
   switch(p[0]) {
   case '?': ad->color=" "; break;
-  case 'W': ad->color="white"; break;
-  case 'B': ad->color="black"; break;
+  case 'W': ad->color=_("white"); break;
+  case 'B': ad->color=_("black"); break;
   }
 
   ad->range=pm.getRToken(3);
@@ -1585,12 +1728,12 @@ vector<string *> * FicsProtocol::getGameActions() {
 }
 
 void FicsProtocol::callPlayerAction(char *player, string *action) {
-  for(int i=0;i<PActions.size();i++)
+  for(unsigned int i=0;i<PActions.size();i++)
     if ( (*PActions[i]) == (*action) ) { doPlayerAction(player, i); break; }
 }
 
 void FicsProtocol::callGameAction(int gameid, string *action) {
-  for(int i=0;i<GActions.size();i++)
+  for(unsigned int i=0;i<GActions.size();i++)
     if ( (*GActions[i]) == (*action) ) { doGameAction(gameid, i); break; }
 }
 
@@ -1604,11 +1747,12 @@ void FicsProtocol::doPlayerAction(char *player, int id) {
   case 2: sprintf(z,"history %s",player); break;
   case 3: sprintf(z,"ping %s",player); break;
   case 4: sprintf(z,"log %s",player); break;
+  case 5: sprintf(z,"var %s",player); break;
   default: ok=false;
   }
 
   if (ok) {
-    sprintf(w,"> [issued from menu] %s",z);
+    sprintf(w,_("> [issued from menu] %s"),z);
     global.output->append(w,global.SelfInputColor);
     global.network->writeLine(z);
   }
@@ -1625,7 +1769,7 @@ void FicsProtocol::doGameAction(int gameid, int id) {
   }
 
   if (ok) {
-    sprintf(w,"> [issued from menu] %s",z);
+    sprintf(w,_("> [issued from menu] %s"),z);
     global.output->append(w,global.SelfInputColor);
     global.network->writeLine(z);
   }
@@ -1637,3 +1781,121 @@ void FicsProtocol::prepareBugPane() {
   sprintf(z,"observe %d",PartnerGame);
   global.network->writeLine(z);
 }
+
+void FicsProtocol::updateVar(ProtocolVar pv) {
+  char z[256];
+  switch(pv) {
+  case PV_premove:
+    sprintf(z,"iset premove %d\n",global.Premove ? 1 : 0);
+    global.network->writeLine(z);
+    break;
+  default:
+    break;
+  }
+}  
+
+gboolean fics_allob(gpointer data) {
+  ChessGame *cg = (ChessGame *) data;
+  FicsProtocol *proto;
+  char z[64];
+
+  if (global.protocol == 0)  return FALSE;
+  if (cg == 0)               return FALSE;
+  if (cg->protodata[3] == 0) return FALSE;
+  if (cg->isOver())          return FALSE;
+  if (cg->source != GS_ICS)  return FALSE;
+  if (global.network == 0)   return FALSE;
+
+  proto = (FicsProtocol *) (global.protocol);
+  proto->AllObReqs.push_front(cg->GameNumber);
+
+  sprintf(z,"allob %d",cg->GameNumber);
+  global.network->writeLine(z);
+
+  return TRUE;
+}
+
+// AllObFirstLine Matched
+bool FicsProtocol::doAllOb1() {
+  int n,i;
+  tstring t;
+  string *p;
+  bool islast = false;
+
+  //  cerr << "allob2\n";
+
+  n = atoi(AllObFirstLine.getNToken(0));
+
+  // cerr << "n0=[" << AllObFirstLine.getNToken(0) << "]\n";
+  // cerr << "n=" << n << ", back=" << AllObReqs.back() << endl;
+  if (n!=AllObReqs.back())
+    return false;
+
+  t.set(AllObFirstLine.getStarToken(1));
+ 
+  sprintf(AllObAcc,"Observing game %d: ",n);
+  i=0;
+  while((p=t.token(" "))!=0) {
+
+    if ( (*p)[0] == '(' ) {
+      islast = true;
+      break;
+    }
+
+    if(i!=0) strcat(AllObAcc,", ");
+    strcat(AllObAcc,p->c_str());
+    ++i;
+
+    //    cerr << "partial: [" << AllObAcc << "]\n";
+
+    if (strlen(AllObAcc) > 128) {
+      strcat(AllObAcc,"...");
+      break;
+    }
+  }
+  
+  if (islast) {
+    AllObState = 2;
+    global.status->setText(AllObAcc,28);
+    AllObReqs.pop_back();
+  } else
+    AllObState = 1;
+    
+  return true;
+}
+
+// AllObMidLine matched
+void FicsProtocol::doAllOb2() {
+  int i;
+  bool islast = false;
+  tstring t;
+  string *p;
+
+  t.set(AllObFirstLine.getStarToken(0));
+ 
+  while((p=t.token(" "))!=0) {
+
+    if ( (*p)[0] == '(' ) {
+      islast = true;
+      break;
+    }
+    if (strlen(AllObAcc) > 128)
+      continue;
+
+    strcat(AllObAcc,", ");
+    strcat(AllObAcc,p->c_str());
+    ++i;
+    if (strlen(AllObAcc) > 128) {
+      strcat(AllObAcc,"...");
+      break;
+    }
+  }
+
+  if (islast) {
+    AllObState = 2;
+    global.status->setText(AllObAcc,28);
+    AllObReqs.pop_back();
+  } else
+    AllObState = 1;
+
+}
