@@ -1,10 +1,10 @@
-/* $Id: proto_fics.h,v 1.36 2002/06/14 17:32:57 bergo Exp $ */
+/* $Id: proto_fics.h,v 1.42 2003/07/05 02:48:19 bergo Exp $ */
 
 /*
 
     eboard - chess client
     http://eboard.sourceforge.net
-    Copyright (C) 2000-2001 Felipe Paulo Guazzi Bergo
+    Copyright (C) 2000-2003 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -28,13 +28,14 @@
 #ifndef PROTO_FICS_H
 #define PROTO_FICS_H 1
 
+#include <gtk/gtk.h>
 #include "stl.h"
 #include "position.h"
 
 class FicsProtocol : public Protocol {
  public:
   FicsProtocol();
-  ~FicsProtocol();
+  virtual ~FicsProtocol();
 
   void receiveString(char *netstring);
   void sendMove(int x1,int y1,int x2,int y2,int prom);
@@ -84,6 +85,7 @@ class FicsProtocol : public Protocol {
 
   void sendGameListEntry();
   void sendAdListEntry(char *pat);
+  void sendInitialization();
 
   char *knowRating(char *player);
   void clearRatingBuffer();
@@ -97,7 +99,15 @@ class FicsProtocol : public Protocol {
 
   void prepareBugPane();
 
+  void updateVar(ProtocolVar pv);
+
+  bool doAllOb1();
+  void doAllOb2();
+
+  friend gboolean fics_allob(gpointer data);
+
   int AuthenticationGone;
+  bool InitializationSent;
   bool LectureBotSpecial;
 
   PatternBinder Binder;
@@ -169,6 +179,12 @@ class FicsProtocol : public Protocol {
   ExtPatternMatcher SgAdd2;
   ExtPatternMatcher SgRemove;
 
+  ExtPatternMatcher AllObNone;
+  ExtPatternMatcher AllObFirstLine;
+  ExtPatternMatcher AllObMidLine;
+  int               AllObState;
+  char              AllObAcc[1024];
+
   int LastWasStyle12;
 
   char nick[64];
@@ -183,6 +199,7 @@ class FicsProtocol : public Protocol {
 
   list<Position> MoveBuf;
   deque<int> MoveRetrievals;
+  deque<int> AllObReqs;
   vector<string *> PActions, GActions;
 
   GameListConsumer *lastGLC, *lastALC;
@@ -192,7 +209,8 @@ class FicsProtocol : public Protocol {
   char xxrating[2][64];
   int xxnext;
 
-  int PartnerGame;
+  int  PartnerGame;
+  bool UserPlayingWithWhite; /* for won/lost sound event tracking */
 };
 
 #endif
