@@ -1,4 +1,4 @@
-/* $Id: proto_xboard.cc,v 1.25 2002/06/17 03:24:31 bergo Exp $ */
+/* $Id: proto_xboard.cc,v 1.36 2003/07/02 23:50:24 bergo Exp $ */
 
 /*
 
@@ -23,10 +23,15 @@
 
 */
 
-#include <iostream.h>
+#include <iostream>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
+#endif
+
 #include <unistd.h>
 #include <signal.h>
 #include <sys/types.h>
@@ -50,17 +55,21 @@ XBoardProtocol::XBoardProtocol() {
   timeout_id=-1;
   need_handshake=true; // send xboard/protover 2
 
-  got_init_pos=false;
+  got_init_pos = false;
+  just_sent_fen = false;
 
-  strcpy(ComputerName,"Computer");
+  timecontrol.setSecondsPerMove(20);
+
+  strcpy(ComputerName,_("Computer"));
   strcpy(EngineCommandLine,"gnuchess");
   strcpy(EngineRunDir,"~/.eboard/eng-out");
 
   Features.set("*feature*");
   IllegalMove.set("Illegal move*");
 
-  Moved[0].set("move *");
-  Moved[1].set("%n*... *");
+  Moved[0].set("move%b%r");
+  Moved[1].set("%N.%b...%b%r");
+  Moved[2].set("%N%b...%b%r");
 
   WhiteWin[0].set("1-0 {*}*");
   BlackWin[0].set("0-1 {*}*");
@@ -115,6 +124,7 @@ void XBoardProtocol::receiveString(char *netstring) {
   char *p;
   char a[64],b[64];
   int i,moved;
+  int wc,bc;
 
   p=xlateDialect(netstring);
   if (p)
@@ -127,37 +137,45 @@ void XBoardProtocol::receiveString(char *netstring) {
 
   if (IllegalMove.match(netstring)) {
 
-    if ((CurrentPosition.sidehint && EngineWhite) ||
-	(!CurrentPosition.sidehint && !EngineWhite) ) {
-      // engine sent illegal move string during HIS move
-      // happens when FEN string is sent to GNU Chess 5 and
-      // it doesn't understand the castling permissions
-      global.output->append("engine claimed illegal move but we didn't move, ignoring it.", global.Colors.TextBright, IM_IGNORE);
-      return;
-    }
+    if (just_sent_fen)
+      if ((CurrentPosition.sidehint && EngineWhite) ||
+	  (!CurrentPosition.sidehint && !EngineWhite) ) {
+	// engine sent illegal move string during HIS move
+	// happens when FEN string is sent to GNU Chess 5 and
+	// it doesn't understand the castling permissions
+	global.output->append(_("engine claimed illegal move but we didn't move, ignoring it."), global.Colors.TextBright, IM_IGNORE);
+	return;
+      }
 
     CurrentPosition=LegalityBackup;
     refgame=getGame();
     backMove();
     refgame->retreat(1);
+    
+    wc = bc = CLOCK_UNCHANGED;
+    if (timecontrol.mode == TC_SPM) {
+      if (EngineWhite) bc=0; else wc=0;
+    }
     refgame->updatePosition(CurrentPosition, MoveNumber,
 			    EngineWhite?1:0, 
-                            EngineWhite?0:CLOCK_UNCHANGED,
-			    EngineWhite?CLOCK_UNCHANGED:0,
-			    "illegal move!");
-    global.status->setText(netstring);
+			    wc,bc,
+			    _("illegal move!"));
+    global.status->setText(netstring,15);
     global.BoardList.front()->setCanMove(true);
     return;
   }
 
   moved=-1;
-  if (Moved[0].match(netstring)) moved=0;
-  if (Moved[1].match(netstring)) moved=1;
+  for(i=0;i<3;i++)
+    if (Moved[i].match(netstring)) {
+      moved = i;
+      break;
+    }
 
   if (moved>=0) {
     LegalityBackup=CurrentPosition;
-    CurrentPosition.SANstring(Moved[moved].getStarToken(moved),a);
-    CurrentPosition.moveAnyNotation(Moved[moved].getStarToken(moved),
+    CurrentPosition.SANstring(Moved[moved].getRToken(0),a);
+    CurrentPosition.moveAnyNotation(Moved[moved].getRToken(0),
                                     (MoveBlack?BLACK:WHITE),Variant);
     if (EngineWhite)
       sprintf(b,"%d. %s",MoveNumber,a);
@@ -165,14 +183,26 @@ void XBoardProtocol::receiveString(char *netstring) {
       sprintf(b,"%d. ... %s",MoveNumber,a);
 
     refgame=getGame();
-    refgame->updatePosition(CurrentPosition, MoveNumber, 
+
+    wc = bc = CLOCK_UNCHANGED;
+    if (timecontrol.mode == TC_ICS) {
+      if (MoveNumber == 1)
+	wc = bc = timecontrol.startValue();
+      else
+	refgame->incrementActiveClock(timecontrol.value[1]);
+    }    
+    if (timecontrol.mode == TC_SPM) {
+      if (EngineWhite) bc=0; else wc=0;
+    }
+
+    refgame->updatePosition(CurrentPosition, MoveNumber,
                             !MoveBlack, 
-			    EngineWhite?CLOCK_UNCHANGED:0,
-			    EngineWhite?0:CLOCK_UNCHANGED,
+			    wc, bc,
 			    b, true);
     advanceMove();
     global.BoardList.front()->setCanMove(true);
     global.opponentMoved();
+    just_sent_fen = false;
     return;
   }
 
@@ -213,6 +243,7 @@ void XBoardProtocol::sendMove(int x1,int y1,int x2,int y2,int prom) {
   char move[6];
   char mymove[12],ppfmm[18]; // [p]retty [p]rint [f]or [m]y [m]ove
   piece pp;
+  int wc, bc;
 
   refgame=getGame();
   LegalityBackup=CurrentPosition;
@@ -227,11 +258,21 @@ void XBoardProtocol::sendMove(int x1,int y1,int x2,int y2,int prom) {
   else
     sprintf(ppfmm,"%d. %s",MoveNumber,mymove);
 
+  wc = bc = CLOCK_UNCHANGED;
+  if (timecontrol.mode == TC_ICS) {
+    if (MoveNumber == 1)
+      wc = bc = timecontrol.startValue();
+    else
+      refgame->incrementActiveClock(timecontrol.value[1]);
+  }
+  if (timecontrol.mode == TC_SPM) {
+    if (EngineWhite) wc=0; else bc=0;
+  }
+
   refgame->updatePosition(CurrentPosition,
                           MoveNumber,
                           !MoveBlack,
-			  EngineWhite?0:CLOCK_UNCHANGED,
-			  EngineWhite?CLOCK_UNCHANGED:0,
+			  wc,bc,
 			  ppfmm);
   move[4]=0;
   move[5]=0;
@@ -267,7 +308,7 @@ void XBoardProtocol::finalize() {
 }
 
 void XBoardProtocol::gameOver(ExtPatternMatcher &pm,GameResult gr,
-                                    int hasreason=1)
+                                    int hasreason)
 {
   char reason[64];
   ChessGame *refgame;  
@@ -278,14 +319,22 @@ void XBoardProtocol::gameOver(ExtPatternMatcher &pm,GameResult gr,
   } else
     strcpy(reason,"Game over"); // I can't think of anything better...
 
+  if ( (gr==BLACK_WIN && EngineWhite) ||
+       (gr==WHITE_WIN && !EngineWhite) )
+    global.gameWon();
+
+  if ( (gr==WHITE_WIN && EngineWhite) ||
+       (gr==BLACK_WIN && !EngineWhite) )
+    global.gameLost();
+
   refgame=getGame();
   refgame->endGame(reason,gr);
 
   if (global.AppendPlayed) {
     if  (refgame->savePGN(global.AppendFile,true)) {
       char z[128];
-      sprintf(z,"Game appended to %s",global.AppendFile);
-      global.status->setText(z);
+      sprintf(z,_("Game appended to %s"),global.AppendFile);
+      global.status->setText(z,10);
     }
   }
 
@@ -294,28 +343,52 @@ void XBoardProtocol::gameOver(ExtPatternMatcher &pm,GameResult gr,
 
 void XBoardProtocol::resign() {
   global.network->writeLine("resign");
-  global.status->setText("Resigned.");
+  global.status->setText(_("Resigned."),10);
 }
 
 void XBoardProtocol::draw() {
   global.network->writeLine("draw");
-  global.status->setText("<XBoard Protocol> draw request sent");
+  global.status->setText(_("<XBoard Protocol> draw request sent"),20);
 }
 
 void XBoardProtocol::adjourn() {
-  global.status->setText("<XBoard Protocol> Adjourning not supported");
+  global.status->setText(_("<XBoard Protocol> Adjourning not supported"),10);
 }
 
 void XBoardProtocol::abort() {
   finalize();
-  global.status->setText("<XBoard Protocol> Session Aborted");
+  global.status->setText(_("<XBoard Protocol> Session Aborted"),15);
+}
+
+void XBoardProtocol::retractMove() {
+  ChessGame *ref;
+  ref = getGame();
+  if (ref && MoveNumber>1) {
+    if ((!MoveBlack && EngineWhite)||
+	(MoveBlack && !EngineWhite)) {
+      global.status->setText(_("You can only retract when it's your turn to move."),15);
+    } else {
+      Position p;
+      char z[64];
+      ref->retreat(2);
+      p = ref->getLastPosition();
+      --MoveNumber;
+      memset(z,0,64);
+      p.getLastMove().copy(z,63);
+      ref->updatePosition(p,MoveNumber,MoveBlack,
+			  CLOCK_UNCHANGED, CLOCK_UNCHANGED,z);
+      global.network->writeLine("remove");
+      global.status->setText(_("Retracted last move."),15);
+      CurrentPosition = p;
+    }
+  }
 }
 
 ChessGame * XBoardProtocol::getGame() {
   ChessGame *refgame;
   refgame=global.getGame(XBOARD_GAME);
   if (!refgame) {
-    cerr << "* game not found: " << XBOARD_GAME << endl;
+    cerr << _("* game not found: ") << XBOARD_GAME << endl;
     exit(5);
   }
   return refgame;
@@ -350,7 +423,7 @@ gboolean xb_initengine_timeout(gpointer data) {
   XBoardProtocol *me;
   me=(XBoardProtocol *) data;
 
-  global.status->setText("Initializing engine");
+  global.status->setText(_("Initializing engine"),30);
 
   me->initEngine();
   return FALSE;
@@ -372,7 +445,7 @@ int XBoardProtocol::loadEngine() {
     link->setHandshake("xboard\nprotover 2\n");
 
   if (link->open()==0) { 
-    global.status->setText("Engine started (2 sec timeout)");
+    global.status->setText(_("Engine started (2 sec timeout)"),30);
     global.network=link;
 
     // run initEngine after 2 seconds
@@ -386,7 +459,7 @@ int XBoardProtocol::loadEngine() {
     }
   }
 
-  global.status->setText("Failed to run engine.");
+  global.status->setText(_("Failed to run engine."),30);
   delete link;
   return 0;
 }
@@ -404,13 +477,12 @@ void XBoardProtocol::initEngine() {
 
   global.network->writeLine("new");
 
-  if (SecsPerMove > 0) {
-    sprintf(z,"st %d",SecsPerMove);
+  timecontrol.toXBoard(z);
+  if (z[0])
     global.network->writeLine(z);
-  }
 
   if (MaxDepth > 0) {
-    sprintf(z,"sd %d",SecsPerMove);
+    sprintf(z,"sd %d",MaxDepth);
     global.network->writeLine(z);
   }
 
@@ -420,8 +492,8 @@ void XBoardProtocol::initEngine() {
 
 void XBoardProtocol::endInit() {
   char z[256];
-  sprintf(z,"%s engine started.", ComputerName);
-  global.status->setText(z);
+  sprintf(z,_("%s engine started."), ComputerName);
+  global.status->setText(z,15);
 }
 
 void XBoardProtocol::lateInit() {
@@ -463,6 +535,7 @@ void XBoardProtocol::setupPosition() {
     s+=initpos.getFEN();
     strcpy(z,s.c_str());
     global.network->writeLine(z);    
+    just_sent_fen = true;
 
   } else {
     char assoc[128];
@@ -500,6 +573,7 @@ void XBoardProtocol::setupPosition() {
       }
 
     global.network->writeLine(".");
+    just_sent_fen = true;
   }
 
 }
@@ -520,9 +594,9 @@ void XBoardProtocol::createGame() {
 
   cg->MyColor=EngineWhite?BLACK:WHITE;
 
-  sprintf(cg->info0,"%d seconds per move",SecsPerMove);
+  timecontrol.toString(cg->info0);
 
-  cg->clock_regressive=0;
+  cg->clock_regressive=timecontrol.isRegressive()?1:0;
   cg->Rated=0;
   cg->Variant=Variant;
   global.appendGame(cg);
@@ -535,7 +609,8 @@ void XBoardProtocol::createGame() {
   if (EngineWhite)
     global.BoardList.front()->setFlipped(true);
   cg->acknowledgeInfo();
-  cg->fireWhiteClock(0,0);
+  cg->fireWhiteClock(timecontrol.startValue(),
+		     timecontrol.startValue());
 
   pos=new Position();
 
@@ -546,8 +621,13 @@ void XBoardProtocol::createGame() {
     MoveBlack = pos->sidehint ? 0 : 1;
   }
 
-  cg->updatePosition(*pos,1,pos->sidehint ? 0:1,0,0,"0. startpos");
+  cg->updatePosition(*pos,1,pos->sidehint ? 0:1,
+		     timecontrol.startValue(),
+		     timecontrol.startValue(),
+		     "0. startpos");
   delete pos;
+
+  global.gameStarted();
 }
 
 void XBoardProtocol::dumpGame() {
@@ -578,7 +658,7 @@ void XBoardProtocol::parseFeatures() {
       if (!strcmp(p,"myname")) {
 	ComputerName[63]=0;
 	strncpy(ComputerName,sf.getStarToken(0),64);
-	global.status->setText(ComputerName);
+	global.status->setText(ComputerName,10);
 	cg=global.getGame(XBOARD_GAME);
 	if (cg) {
 	  strncpy(cg->PlayerName[EngineWhite?0:1],ComputerName,64);
@@ -602,7 +682,7 @@ void XBoardProtocol::parseFeatures() {
 	  gotdone=true;
 	else {
 	  // engine asked more time
-	  global.status->setText("Engine asked more time to startup, waiting.");
+	  global.status->setText(_("Engine asked more time to startup, waiting."),120);
 	  if (timeout_id >= 0)
 	    gtk_timeout_remove(timeout_id);
 	  
@@ -622,13 +702,13 @@ void XBoardProtocol::parseFeatures() {
   if (gotdone && timeout_id >= 0) {
     gtk_timeout_remove(timeout_id);
     timeout_id=-1;
-    global.status->setText("Engine loaded.");
+    global.status->setText(_("Engine loaded."),20);
     initEngine();
   }
 }
 
 char * XBoardProtocol::getDialogName() {
-  return("Play against Engine");
+  return(_("Play against Engine"));
 }
 
 char * XBoardProtocol::getComputerName() {
@@ -640,11 +720,12 @@ char * XBoardProtocol::getComputerName() {
 void XBoardProtocol::createDialog() {
   GtkWidget *v,*v2;
   GtkWidget *cfr,*rd[2],*ok,*cancel,*bhb,*hs;
-  GtkWidget *f2,*tabl,*l2,*spm,*tl1,*tl2,*te1,*tv;
+  GtkWidget *f2,*tabl,*l2,*tch,*tce,*tl1,*tl2,*te1,*tv;
   GSList *rg;  
+  char z[64];
 
-  GtkWidget *bp_l,*bp_v, *pp_l, *pp_v;
-  GtkWidget *h3,*l3,*path3,*b31,*l31,*h4,*l4,*path4;
+  GtkWidget *bp_l,*bp_v, *pp_l=0, *pp_v=0;
+  GtkWidget *h3=0,*l3=0,*path3,*b31,*l31,*h4=0,*l4=0,*path4;
 
   eng_dlg=gtk_window_new(GTK_WINDOW_DIALOG);
   gtk_window_set_title(GTK_WINDOW(eng_dlg),getDialogName());
@@ -660,76 +741,71 @@ void XBoardProtocol::createDialog() {
   
   // basic pane
   bp_v=gtk_vbox_new(FALSE,0);
-  bp_l=gtk_label_new("Side & Time");
+  bp_l=gtk_label_new(_("Side & Time"));
   gtk_container_set_border_width(GTK_CONTAINER(bp_v),4);
-  cfr=gtk_frame_new("Side Selection");
+  cfr=gtk_frame_new(_("Side Selection"));
   gtk_frame_set_shadow_type(GTK_FRAME(cfr),GTK_SHADOW_ETCHED_IN);
   gtk_box_pack_start(GTK_BOX(bp_v),cfr,FALSE,TRUE,4);
   v2=gtk_vbox_new(TRUE,2);
   gtk_container_add(GTK_CONTAINER(cfr),v2);
-  rd[0]=gtk_radio_button_new_with_label( 0, "Human White vs. Computer Black" );
-  rg=gtk_radio_button_group(GTK_RADIO_BUTTON(rd[0]));
-  rd[1]=gtk_radio_button_new_with_label(rg, "Computer White vs. Human Black" );
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(rd[0]), TRUE);
+  rd[0]=grnew( 0, _("Human White vs. Computer Black") );
+  rg=grg(GTK_RADIO_BUTTON(rd[0]));
+  rd[1]=grnew(rg, _("Computer White vs. Human Black") );
+  gtset(GTK_TOGGLE_BUTTON(rd[0]), TRUE);
   gtk_box_pack_start(GTK_BOX(v2),rd[0],FALSE,TRUE,2);
   gtk_box_pack_start(GTK_BOX(v2),rd[1],FALSE,TRUE,2);
   // time per move
-  f2=gtk_frame_new("Time Control");
+  f2=gtk_frame_new(_("Time Control"));
   gtk_frame_set_shadow_type(GTK_FRAME(f2),GTK_SHADOW_ETCHED_IN);
   gtk_box_pack_start(GTK_BOX(bp_v),f2,FALSE,TRUE,4);  
   tv=gtk_vbox_new(FALSE,2);
   gtk_container_add(GTK_CONTAINER(f2),tv);
-  tl1=gtk_label_new("Set any of these fields to zero to\nuse the engine's default value.");
-  gtk_box_pack_start(GTK_BOX(tv),tl1,FALSE,FALSE,2);
-  tabl=gtk_table_new(3,2,FALSE);
+
+  tch = gtk_hbox_new(FALSE,2);
+  gtk_box_pack_start(GTK_BOX(tv),tch,TRUE,TRUE,0);
+
+  timecontrol.toString(z);
+  l2=gtk_label_new(z);  
+  gtk_box_pack_start(GTK_BOX(tch),l2,TRUE,TRUE,0);
+  
+  tce = gtk_button_new_with_label(_("Time Control..."));
+  gtk_box_pack_start(GTK_BOX(tch),tce,FALSE,TRUE,0);
+
+  tabl=gtk_table_new(2,2,FALSE);
   gtk_container_set_border_width(GTK_CONTAINER(tabl),4);
   gtk_table_set_row_spacings(GTK_TABLE(tabl),2);
   gtk_table_set_col_spacings(GTK_TABLE(tabl),2);
   gtk_box_pack_start(GTK_BOX(tv),tabl,FALSE,FALSE,2);
-  l2=gtk_label_new("Seconds per move:");
-  spm=gtk_entry_new();
-  gtk_entry_set_text(GTK_ENTRY(spm),"20");
-  gtk_table_attach_defaults(GTK_TABLE(tabl),l2,0,1,0,1);
-  gtk_table_attach_defaults(GTK_TABLE(tabl),spm,1,2,0,1);
-  tl2=gtk_label_new("Depth Limit:");
+
+  tl2=gtk_label_new(_("Depth Limit:"));
   te1=gtk_entry_new();
   gtk_entry_set_text(GTK_ENTRY(te1),"0");
-  gtk_table_attach_defaults(GTK_TABLE(tabl),tl2,0,1,1,2);
-  gtk_table_attach_defaults(GTK_TABLE(tabl),te1,1,2,1,2);
+  gtk_table_attach_defaults(GTK_TABLE(tabl),tl2,0,1,0,1);
+  gtk_table_attach_defaults(GTK_TABLE(tabl),te1,1,2,0,1);
 
-  think=gtk_check_button_new_with_label("Think on opponent's time");
-  gtk_table_attach_defaults(GTK_TABLE(tabl),think,0,2,2,3);
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(think), 1);
+  tl1=gtk_label_new(_("Set depth limit to 0 to use the engine's default."));
+  gtk_box_pack_start(GTK_BOX(tv),tl1,FALSE,FALSE,2);
 
-  bookmark=gtk_check_button_new_with_label("Add to Peer/Engine Bookmarks menu");
+  think=gtk_check_button_new_with_label(_("Think on opponent's time"));
+  gtk_table_attach_defaults(GTK_TABLE(tabl),think,0,2,1,2);
+  gtset(GTK_TOGGLE_BUTTON(think), 1);
+
+  bookmark=gtk_check_button_new_with_label(_("Add to Peer/Engine Bookmarks menu"));
   gtk_box_pack_start(GTK_BOX(v),bookmark,FALSE,FALSE,2);
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(bookmark), 1);
+  gtset(GTK_TOGGLE_BUTTON(bookmark), 1);
 
-  gtk_widget_show(bookmark);
-  gtk_widget_show(think);
-  gtk_widget_show(te1);
-  gtk_widget_show(tl2);
-  gtk_widget_show(tl1);
-  gtk_widget_show(tabl);
-  gtk_widget_show(tv);
-  gtk_widget_show(f2);
-  gtk_widget_show(l2);
-  gtk_widget_show(spm);
-  gtk_widget_show(rd[1]);
-  gtk_widget_show(rd[0]);
-  gtk_widget_show(v2);
-  gtk_widget_show(cfr);
+  Gtk::show(bookmark,think,te1,tl2,tl1,tabl,tv,f2,tch,l2,tce,rd[1],rd[0],
+	    v2,cfr,0);
   gtk_notebook_append_page(GTK_NOTEBOOK(eng_book),bp_v,bp_l);
-  gtk_widget_show(bp_v);
-  gtk_widget_show(bp_l);
+  Gtk::show(bp_v,bp_l,0);
 
   // path pane
   if (want_path_pane) {
     pp_v=gtk_vbox_new(FALSE,0);
-    pp_l=gtk_label_new("Engine Command");
+    pp_l=gtk_label_new(_("Engine Command"));
     gtk_container_set_border_width(GTK_CONTAINER(pp_v),4);
     h3=gtk_hbox_new(FALSE,0);
-    l3=gtk_label_new("Engine command line");
+    l3=gtk_label_new(_("Engine command line"));
     gtk_box_pack_start(GTK_BOX(pp_v),h3,FALSE,FALSE,2);
     gtk_box_pack_start(GTK_BOX(h3),l3,FALSE,FALSE,0);
   }
@@ -738,7 +814,7 @@ void XBoardProtocol::createDialog() {
   if (want_path_pane) {
     gtk_box_pack_start(GTK_BOX(pp_v),path3,FALSE,TRUE,2);
     h4=gtk_hbox_new(FALSE,0);
-    l4=gtk_label_new("Directory to run from (e.g., where book files are)");
+    l4=gtk_label_new(_("Directory to run from (e.g., where book files are)"));
     gtk_box_pack_start(GTK_BOX(pp_v),h4,FALSE,FALSE,2);
     gtk_box_pack_start(GTK_BOX(h4),l4,FALSE,FALSE,0);
   }
@@ -747,21 +823,14 @@ void XBoardProtocol::createDialog() {
   if (want_path_pane) {
     gtk_box_pack_start(GTK_BOX(pp_v),path4,FALSE,TRUE,2);
     b31=gtk_hbox_new(FALSE,0);
-    l31=gtk_label_new("The engine will be run with\n/bin/sh -c 'cd directory ; command line'");
+    l31=gtk_label_new(_("The engine will be run with\n/bin/sh -c 'cd directory ; command line'"));
     gtk_label_set_justify(GTK_LABEL(l31),GTK_JUSTIFY_LEFT);
     gtk_box_pack_start(GTK_BOX(b31),l31,FALSE,FALSE,2);
     gtk_box_pack_start(GTK_BOX(pp_v),b31,FALSE,FALSE,2);
-    gtk_widget_show(h3);
-    gtk_widget_show(l3);
-    gtk_widget_show(l31);
-    gtk_widget_show(b31);
-    gtk_widget_show(path3);
-    gtk_widget_show(h4);
-    gtk_widget_show(l4);
-    gtk_widget_show(path4);
+
+    Gtk::show(h3,l3,l31,b31,path3,h4,l4,path4,0);
     gtk_notebook_append_page(GTK_NOTEBOOK(eng_book),pp_v,pp_l);
-    gtk_widget_show(pp_v);
-    gtk_widget_show(pp_l);
+    Gtk::show(pp_v,pp_l,0);
   }
 
   // bottom box
@@ -772,18 +841,15 @@ void XBoardProtocol::createDialog() {
   gtk_button_box_set_layout(GTK_BUTTON_BOX(bhb), GTK_BUTTONBOX_END);
   gtk_button_box_set_spacing(GTK_BUTTON_BOX(bhb), 5);
   gtk_box_pack_start(GTK_BOX(v),bhb,FALSE,FALSE,4);
-  ok=gtk_button_new_with_label    ("OK");
+  ok=gtk_button_new_with_label    (_("OK"));
   GTK_WIDGET_SET_FLAGS(ok,GTK_CAN_DEFAULT);
-  cancel=gtk_button_new_with_label("Cancel");
+  cancel=gtk_button_new_with_label(_("Cancel"));
   GTK_WIDGET_SET_FLAGS(cancel,GTK_CAN_DEFAULT);
   gtk_box_pack_start(GTK_BOX(bhb),ok,TRUE,TRUE,0);
   gtk_box_pack_start(GTK_BOX(bhb),cancel,TRUE,TRUE,0);
   gtk_widget_grab_default(ok);
-  gtk_widget_show(ok);
-  gtk_widget_show(cancel);
-  gtk_widget_show(bhb);
-  gtk_widget_show(hs);
-  gtk_widget_show(v);
+
+  Gtk::show(ok,cancel,bhb,hs,v,0);
 
   gtk_signal_connect(GTK_OBJECT(ok), "clicked",
                      GTK_SIGNAL_FUNC(xboard_eng_ok),this);
@@ -792,10 +858,14 @@ void XBoardProtocol::createDialog() {
   gtk_signal_connect(GTK_OBJECT(eng_dlg), "delete_event",
                      GTK_SIGNAL_FUNC(xboard_eng_delete),0);
 
-  gtk_widget_show(eng_book);
+  gtk_signal_connect(GTK_OBJECT(tce), "clicked",
+                     GTK_SIGNAL_FUNC(xboard_edit_time),
+		     (XBoardProtocol *) this);
+
+  gshow(eng_book);
   gtk_notebook_set_page(GTK_NOTEBOOK(eng_book),0);
 
-  engctl_spm=spm;
+  engctl_lbl=l2;
   engctl_ply=te1;
   engctl_ewhite=rd[1];
   engctl_engcmd=path3;
@@ -803,21 +873,27 @@ void XBoardProtocol::createDialog() {
 
   if (got_init_pos) {
     if (initpos.sidehint) // white to move
-      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(rd[1]), TRUE);
+      gtset(GTK_TOGGLE_BUTTON(rd[1]), TRUE);
     else
-      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(rd[0]), TRUE);
+      gtset(GTK_TOGGLE_BUTTON(rd[0]), TRUE);
 
     gtk_widget_set_sensitive(rd[0],FALSE);
     gtk_widget_set_sensitive(rd[1],FALSE);
   }
 }
 
+void XBoardProtocol::update() {
+  char z[64];
+  timecontrol.toString(z);
+  gtk_label_set_text(GTK_LABEL(engctl_lbl), z);
+  gtk_widget_queue_resize(engctl_lbl);
+}
+
 void XBoardProtocol::readDialog() {
 
   EngineWhite=0;
   if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(engctl_ewhite)))
     EngineWhite=1;
-  SecsPerMove=atoi(gtk_entry_get_text(GTK_ENTRY(engctl_spm)));
   MaxDepth=atoi(gtk_entry_get_text(GTK_ENTRY(engctl_ply)));
   strncpy(EngineCommandLine,gtk_entry_get_text(GTK_ENTRY(engctl_engcmd)),512);
   strncpy(EngineRunDir,gtk_entry_get_text(GTK_ENTRY(engctl_engdir)),256);
@@ -825,14 +901,14 @@ void XBoardProtocol::readDialog() {
 
   if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(bookmark))) {
     ebm=new EngineBookmark();
-    ebm->secspermove=SecsPerMove;
-    ebm->maxply     =MaxDepth;
-    ebm->humanwhite =1-EngineWhite;
-    ebm->think      =ThinkAlways?1:0;
-    ebm->directory  =EngineRunDir;
-    ebm->cmdline    =EngineCommandLine;
-    ebm->mode       =REGULAR;
-    ebm->proto      =0; // base xboard protocol
+    ebm->timecontrol =timecontrol;
+    ebm->maxply      =MaxDepth;
+    ebm->humanwhite  =1-EngineWhite;
+    ebm->think       =ThinkAlways?1:0;
+    ebm->directory   =EngineRunDir;
+    ebm->cmdline     =EngineCommandLine;
+    ebm->mode        =REGULAR;
+    ebm->proto       =0; // base xboard protocol
 
     makeBookmarkCaption();
 
@@ -847,7 +923,7 @@ void XBoardProtocol::readDialog() {
 void XBoardProtocol::makeBookmarkCaption() {
   tstring t,u;
   string x, *p;
-  char caption[1024];
+  char caption[1024], z[128];
 
   if (!ebm) return;
 
@@ -862,18 +938,19 @@ void XBoardProtocol::makeBookmarkCaption() {
       x=*p;
   }
   
-  sprintf(caption,"Play %s as %s vs. %s (%d secs/move, maxdepth %d, think always: %s)",
+  ebm->timecontrol.toString(z);
+  sprintf(caption,_("Play %s as %s vs. %s (%s, maxdepth %d, think always: %s)"),
 	  ChessGame::variantName(ebm->mode),
-	  ebm->humanwhite?"White":"Black",
+	  ebm->humanwhite?_("White"):_("Black"),
 	  x.c_str(),
-	  ebm->secspermove,
+	  z,
 	  ebm->maxply,
-	  ebm->think?"yes":"no");
+	  ebm->think?_("yes"):_("no"));
   ebm->caption = caption;
 }
 
 void XBoardProtocol::loadBookmark(EngineBookmark *bm) {
-  SecsPerMove = bm->secspermove;
+  timecontrol = bm->timecontrol;
   MaxDepth    = bm->maxply;
   ThinkAlways = (bm->think!=0);
   EngineWhite = 1-bm->humanwhite;
@@ -887,7 +964,7 @@ void XBoardProtocol::loadBookmark(EngineBookmark *bm) {
 
 int XBoardProtocol::runDialog() {
   GotResult=0;
-  gtk_widget_show(eng_dlg);
+  gshow(eng_dlg);
   gtk_grab_add(eng_dlg);
   while(!GotResult)
     global.WrappedMainIteration();
@@ -899,6 +976,15 @@ void XBoardProtocol::destroyDialog() {
   gtk_widget_destroy(eng_dlg);
 }
 
+void xboard_edit_time(GtkWidget *w,gpointer data) {
+  XBoardProtocol *me = (XBoardProtocol *) data;
+  TimeControlEditDialog *dlg;
+
+  dlg = new TimeControlEditDialog(&(me->timecontrol));
+  dlg->setUpdateListener( (UpdateInterface *) me);
+  dlg->show();
+}
+
 void xboard_eng_ok(GtkWidget *w,gpointer data) {
   XBoardProtocol *me;
   me=(XBoardProtocol *)data;
@@ -914,3 +1000,260 @@ void xboard_eng_cancel(GtkWidget *w,gpointer data) {
 gboolean xboard_eng_delete(GtkWidget *w,GdkEvent *e,gpointer data) {
   return TRUE;
 }
+
+
+/* ************************************************** */
+/* *********** specific engines ********************* */
+/* ************************************************** */
+
+// =============== CRAFTY ============================
+
+CraftyProtocol::CraftyProtocol() : XBoardProtocol() {
+  strcpy(ComputerName,"Crafty");
+  want_path_pane=0;
+}
+
+char * CraftyProtocol::getDialogName() {
+  return(_("Play against Crafty"));
+}
+
+void CraftyProtocol::readDialog() {
+  XBoardProtocol::readDialog();
+  resolvePaths();
+  sprintf(EngineCommandLine,"crafty bookpath=%s logpath=%s tbpath=%s",
+	  BookPath,LogPath,LogPath);
+  if (!global.env.Home.empty())
+    sprintf(EngineRunDir,"%s/.eboard/craftylog",global.env.Home.c_str());
+  else
+    strcpy(EngineRunDir,"/tmp");
+
+  if (ebm) {
+    ebm->proto     = 1;
+    ebm->cmdline   = EngineCommandLine;
+    ebm->directory = EngineRunDir;
+    makeBookmarkCaption();
+  }
+}
+
+void CraftyProtocol::initEngine() {
+  global.network->writeLine("log off");
+  XBoardProtocol::initEngine();
+}
+
+// keep this and Documentation/Crafty.txt coherent
+void CraftyProtocol::resolvePaths() {
+  FileFinder ff;
+  char z[256],zz[256],*p;
+
+  // book path
+  if (!global.env.Home.empty())
+    sprintf(z,"%s/.eboard",global.env.Home.c_str());
+  else
+    strcpy(z,"/tmp");
+
+  ff.addMyDirectory(".eboard");
+  ff.addMyDirectory(".");
+  ff.addDirectory("/usr/local/share/eboard");
+  ff.addDirectory("/usr/share/eboard");
+  ff.addDirectory("/usr/local/share/crafty");
+  ff.addDirectory("/usr/share/crafty");
+
+  if (ff.find("book.bin",zz)) {
+    p=rindex(zz,'/');
+    *p=0;
+  } else
+    strcpy(zz,z);
+
+  strcpy(BookPath,zz);
+
+  // log path
+  if (!global.env.Home.empty())
+    sprintf(z,"%s/.eboard/craftylog",global.env.Home.c_str());
+  else
+    strcpy(z,"/tmp");
+  strcpy(LogPath,z);
+}
+
+// =============== SJENG ============================
+
+SjengProtocol::SjengProtocol() : XBoardProtocol() {
+  strcpy(ComputerName,"Sjeng");
+  strcpy(EngineCommandLine,"sjeng");
+  want_path_pane=1;
+}
+
+void SjengProtocol::initEngine() {
+  char z[32];
+  createGame();
+  global.network->writeLine("xboard");
+  global.network->writeLine("protover 2");
+  global.network->writeLine("nopost");
+
+  if (ThinkAlways)
+    global.network->writeLine("hard");
+  else
+    global.network->writeLine("easy");
+
+  global.network->writeLine("new");
+
+  switch(Variant) {
+  case CRAZYHOUSE: global.network->writeLine("variant crazyhouse"); break;
+  case SUICIDE:    global.network->writeLine("variant suicide"); break;
+  case LOSERS:     global.network->writeLine("variant losers"); break;
+  case GIVEAWAY:   global.network->writeLine("variant giveaway"); break;
+  default: z[31]=0; /* nothing! */ break;
+  }
+
+  timecontrol.toXBoard(z);
+  if (z[0])
+    global.network->writeLine(z);
+
+  if (MaxDepth>0) {
+    sprintf(z,"sd %d",MaxDepth);
+    global.network->writeLine(z);
+  }
+
+  lateInit();
+  endInit();
+}
+
+void SjengProtocol::createDialog() {
+  int i;
+  GtkWidget *fr,*v,*rd[5],*vl;
+  GSList *rg;
+
+  XBoardProtocol::createDialog();
+
+  // add variant pane
+  fr=gtk_frame_new(_("Variant"));
+  gtk_frame_set_shadow_type(GTK_FRAME(fr),GTK_SHADOW_ETCHED_IN);
+  v=gtk_vbox_new(TRUE,2);
+  gtk_container_add(GTK_CONTAINER(fr),v);
+
+  rd[0]=grnew( 0, _("Normal Chess") );
+  rg=grg(GTK_RADIO_BUTTON(rd[0]));
+  rd[1]=grnew(rg, _("Crazyhouse") );
+  rg=grg(GTK_RADIO_BUTTON(rd[1]));
+  rd[2]=grnew(rg, _("Suicide") );
+  rg=grg(GTK_RADIO_BUTTON(rd[2]));
+  rd[3]=grnew(rg, _("Losers") );
+  rg=grg(GTK_RADIO_BUTTON(rd[3]));
+  rd[4]=grnew(rg, _("Giveaway") );
+
+  gtset(GTK_TOGGLE_BUTTON(rd[0]), TRUE);
+  for(i=0;i<5;i++) {
+    gtk_box_pack_start(GTK_BOX(v),rd[i],FALSE,TRUE,2);
+    gshow(rd[i]);
+    varbutton[i]=rd[i];
+  }
+  Gtk::show(v,fr,0);
+  vl=gtk_label_new(_("Variant"));
+  gshow(vl);
+  gtk_notebook_append_page(GTK_NOTEBOOK(eng_book),fr,vl);
+  gtk_notebook_set_page(GTK_NOTEBOOK(eng_book),0);
+}
+
+void SjengProtocol::readDialog() {
+  XBoardProtocol::readDialog();
+  if (gtget(GTK_TOGGLE_BUTTON(varbutton[0]))) Variant=REGULAR;
+  if (gtget(GTK_TOGGLE_BUTTON(varbutton[1]))) Variant=CRAZYHOUSE;
+  if (gtget(GTK_TOGGLE_BUTTON(varbutton[2]))) Variant=SUICIDE;
+  if (gtget(GTK_TOGGLE_BUTTON(varbutton[3]))) Variant=LOSERS;
+  if (gtget(GTK_TOGGLE_BUTTON(varbutton[4]))) Variant=GIVEAWAY;
+
+  if (ebm) {
+    ebm->proto=2;
+    ebm->mode=Variant;
+
+    makeBookmarkCaption();
+  }
+}
+
+char * SjengProtocol::getDialogName() {
+  return(_("Play against Sjeng"));
+}
+
+void SjengProtocol::sendDrop(piece p,int x,int y) {
+  ChessGame *refgame;
+  char mymove[12],ppfmm[18]; // [p]retty [p]rint [f]or [m]y [m]ove
+  piece clp;
+
+  refgame=getGame();
+  LegalityBackup=CurrentPosition;
+
+  sprintf(mymove,"P@%c%c",'a'+x,'1'+y);
+  clp=p&PIECE_MASK;
+  switch(clp) {
+  case PAWN:    mymove[0]='P'; break;
+  case ROOK:    mymove[0]='R'; break;
+  case KNIGHT:  mymove[0]='N'; break;
+  case BISHOP:  mymove[0]='B'; break;
+  case QUEEN:   mymove[0]='Q'; break;
+  case KING:    mymove[0]='K'; break;
+  }
+  CurrentPosition.moveDrop(p|(refgame->MyColor),x,y);
+
+  if (EngineWhite)
+    sprintf(ppfmm,"%d. ... %s",MoveNumber,mymove);
+  else
+    sprintf(ppfmm,"%d. %s",MoveNumber,mymove);
+
+  refgame->updatePosition(CurrentPosition, 
+                          MoveNumber,!MoveBlack,
+			  CLOCK_UNCHANGED,
+			  CLOCK_UNCHANGED,
+			  ppfmm);
+
+  global.network->writeLine(mymove);
+  advanceMove();
+  global.BoardList.front()->setCanMove(false);
+}
+
+// ==================== GNU CHESS 4 =========================
+
+GnuChess4Protocol::GnuChess4Protocol() : XBoardProtocol() {
+  strcpy(ComputerName,"GNU Chess");
+  want_path_pane=0;
+  need_handshake=false; // sending xboard confuses GNU chess 4
+}
+
+void GnuChess4Protocol::initEngine() {
+  char z[32];
+  createGame();
+  global.network->writeLine("nopost");
+
+  global.network->writeLine("hard");
+  if (!ThinkAlways)
+    global.network->writeLine("easy");
+
+  global.network->writeLine("new");
+
+  timecontrol.toXBoard(z,TCF_GNUCHESS4QUIRK);
+  if (z[0])
+    global.network->writeLine(z);
+
+  if (MaxDepth > 0) {
+    sprintf(z,"depth %d",MaxDepth);
+    global.network->writeLine(z);
+  }
+
+  lateInit();
+  endInit();
+}
+
+void GnuChess4Protocol::readDialog() {
+  XBoardProtocol::readDialog();
+  strcpy(EngineCommandLine,"gnuchessx");
+  EngineRunDir[0]=0;
+
+  if (ebm) {
+    ebm->proto=3;
+    ebm->cmdline=EngineCommandLine;
+    ebm->directory=EngineRunDir;
+    makeBookmarkCaption();
+  }
+}
+
+char * GnuChess4Protocol::getDialogName() {
+  return(_("Play against GNU Chess 4"));
+}
