@@ -1,10 +1,10 @@
-/* $Id: seekgraph.cc,v 1.9 2002/02/13 03:53:56 bergo Exp $ */
+/* $Id: seekgraph.cc,v 1.16 2003/07/06 18:17:36 bergo Exp $ */
 
 /*
 
     eboard - chess client
     http://eboard.sourceforge.net
-    Copyright (C) 2000-2002 Felipe Paulo Guazzi Bergo
+    Copyright (C) 2000-2003 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -23,12 +23,13 @@
 
 */
 
-#include <iostream.h>
+#include <iostream>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include "seekgraph.h"
 #include "global.h"
+#include "eboard.h"
 
 static void skg_refresh(GtkWidget *w, gpointer data) {
   if (global.protocol)
@@ -121,7 +122,7 @@ const char ** SeekAd::getListLine() {
   ll[3]=flags.c_str();
   ll[4]=z[1];
   ll[5]=kind.c_str();
-  ll[6]=rated?"rated":"unrated";
+  ll[6]=rated?_("rated"):_("unrated");
   ll[7]=color.c_str();
   ll[8]=range.c_str();
 
@@ -134,8 +135,8 @@ const char ** SeekAd::getListLine() {
 
 SeekGraph::SeekGraph() {
   static char * skg_col_titles[10] = {
-    "Id","Player","Rating","Flags","Time","Variant","Rated",
-    "Color","Rating Range","Manual/Formula" };
+    N_("Id"),N_("Player"),N_("Rating"),N_("Flags"),N_("Time"),N_("Variant"),N_("Rated"),
+    N_("Color"),N_("Rating Range"),N_("Manual/Formula") };
 
   GtkWidget *sw,*h,*pb, *rb;
   int i;
@@ -156,7 +157,8 @@ SeekGraph::SeekGraph() {
   // 0  1    2      3      4           5    6     7     8     9
 
   for(i=0;i<10;i++)
-    gtk_clist_set_column_title(GTK_CLIST(clist),i,skg_col_titles[i]);
+    gtk_clist_set_column_title(GTK_CLIST(clist),i,
+			       eboard_gettext(skg_col_titles[i]));
 
   gtk_clist_column_titles_active(GTK_CLIST(clist));
   gtk_clist_column_titles_show(GTK_CLIST(clist));
@@ -169,6 +171,8 @@ SeekGraph::SeekGraph() {
                                  GTK_POLICY_AUTOMATIC,
                                  GTK_POLICY_AUTOMATIC);
 
+  updateFont();
+
   gtk_container_add(GTK_CONTAINER(sw),clist);  
 
   for(i=0;i<10;i++)
@@ -176,21 +180,17 @@ SeekGraph::SeekGraph() {
   gtk_clist_columns_autosize(GTK_CLIST(clist));
 
   gtk_box_pack_start(GTK_BOX(widget),sw,TRUE,TRUE,0);
-  gtk_widget_show(sw);
-  gtk_widget_show(clist);
+  Gtk::show(sw,clist,0);
 
   h=gtk_hbox_new(FALSE,4);
-  pb=gtk_button_new_with_label(" Play Selected ");
-  rb=gtk_button_new_with_label(" Refresh ");
+  pb=gtk_button_new_with_label(_(" Play Selected "));
+  rb=gtk_button_new_with_label(_(" Refresh "));
   gtk_box_pack_start(GTK_BOX(widget),h,FALSE,FALSE,4);
   gtk_box_pack_end(GTK_BOX(h),pb,FALSE,FALSE,4);
   gtk_box_pack_end(GTK_BOX(h),rb,FALSE,FALSE,4);
   gtk_widget_set_sensitive(pb,FALSE);
 
-  gtk_widget_show(h);
-  gtk_widget_show(pb);
-  gtk_widget_show(rb);
-
+  Gtk::show(h,pb,rb,0);
   gtk_signal_connect(GTK_OBJECT(clist),"select_row",
 		     GTK_SIGNAL_FUNC(skg_select),(gpointer)this);
 
@@ -248,6 +248,21 @@ void SeekGraph::clear() {
   contentUpdated();
 }
 
+void SeekGraph::updateFont() {
+  GdkFont *fxd;
+  GtkStyle *oStyle, *nStyle;
+  fxd=gdk_font_load(global.SeekFont);
+  if (!fxd) {
+    cerr << _("<SeekGraph::updatefont> failed to load ") << global.SeekFont << endl;
+    return;
+  }		
+  oStyle = gtk_widget_get_style(GTK_WIDGET(clist));
+  nStyle = gtk_style_copy(oStyle);
+  nStyle->font = fxd;
+  gtk_widget_set_style(GTK_WIDGET(clist), nStyle);
+  gtk_style_unref(oStyle);
+}
+
 // --- callbacks
 
 void
@@ -313,10 +328,10 @@ skg_play(GtkWidget *w, gpointer data)
   SeekGraph *me;
   char z[64];
   me=(SeekGraph *)data;
-  if (me->Selected >= 0) {
+  if (me->Selected >= 0 && global.protocol) {
     sprintf(z,"play %d",me->Selected);
-    global.network->writeLine(z);
-    sprintf(z,"Replied to seek #%d",me->Selected);
-    global.status->setText(z);
+    global.protocol->sendUserInput(z);
+    sprintf(z,_("Replied to seek #%d"),me->Selected);
+    global.status->setText(z,10);
   }
 }
