@@ -1,4 +1,4 @@
-/* $Id: sound.cc,v 1.12 2002/06/17 03:24:31 bergo Exp $ */
+/* $Id: sound.cc,v 1.19 2003/01/19 19:24:03 bergo Exp $ */
 
 /*
 
@@ -23,7 +23,7 @@
 
 */
 
-#include <iostream.h>
+#include <iostream>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -45,6 +45,8 @@
 #include "config.h"
 #endif
 
+#include "eboard.h"
+
 #define SOME_DRIVER 1
 
 #ifdef HAVE_SYS_SOUNDCARD_H
@@ -76,6 +78,7 @@ SoundEvent::SoundEvent() {
   Count=1;
   strcpy(Device,"/dev/dsp");
   ExtraData[0]=0;
+  enabled = true;
 }
 
 SoundEvent SoundEvent::operator=(SoundEvent &se) {
@@ -83,12 +86,14 @@ SoundEvent SoundEvent::operator=(SoundEvent &se) {
   Pitch=se.Pitch;
   Duration=se.Duration;
   Count=se.Count;
+  enabled = se.enabled;
   strcpy(Device,se.Device);
   strcpy(ExtraData,se.ExtraData);
   return(*this);
 }
 
 int SoundEvent::operator==(SoundEvent &se) {
+  if (enabled!=se.enabled) return 0;
   if (type!=se.type) return 0;
   if (Pitch!=se.Pitch) return 0;
   if (Duration!=se.Duration) return 0;
@@ -121,20 +126,29 @@ void SoundEvent::read(tstring &rcline) {
     p->copy(Device,63);
     Count=rcline.tokenvalue(sep);
     if (!Count) Count=1;
+    enabled = rcline.tokenbool(sep,true);
     break;
   case 1: // EXT_WAVE
     type=EXT_WAVE;
     Pitch=Duration=0;
     p=rcline.token(sep); p->copy(Device,63);
     p=rcline.token(sep); p->copy(ExtraData,255);
+    enabled = rcline.tokenbool(sep,true);
     break;
   case 2: // EXT_PROGRAM
     type=EXT_PROGRAM;
     Pitch=Duration=0;
     p=rcline.token(sep); p->copy(ExtraData,255);
+    enabled = rcline.tokenbool(sep,true);
+    break;
+  case 3: // PLAIN_BEEP
+    type=PLAIN_BEEP;
+    Pitch=Duration=0;
+    rcline.token(sep); /* beep string */
+    enabled = rcline.tokenbool(sep,true);
     break;
   default:
-    cerr << "[eboard] bad RC line\n";
+    cerr << _("[eboard] bad RC line\n");
   }
 }
 
@@ -142,13 +156,18 @@ ostream & operator<<(ostream &s,  SoundEvent e) {
   switch(e.type) {
   case INT_WAVE:
     s << "0," << e.Pitch << ',' << e.Duration << ',';
-    s << e.Device << ',' << e.Count;
+    s << e.Device << ',' << e.Count << ',' << (e.enabled?1:0);
     break;
   case EXT_WAVE:
+    if (e.Device[0] == 0) strcpy(e.Device,"/dev/dsp");
     s << "1," << e.Device << ',' << e.ExtraData;
+    s << ',' << (e.enabled?1:0);
     break;
   case EXT_PROGRAM:
-    s << "2," << e.ExtraData;
+    s << "2," << e.ExtraData << ',' << (e.enabled?1:0);
+    break;
+  case PLAIN_BEEP:
+    s << "3,beep" << ',' << (e.enabled?1:0);
     break;
   }
   return(s);
@@ -159,6 +178,10 @@ void SoundEvent::playHere() {
   case INT_WAVE:
     sine_beep(Device,Pitch,Duration);
     break;
+  case PLAIN_BEEP:
+    printf("%c",7);
+    fflush(stdout);
+    break;
   default:
     play();
   }
@@ -170,7 +193,7 @@ void SoundEvent::safePlay() {
 
 void SoundEvent::play() {
 
-  if (type==INT_WAVE) {
+  if (type==INT_WAVE || type==PLAIN_BEEP) {
     playHere();
     return;
   }
@@ -185,6 +208,9 @@ void SoundEvent::play() {
     case EXT_PROGRAM:
       execlp("/bin/sh","/bin/sh","-c",ExtraData,0);
       break;
+    default:
+      /* will never happen */
+      break;
     }
     _exit(0);
 
@@ -285,17 +311,20 @@ void SoundEvent::sine_beep(char *device,int pitch,int duration) {
 char *SoundEvent::getDescription() {
   switch(type) {
   case INT_WAVE:
-    sprintf(pvt,"%d beep%sto %s, %d Hz for %d msec",
-	    Count,(Count>1)?"s ":" ",Device,Pitch,Duration);
+    sprintf(pvt,_("%d %s to %s, %d Hz for %d msec"),
+	    Count,(Count>1)?_("beeps"):_("beep"),Device,Pitch,Duration);
     break;
   case EXT_WAVE:
-    sprintf(pvt,"play file %s",ExtraData);
+    sprintf(pvt,_("play file %s"),ExtraData);
     break;
   case EXT_PROGRAM:
-    sprintf(pvt,"run %s",ExtraData);
+    sprintf(pvt,_("run %s"),ExtraData);
+    break;
+  case PLAIN_BEEP:
+    sprintf(pvt,_("plain console beep"));
     break;
   default:
-    strcpy(pvt,"nothing");
+    strcpy(pvt,_("nothing"));
   }
   return pvt;
 }
@@ -306,13 +335,12 @@ void SoundEvent::edit(SoundEventChangeListener *listener) {
 
 // dialog
 
-SoundEventDialog::SoundEventDialog(SoundEvent *src, SoundEventChangeListener *listener) : ModalDialog("Sound Event") {
+SoundEventDialog::SoundEventDialog(SoundEvent *src, SoundEventChangeListener *listener) : ModalDialog(N_("Sound Event")) {
   GtkWidget *v,*tf,*rh,*mh[4],*ml[5],*hs,*bb,*ok,*cancel,*test,*brw;
   GSList *rg;
   int i,j;
-  char z[256];
   GtkObject *pitch,*dur,*cou;
-  GtkWidget *tl,*om,*cbh,*omm,*ommi;
+  GtkWidget *tl=0,*om=0,*cbh=0,*omm,*ommi;
 
   gtk_window_set_default_size(GTK_WINDOW(widget),480,340);
 
@@ -322,22 +350,24 @@ SoundEventDialog::SoundEventDialog(SoundEvent *src, SoundEventChangeListener *li
   v=gtk_vbox_new(FALSE,4);
   gtk_container_add(GTK_CONTAINER(widget),v);
 
-  tf=gtk_frame_new("Event Type");
+  tf=gtk_frame_new(_("Event Type"));
   gtk_frame_set_shadow_type(GTK_FRAME(tf),GTK_SHADOW_ETCHED_IN);
   gtk_box_pack_start(GTK_BOX(v),tf,FALSE,FALSE,4);
 
   rh=gtk_vbox_new(FALSE,4);
   gtk_container_add(GTK_CONTAINER(tf),rh);
 
-  rd[0]=gtk_radio_button_new_with_label( 0, "Beep (need Pitch, Duration, Count and Device)" );
+  rd[0]=gtk_radio_button_new_with_label( 0, _("Beep (need Pitch, Duration, Count and Device)") );
   rg=gtk_radio_button_group(GTK_RADIO_BUTTON(rd[0]));
-  rd[1]=gtk_radio_button_new_with_label(rg, "Play WAV (need Device and Filename, sox must be installed)" );
+  rd[1]=gtk_radio_button_new_with_label(rg, _("Play WAV (need Device and Filename, sox must be installed)") );
   rg=gtk_radio_button_group(GTK_RADIO_BUTTON(rd[1]));
-  rd[2]=gtk_radio_button_new_with_label(rg, "Run Program (need Filename)" );
+  rd[2]=gtk_radio_button_new_with_label(rg, _("Run Program (need Filename)") );
+  rg=gtk_radio_button_group(GTK_RADIO_BUTTON(rd[2]));
+  rd[3]=gtk_radio_button_new_with_label(rg, _("Console Beep") );
 
-  for(i=0;i<3;i++) {
+  for(i=0;i<4;i++) {
     gtk_box_pack_start(GTK_BOX(rh),rd[i],FALSE,FALSE,4);
-    gtk_widget_show(rd[i]);
+    gshow(rd[i]);
   }
 
   mh[0]=gtk_hbox_new(FALSE,4);
@@ -345,13 +375,13 @@ SoundEventDialog::SoundEventDialog(SoundEvent *src, SoundEventChangeListener *li
   mh[2]=gtk_hbox_new(FALSE,4);
   mh[3]=gtk_hbox_new(FALSE,4);
 
-  ml[0]=gtk_label_new("Pitch (Hz):");
-  ml[1]=gtk_label_new("Duration (msec):");
-  ml[2]=gtk_label_new("Device:");
-  ml[3]=gtk_label_new("File to play / Program to run:");
-  ml[4]=gtk_label_new("Count:");
+  ml[0]=gtk_label_new(_("Pitch (Hz):"));
+  ml[1]=gtk_label_new(_("Duration (msec):"));
+  ml[2]=gtk_label_new(_("Device:"));
+  ml[3]=gtk_label_new(_("File to play / Program to run:"));
+  ml[4]=gtk_label_new(_("Count:"));
 
-  brw=gtk_button_new_with_label(" Browse... ");
+  brw=gtk_button_new_with_label(_(" Browse... "));
 
   for(i=2;i<4;i++)
     en[i]=gtk_entry_new();
@@ -377,11 +407,11 @@ SoundEventDialog::SoundEventDialog(SoundEvent *src, SoundEventChangeListener *li
       gtk_signal_connect(GTK_OBJECT(ommi),"activate",
 			 GTK_SIGNAL_FUNC(snddlg_picktheme),(gpointer)this);
       gtk_menu_append(GTK_MENU(omm),ommi);
-      gtk_widget_show(ommi);
+      gshow(ommi);
       sthemes.push_back(ommi);
     }
     gtk_option_menu_set_menu(GTK_OPTION_MENU(om),omm);
-    tl=gtk_label_new("Configured Sound Files:");
+    tl=gtk_label_new(_("Configured Sound Files:"));
   }
 
   gtk_box_pack_start(GTK_BOX(v),mh[0],TRUE,TRUE,4);
@@ -410,21 +440,15 @@ SoundEventDialog::SoundEventDialog(SoundEvent *src, SoundEventChangeListener *li
     gtk_box_pack_start(GTK_BOX(v),cbh,FALSE,FALSE,4);
   }
 
-  for(i=0;i<4;i++) {
-    gtk_widget_show(ml[i]);
-    gtk_widget_show(en[i]);
-    gtk_widget_show(mh[i]);
-  }
-  gtk_widget_show(ml[4]);
-  gtk_widget_show(en[4]);
+  for(i=0;i<4;i++)
+    Gtk::show(ml[i],en[i],mh[i],0);
 
-  if (j) {
-    gtk_widget_show(cbh);
-    gtk_widget_show(tl);
-    gtk_widget_show(om);
-  }
+  Gtk::show(ml[4],en[4],0);
 
-  gtk_widget_show(brw);
+  if (j)
+    Gtk::show(cbh,tl,om,0);
+
+  gshow(brw);
 
   hs=gtk_hseparator_new();
   gtk_box_pack_start(GTK_BOX(v),hs,FALSE,FALSE,4);
@@ -434,38 +458,31 @@ SoundEventDialog::SoundEventDialog(SoundEvent *src, SoundEventChangeListener *li
   gtk_button_box_set_spacing(GTK_BUTTON_BOX(bb), 5);
   gtk_box_pack_start(GTK_BOX(v),bb,FALSE,FALSE,2);
 
-  ok=gtk_button_new_with_label("Ok");
+  ok=gtk_button_new_with_label(_("Ok"));
   GTK_WIDGET_SET_FLAGS(ok,GTK_CAN_DEFAULT);
-  test=gtk_button_new_with_label("Test");
+  test=gtk_button_new_with_label(_("Test"));
   GTK_WIDGET_SET_FLAGS(test,GTK_CAN_DEFAULT);
-  cancel=gtk_button_new_with_label("Cancel");
+  cancel=gtk_button_new_with_label(_("Cancel"));
   GTK_WIDGET_SET_FLAGS(cancel,GTK_CAN_DEFAULT);
   gtk_box_pack_start(GTK_BOX(bb),ok,TRUE,TRUE,0);
   gtk_box_pack_start(GTK_BOX(bb),test,TRUE,TRUE,0);
   gtk_box_pack_start(GTK_BOX(bb),cancel,TRUE,TRUE,0);
   gtk_widget_grab_default(ok);
 
-  gtk_widget_show(bb);
-  gtk_widget_show(ok);
-  gtk_widget_show(test);
-  gtk_widget_show(cancel);
-
-  gtk_widget_show(hs);
-  gtk_widget_show(rh);
-  gtk_widget_show(tf);
-  gtk_widget_show(v);
-
+  Gtk::show(bb,ok,test,cancel,hs,rh,tf,v,0);
   setDismiss(GTK_OBJECT(cancel),"clicked");
 
   switch(src->type) {
   case INT_WAVE:
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(rd[0]), 1);
+    gtset(GTK_TOGGLE_BUTTON(rd[0]), 1);
     break;
   case EXT_WAVE:
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(rd[1]), 1);
+    gtset(GTK_TOGGLE_BUTTON(rd[1]), 1);
     break;
   case EXT_PROGRAM:
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(rd[2]), 1);
+    gtset(GTK_TOGGLE_BUTTON(rd[2]), 1);
+  case PLAIN_BEEP:
+    gtset(GTK_TOGGLE_BUTTON(rd[3]), 1);
     break;
   }
 
@@ -499,7 +516,7 @@ void snddlg_picktheme(GtkMenuItem *w,gpointer data) {
 	if (eff.find(z,zz))
 	  strcpy(z,zz);
 	gtk_entry_set_text(GTK_ENTRY(me->en[3]),z);
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(me->rd[1]),TRUE);
+	gtset(GTK_TOGGLE_BUTTON(me->rd[1]),TRUE);
       }      
       return;
     }
@@ -510,7 +527,7 @@ void snddlg_browse(GtkWidget *w,gpointer data) {
   FileDialog *fd;
 
   me=(SoundEventDialog *)data;
-  fd=new FileDialog("Browse");
+  fd=new FileDialog(_("Browse"));
   
   if (fd->run()) {
     gtk_entry_set_text(GTK_ENTRY(me->en[3]), fd->FileName);
@@ -542,6 +559,8 @@ void SoundEventDialog::apply(SoundEvent *dest) {
     dest->type=EXT_WAVE;
   if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(rd[2])))
     dest->type=EXT_PROGRAM;
+  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(rd[3])))
+    dest->type=PLAIN_BEEP;
   dest->Pitch=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(en[0]));
   dest->Duration=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(en[1]));
   dest->Count=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(en[4]));
