@@ -1,10 +1,10 @@
-/* $Id: util.cc,v 1.22 2002/06/28 01:33:05 bergo Exp $ */
+/* $Id: util.cc,v 1.29 2003/07/05 02:48:19 bergo Exp $ */
 
 /*
 
     eboard - chess client
     http://eboard.sourceforge.net
-    Copyright (C) 2000-2002 Felipe Paulo Guazzi Bergo
+    Copyright (C) 2000-2003 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -25,10 +25,11 @@
 
 
 #include "util.h"
-#include <iostream.h>
+#include <iostream>
 #include <stdlib.h>
 #include <string.h>
 #include <stdarg.h>
+#include <unistd.h>
 #include <ctype.h>
 #include "global.h"
 #include "prefix.h"
@@ -38,6 +39,8 @@
 #include "config.h"
 #endif
 
+#include "eboard.h"
+
 FileFinder::FileFinder() {
 
 }
@@ -50,7 +53,7 @@ int FileFinder::getPathCount() {
   return(path.size());
 }
 
-string &FileFinder::getPath(int i) {
+string &FileFinder::getPath(unsigned int i) {
   static string dot(".");
   if ((i<0)||(i>=path.size()))
     return(dot);
@@ -142,11 +145,13 @@ EboardFileFinder::EboardFileFinder() {
 // ----------------------------------------  pattern matching
 
 PercASetPat      percA;
+PercUpperASetPat percUA;
 PercBSetPat      percB;
 PercNSetPat      percN;
 PercUpperNSetPat percUN;
 PercRSetPat      percR;
 PercSSetPat      percS;
+PercUpperSSetPat percUS;
 
 Pattern::Pattern() {
   eternal=false;
@@ -287,12 +292,25 @@ CharSetPat::CharSetPat() {
 
 // ----------------------------------- the percent-patterns
 
+/*
+   %N = [0-9]
+   %n = [0-9] U {+,-}
+   %s = [A-Za-z]
+   %S = [A-Za-z0-9]
+   %a = [A-Za-z] U {(,),*}
+   %A = [A-Za-z0-9] U {(,),*}
+   %b = {space} U {tab}
+   %r = [0-9A-Za-z] U "()[]{}.,;:!@#$%^&*_-+=\\|<>?/~"
+*/
+
 bool PercNSetPat::theset[256];
 bool PercBSetPat::theset[256];
 bool PercSSetPat::theset[256];
 bool PercASetPat::theset[256];
 bool PercRSetPat::theset[256];
 bool PercUpperNSetPat::theset[256];
+bool PercUpperASetPat::theset[256];
+bool PercUpperSSetPat::theset[256];
 
 PercNSetPat::PercNSetPat() {
   myset=theset;
@@ -307,6 +325,13 @@ PercSSetPat::PercSSetPat() {
   includeLetters();
 }
 
+PercUpperSSetPat::PercUpperSSetPat() {
+  myset=theset;
+  clearSet();
+  includeLetters();
+  includeNumbers();
+}
+
 PercASetPat::PercASetPat() {
   myset=theset;
   clearSet();
@@ -314,6 +339,14 @@ PercASetPat::PercASetPat() {
   addToSet("()*");
 }
 
+PercUpperASetPat::PercUpperASetPat() {
+  myset=theset;
+  clearSet();
+  includeLetters();
+  includeNumbers();
+  addToSet("()*");
+}
+
 PercBSetPat::PercBSetPat() {
   myset=theset;
   clearSet();
@@ -379,6 +412,15 @@ void PatternMatcher::append(Pattern *p) {
   pattern.push_back(p);
 }
 
+void PatternMatcher::reset() {
+  list<Pattern *>::iterator pi;
+  cleanUp();
+  for(pi=pattern.begin();pi!=pattern.end();pi++)
+    if (! (*pi)->eternal )
+      delete(*pi);
+  pattern.clear();
+}
+
 void PatternMatcher::bindData(list<char> *newdata) {
   bound=newdata;
 }
@@ -440,7 +482,7 @@ int PatternMatcher::recursiveMatch(list<Pattern *>::iterator & pat,
 				   list<char>::iterator & p1) {
   list<char>::iterator b,*z,ic1;
   list<Pattern *>::iterator pat2;
-  int i,j,t,s;
+  int t,s;
 
   // null pattern matches null string
   if ((pat==pattern.end())&&(p0==bound->end()))
@@ -501,11 +543,13 @@ void ExtPatternMatcher::set(char *hrp) {
 
   if (!percA.eternal) {
     percA.eternal=true;
+    percUA.eternal=true;
     percB.eternal=true;
     percN.eternal=true;
     percUN.eternal=true;
     percR.eternal=true;
     percS.eternal=true;
+    percUS.eternal=true;
   }
 
   tc=0; // token count
@@ -527,7 +571,7 @@ void ExtPatternMatcher::set(char *hrp) {
       break;
     case '%':
       if (i==(L-1)) {
-	cerr << "<PatternMatcher::set> ** bad pattern string: " << hrp << endl;
+	cerr << _("<PatternMatcher::set> ** bad pattern string: ") << hrp << endl;
 	exit(67);
       }
       ++i;
@@ -550,6 +594,10 @@ void ExtPatternMatcher::set(char *hrp) {
 	  append(&percS);
 	  stokens.push_back(tc++);
 	  break;
+	case 'S':
+	  append(&percUS);
+	  stokens.push_back(tc++);
+	  break;
 	case 'n':
 	  append(&percN);
 	  ntokens.push_back(tc++);
@@ -562,6 +610,10 @@ void ExtPatternMatcher::set(char *hrp) {
 	  append(&percA);
 	  atokens.push_back(tc++);
 	  break;
+	case 'A':
+	  append(&percUA);
+	  atokens.push_back(tc++);
+	  break;
 	case 'b':
 	  append(&percB);
 	  btokens.push_back(tc++);
@@ -571,7 +623,7 @@ void ExtPatternMatcher::set(char *hrp) {
 	  rtokens.push_back(tc++);
 	  break;
 	default:
-	  cerr << "<PatternMatcher::set> ** bad pattern string: " << hrp << endl;
+	  cerr << _("<PatternMatcher::set> ** bad pattern string: ") << hrp << endl;
 	  exit(68);
 	}
       }
@@ -586,7 +638,7 @@ void ExtPatternMatcher::set(char *hrp) {
 }
 
 char *ExtPatternMatcher::getXToken(vector<int> &v, int index) {
-  if (index>=v.size())
+  if (index >= (signed int) (v.size()) )
     return 0;
   return(getToken(v[index]));  
 }
@@ -643,3 +695,127 @@ void PatternBinder::prepare(const char *target) {
   for(gi=group.begin();gi!=group.end();gi++)
     (*gi)->bindData(&data);
 }
+
+// ----------------------- zifstream
+
+zifstream::zifstream() {
+  compressed  = false;
+  isopen      = false;
+  failure     = 0;
+  tmpfile[0]  = 0;
+  origfile[0] = 0;
+}
+
+zifstream::zifstream(char *name) {
+  compressed  = false;
+  isopen      = false;
+  failure     = 0;
+  tmpfile[0]  = 0;
+  origfile[0] = 0;
+  open(name);
+}
+
+void zifstream::open(char *name) {
+  int n;
+
+  n = strlen(name);
+  if (n>255) {
+    failure = 1;
+    return;
+  }
+
+  strcpy(origfile,name);
+  
+  if (n>3)
+    if (!strcmp(&origfile[n-3],".gz")) {
+      compressed = true;
+      copen();
+      return;
+    }
+  
+  x.open(origfile,ios::in);
+
+  if (!x) failure = 1;
+  if (x.is_open()) isopen = true;
+}
+
+bool zifstream::getline(char *ptr, int len, char delim) {
+  if (x.getline(ptr,len,delim))
+    return true;
+  else
+    return false;
+}
+  
+bool zifstream::seekg(unsigned long offset) {
+  x.seekg(offset);
+  return((bool)(x.good()));
+}
+
+unsigned long zifstream::tellg() {
+  return( (unsigned long) x.tellg() );
+}
+  
+bool zifstream::operator!() {
+  return(x.fail()!=0);
+}
+
+bool zifstream::eof() {
+  return((bool)(x.eof()));
+}
+
+void zifstream::close() {
+  x.close();
+  if (isopen && compressed)
+    cclose();
+  isopen=false;
+}
+
+bool zifstream::fail() {
+  return(failure!=0 || x.fail()!=0);
+}
+
+bool zifstream::is_open() {
+  return(isopen);
+}
+
+void zifstream::copen() {
+  int i,n;
+  char cmd[1024];
+
+  sprintf(tmpfile,"/tmp/eb%d-%s",(int) getpid(), origfile);
+  n = strlen(tmpfile);
+
+  for(i=5;i<n;i++)
+    if (tmpfile[i] == '/') tmpfile[i] = '_';
+
+  strcpy(ungzfile, tmpfile);
+  ungzfile[n-3] = 0;
+
+  sprintf(cmd,"cp -f %s %s",origfile, tmpfile);
+  
+  if (system(cmd)!=0) {
+    failure = 1;
+    return;
+  }
+
+  sprintf(cmd,"gzip -d %s",tmpfile);
+
+  if (system(cmd)!=0) {
+    failure = 1;
+    unlink(tmpfile);
+    return;
+  }
+
+  x.open(ungzfile,ios::in);
+  if (!x) failure = 1;
+  if (x.is_open()) isopen = true;
+
+  if (!isopen) {
+    unlink(tmpfile);
+    unlink(ungzfile);
+  }
+}
+
+void zifstream::cclose() {
+  unlink(ungzfile);  
+}
